@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import click
 
 from chinilla.util.config import load_config
@@ -10,8 +12,12 @@ from chinilla.util.service_groups import all_groups
 @click.pass_context
 def start_cmd(ctx: click.Context, restart: bool, group: str) -> None:
     import asyncio
+
+    from chinilla.cmds.beta_funcs import warn_if_beta_enabled
+
     from .start_funcs import async_start
 
     root_path = ctx.obj["root_path"]
     config = load_config(root_path, "config.yaml")
-    asyncio.run(async_start(root_path, config, group, restart, ctx.obj["force_legacy_keyring_migration"]))
+    warn_if_beta_enabled(config)
+    asyncio.run(async_start(root_path, config, group, restart))
