@@ -1,10 +1,16 @@
+from __future__ import annotations
+
 import sys
-from typing import Any, Dict, Optional, Tuple
+from typing import Any, Dict, List, Optional, Tuple
 
 import click
 
+from chinilla.cmds.cmds_util import execute_with_wallet
+
+# from chinilla.cmds.coins import coins_cmd
 from chinilla.cmds.plotnft import validate_fee
 from chinilla.wallet.transaction_sorting import SortKey
+from chinilla.wallet.util.address_type import AddressType
 from chinilla.wallet.util.wallet_types import WalletType
 
 
@@ -29,8 +35,8 @@ def wallet_cmd(ctx: click.Context) -> None:
 def get_transaction_cmd(wallet_rpc_port: Optional[int], fingerprint: int, id: int, tx_id: str, verbose: int) -> None:
     extra_params = {"id": id, "tx_id": tx_id, "verbose": verbose}
     import asyncio
+
     from .wallet_funcs import get_transaction
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, get_transaction))
 
@@ -59,7 +65,7 @@ def get_transaction_cmd(wallet_rpc_port: Optional[int], fingerprint: int, id: in
     "--limit",
     help="Max number of transactions to return",
     type=int,
-    default=(2 ** 32 - 1),
+    default=(2**32 - 1),
     show_default=True,
     required=False,
 )
@@ -73,12 +79,14 @@ def get_transaction_cmd(wallet_rpc_port: Optional[int], fingerprint: int, id: in
     "--sort-by-height",
     "sort_key",
     flag_value=SortKey.CONFIRMED_AT_HEIGHT,
+    type=SortKey,
     help="Sort transactions by height",
 )
 @click.option(
     "--sort-by-relevance",
     "sort_key",
     flag_value=SortKey.RELEVANCE,
+    type=SortKey,
     default=True,
     help="Sort transactions by {confirmed, height, time}",
 )
@@ -110,8 +118,8 @@ def get_transactions_cmd(
     }
 
     import asyncio
+
     from .wallet_funcs import get_transactions
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, get_transactions))
 
@@ -151,12 +159,26 @@ def get_transactions_cmd(
 )
 @click.option(
     "-ma",
-    "--min_coin_amount",
+    "--min-coin-amount",
     help="Ignore coins worth less then this much HCX or CAT units",
     type=str,
     required=False,
     default="0",
 )
+@click.option(
+    "-l",
+    "--max-coin-amount",
+    help="Ignore coins worth more then this much HCX or CAT units",
+    type=str,
+    required=False,
+    default="0",
+)
+@click.option(
+    "--exclude-coin",
+    "coins_to_exclude",
+    multiple=True,
+    help="Exclude this coin from being spent.",
+)
 def send_cmd(
     wallet_rpc_port: Optional[int],
     fingerprint: int,
@@ -167,6 +189,8 @@ def send_cmd(
     address: str,
     override: bool,
     min_coin_amount: str,
+    max_coin_amount: str,
+    coins_to_exclude: Tuple[str],
 ) -> None:
     extra_params = {
         "id": id,
@@ -176,10 +200,12 @@ def send_cmd(
         "address": address,
         "override": override,
         "min_coin_amount": min_coin_amount,
+        "max_coin_amount": max_coin_amount,
+        "exclude_coin_ids": list(coins_to_exclude),
     }
     import asyncio
+
     from .wallet_funcs import send
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, send))
 
@@ -202,8 +228,8 @@ def send_cmd(
 )
 def show_cmd(wallet_rpc_port: Optional[int], fingerprint: int, wallet_type: Optional[str]) -> None:
     import asyncio
+
     from .wallet_funcs import print_balances
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     args: Dict[str, Any] = {}
     if wallet_type is not None:
@@ -234,8 +260,8 @@ def show_cmd(wallet_rpc_port: Optional[int], fingerprint: int, wallet_type: Opti
 def get_address_cmd(wallet_rpc_port: Optional[int], id, fingerprint: int, new_address: bool) -> None:
     extra_params = {"id": id, "new_address": new_address}
     import asyncio
+
     from .wallet_funcs import get_address
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, get_address))
 
@@ -255,8 +281,8 @@ def get_address_cmd(wallet_rpc_port: Optional[int], id, fingerprint: int, new_ad
 def delete_unconfirmed_transactions_cmd(wallet_rpc_port: Optional[int], id, fingerprint: int) -> None:
     extra_params = {"id": id}
     import asyncio
+
     from .wallet_funcs import delete_unconfirmed_transactions
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, delete_unconfirmed_transactions))
 
@@ -273,12 +299,32 @@ def delete_unconfirmed_transactions_cmd(wallet_rpc_port: Optional[int], id, fing
 def get_derivation_index_cmd(wallet_rpc_port: Optional[int], fingerprint: int) -> None:
     extra_params: Dict[str, Any] = {}
     import asyncio
+
     from .wallet_funcs import get_derivation_index
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, get_derivation_index))
 
 
+@wallet_cmd.command("sign_message", short_help="Sign a message by a derivation address")
+@click.option(
+    "-wp",
+    "--wallet-rpc-port",
+    help="Set the port where the Wallet is hosting the RPC interface. See the rpc_port under wallet in config.yaml",
+    type=int,
+    default=None,
+)
+@click.option("-f", "--fingerprint", help="Set the fingerprint to specify which wallet to use", type=int)
+@click.option("-a", "--address", help="The address you want to use for signing", type=str, required=True)
+@click.option("-m", "--hex_message", help="The hex message you want sign", type=str, required=True)
+def address_sign_message(wallet_rpc_port: Optional[int], fingerprint: int, address: str, hex_message: str) -> None:
+    extra_params: Dict[str, Any] = {"address": address, "message": hex_message, "type": AddressType.HCX}
+    import asyncio
+
+    from .wallet_funcs import sign_message
+
+    asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, sign_message))
+
+
 @wallet_cmd.command(
     "update_derivation_index", short_help="Generate additional derived puzzle hashes starting at the provided index"
 )
@@ -296,8 +342,8 @@ def get_derivation_index_cmd(wallet_rpc_port: Optional[int], fingerprint: int) -
 def update_derivation_index_cmd(wallet_rpc_port: Optional[int], fingerprint: int, index: int) -> None:
     extra_params = {"index": index}
     import asyncio
+
     from .wallet_funcs import update_derivation_index
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, update_derivation_index))
 
@@ -331,8 +377,8 @@ def update_derivation_index_cmd(wallet_rpc_port: Optional[int], fingerprint: int
 def add_token_cmd(wallet_rpc_port: Optional[int], asset_id: str, token_name: str, fingerprint: int) -> None:
     extra_params = {"asset_id": asset_id, "token_name": token_name}
     import asyncio
+
     from .wallet_funcs import add_token
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, add_token))
 
@@ -367,8 +413,8 @@ def make_offer_cmd(
 ) -> None:
     extra_params = {"offers": offer, "requests": request, "filepath": filepath, "fee": fee}
     import asyncio
+
     from .wallet_funcs import make_offer
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, make_offer))
 
@@ -414,8 +460,8 @@ def get_offers_cmd(
         "reverse": reverse,
     }
     import asyncio
+
     from .wallet_funcs import get_offers
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, get_offers))
 
@@ -437,8 +483,8 @@ def take_offer_cmd(
 ) -> None:
     extra_params = {"file": path_or_hex, "examine_only": examine_only, "fee": fee}
     import asyncio
+
     from .wallet_funcs import take_offer
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, take_offer))
 
@@ -458,8 +504,8 @@ def take_offer_cmd(
 def cancel_offer_cmd(wallet_rpc_port: Optional[int], fingerprint: int, id: str, insecure: bool, fee: str) -> None:
     extra_params = {"id": id, "insecure": insecure, "fee": fee}
     import asyncio
+
     from .wallet_funcs import cancel_offer
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, cancel_offer))
 
@@ -500,13 +546,33 @@ def did_create_wallet_cmd(
     wallet_rpc_port: Optional[int], fingerprint: int, name: Optional[str], amount: Optional[int], fee: Optional[int]
 ) -> None:
     import asyncio
+
     from .wallet_funcs import create_did_wallet
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {"amount": amount, "fee": fee, "name": name}
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, create_did_wallet))
 
 
+@did_cmd.command("sign_message", short_help="Sign a message by a DID")
+@click.option(
+    "-wp",
+    "--wallet-rpc-port",
+    help="Set the port where the Wallet is hosting the RPC interface. See the rpc_port under wallet in config.yaml",
+    type=int,
+    default=None,
+)
+@click.option("-f", "--fingerprint", help="Set the fingerprint to specify which wallet to use", type=int)
+@click.option("-i", "--did_id", help="DID ID you want to use for signing", type=str, required=True)
+@click.option("-m", "--hex_message", help="The hex message you want to sign", type=str, required=True)
+def did_sign_message(wallet_rpc_port: Optional[int], fingerprint: int, did_id: str, hex_message: str) -> None:
+    extra_params: Dict[str, Any] = {"did_id": did_id, "message": hex_message, "type": AddressType.DID}
+    import asyncio
+
+    from .wallet_funcs import sign_message
+
+    asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, sign_message))
+
+
 @did_cmd.command("set_name", short_help="Set DID wallet name")
 @click.option(
     "-wp",
@@ -520,8 +586,8 @@ def did_create_wallet_cmd(
 @click.option("-n", "--name", help="Set the DID wallet name", type=str, required=True)
 def did_wallet_name_cmd(wallet_rpc_port: Optional[int], fingerprint: int, id: int, name: str) -> None:
     import asyncio
+
     from .wallet_funcs import did_set_wallet_name
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {"wallet_id": id, "name": name}
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, did_set_wallet_name))
@@ -539,8 +605,8 @@ def did_wallet_name_cmd(wallet_rpc_port: Optional[int], fingerprint: int, id: in
 @click.option("-i", "--id", help="Id of the wallet to use", type=int, required=True)
 def did_get_did_cmd(wallet_rpc_port: Optional[int], fingerprint: int, id: int) -> None:
     import asyncio
+
     from .wallet_funcs import get_did
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {"did_wallet_id": id}
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, get_did))
@@ -566,13 +632,33 @@ def nft_wallet_create_cmd(
     wallet_rpc_port: Optional[int], fingerprint: int, did_id: Optional[str], name: Optional[str]
 ) -> None:
     import asyncio
+
     from .wallet_funcs import create_nft_wallet
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params: Dict[str, Any] = {"did_id": did_id, "name": name}
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, create_nft_wallet))
 
 
+@nft_cmd.command("sign_message", short_help="Sign a message by a NFT")
+@click.option(
+    "-wp",
+    "--wallet-rpc-port",
+    help="Set the port where the Wallet is hosting the RPC interface. See the rpc_port under wallet in config.yaml",
+    type=int,
+    default=None,
+)
+@click.option("-f", "--fingerprint", help="Set the fingerprint to specify which wallet to use", type=int)
+@click.option("-i", "--nft_id", help="NFT ID you want to use for signing", type=str, required=True)
+@click.option("-m", "--hex_message", help="The hex message you want to sign", type=str, required=True)
+def nft_sign_message(wallet_rpc_port: Optional[int], fingerprint: int, nft_id: str, hex_message: str) -> None:
+    extra_params: Dict[str, Any] = {"nft_id": nft_id, "message": hex_message, "type": AddressType.NFT}
+    import asyncio
+
+    from .wallet_funcs import sign_message
+
+    asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, sign_message))
+
+
 @nft_cmd.command("mint", short_help="Mint an NFT")
 @click.option(
     "-wp",
@@ -630,8 +716,8 @@ def nft_mint_cmd(
     royalty_percentage_fraction: int,
 ) -> None:
     import asyncio
+
     from .wallet_funcs import mint_nft
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     if metadata_uris is None:
         metadata_uris_list = []
@@ -696,8 +782,8 @@ def nft_add_uri_cmd(
     fee: str,
 ) -> None:
     import asyncio
+
     from .wallet_funcs import add_uri_to_nft
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {
         "wallet_id": id,
@@ -740,8 +826,8 @@ def nft_transfer_cmd(
     fee: str,
 ) -> None:
     import asyncio
+
     from .wallet_funcs import transfer_nft
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {
         "wallet_id": id,
@@ -764,8 +850,8 @@ def nft_transfer_cmd(
 @click.option("-i", "--id", help="Id of the NFT wallet to use", type=int, required=True)
 def nft_list_cmd(wallet_rpc_port: Optional[int], fingerprint: int, id: int) -> None:
     import asyncio
+
     from .wallet_funcs import list_nfts
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {"wallet_id": id}
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, list_nfts))
@@ -801,8 +887,8 @@ def nft_set_did_cmd(
     fee: str,
 ) -> None:
     import asyncio
+
     from .wallet_funcs import set_nft_did
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {
         "wallet_id": id,
@@ -829,10 +915,127 @@ def nft_get_info_cmd(
     nft_coin_id: str,
 ) -> None:
     import asyncio
+
     from .wallet_funcs import get_nft_info
-    from chinilla.cmds.cmds_util import execute_with_wallet
 
     extra_params = {
         "nft_coin_id": nft_coin_id,
     }
     asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, get_nft_info))
+
+
+# Keep at bottom.
+# Remove coins subcommand from wallet for now pending further testing
+# wallet_cmd.add_command(coins_cmd)
+
+
+@wallet_cmd.group("notifications", short_help="Send/Manage notifications")
+def notification_cmd():
+    pass
+
+
+@notification_cmd.command("send", short_help="Send a notification to the owner of an address")
+@click.option(
+    "-wp",
+    "--wallet-rpc-port",
+    help="Set the port where the Wallet is hosting the RPC interface. See the rpc_port under wallet in config.yaml",
+    type=int,
+    default=None,
+)
+@click.option("-f", "--fingerprint", help="Set the fingerprint to specify which wallet to use", type=int)
+@click.option("-t", "--to-address", help="The address to send the notification to", type=str, required=True)
+@click.option(
+    "-a",
+    "--amount",
+    help="The amount to send to get the notification past the recipient's spam filter",
+    type=str,
+    default="0.00001",
+    required=True,
+    show_default=True,
+)
+@click.option("-n", "--message", help="The message of the notification", type=str)
+@click.option("-m", "--fee", help="The fee for the transaction", type=str)
+def _send_notification(
+    wallet_rpc_port: Optional[int],
+    fingerprint: int,
+    to_address: str,
+    amount: str,
+    message: str,
+    fee: str,
+) -> None:
+    import asyncio
+
+    from chinilla.cmds.cmds_util import execute_with_wallet
+
+    from .wallet_funcs import send_notification
+
+    extra_params = {
+        "address": to_address,
+        "amount": amount,
+        "message": message,
+        "fee": fee,
+    }
+    asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, send_notification))
+
+
+@notification_cmd.command("get", short_help="Get notification(s) that are in your wallet")
+@click.option(
+    "-wp",
+    "--wallet-rpc-port",
+    help="Set the port where the Wallet is hosting the RPC interface. See the rpc_port under wallet in config.yaml",
+    type=int,
+    default=None,
+)
+@click.option("-f", "--fingerprint", help="Set the fingerprint to specify which wallet to use", type=int)
+@click.option("-i", "--id", help="The specific notification ID to show", type=str, default=[], multiple=True)
+@click.option("-s", "--start", help="The number of notifications to skip", type=int, default=None)
+@click.option("-e", "--end", help="The number of notifications to stop at", type=int, default=None)
+def _get_notifications(
+    wallet_rpc_port: Optional[int],
+    fingerprint: int,
+    id: List[str],
+    start: Optional[int],
+    end: Optional[int],
+) -> None:
+    import asyncio
+
+    from chinilla.cmds.cmds_util import execute_with_wallet
+
+    from .wallet_funcs import get_notifications
+
+    extra_params = {
+        "ids": id,
+        "start": start,
+        "end": end,
+    }
+    asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, get_notifications))
+
+
+@notification_cmd.command("delete", short_help="Delete notification(s) that are in your wallet")
+@click.option(
+    "-wp",
+    "--wallet-rpc-port",
+    help="Set the port where the Wallet is hosting the RPC interface. See the rpc_port under wallet in config.yaml",
+    type=int,
+    default=None,
+)
+@click.option("-f", "--fingerprint", help="Set the fingerprint to specify which wallet to use", type=int)
+@click.option("-i", "--id", help="A specific notification ID to delete", type=str, multiple=True)
+@click.option("--all", help="All notifications can be deleted (they will be recovered during resync)", is_flag=True)
+def _delete_notifications(
+    wallet_rpc_port: Optional[int],
+    fingerprint: int,
+    id: List[str],
+    all: bool,
+) -> None:
+    import asyncio
+
+    from chinilla.cmds.cmds_util import execute_with_wallet
+
+    from .wallet_funcs import delete_notifications
+
+    extra_params = {
+        "ids": id,
+        "all": all,
+    }
+    asyncio.run(execute_with_wallet(wallet_rpc_port, fingerprint, extra_params, delete_notifications))
