@@ -1,4 +1,7 @@
+from __future__ import annotations
+
 import asyncio
+import os
 import pathlib
 import sys
 import time
@@ -7,7 +10,7 @@ from decimal import Decimal
 from typing import Any, Awaitable, Callable, Dict, List, Optional, Tuple, Union
 
 from chinilla.cmds.cmds_util import transaction_status_msg, transaction_submitted_msg
-from chinilla.cmds.show import print_connections
+from chinilla.cmds.peer_funcs import print_connections
 from chinilla.cmds.units import units
 from chinilla.rpc.wallet_rpc_client import WalletRpcClient
 from chinilla.server.start_wallet import SERVICE_NAME
@@ -15,8 +18,10 @@ from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.util.bech32m import bech32_decode, decode_puzzle_hash, encode_puzzle_hash
 from chinilla.util.config import load_config, selected_network_address_prefix
 from chinilla.util.default_root import DEFAULT_ROOT_PATH
-from chinilla.util.ints import uint32, uint64
+from chinilla.util.ints import uint16, uint32, uint64
 from chinilla.wallet.nft_wallet.nft_info import NFTInfo
+from chinilla.wallet.outer_puzzles import AssetType
+from chinilla.wallet.puzzle_drivers import PuzzleInfo
 from chinilla.wallet.trade_record import TradeRecord
 from chinilla.wallet.trading.offer import Offer
 from chinilla.wallet.trading.trade_status import TradeStatus
@@ -189,6 +194,8 @@ async def send(args: dict, wallet_client: WalletRpcClient, fingerprint: int) ->
     address = args["address"]
     override = args["override"]
     min_coin_amount = Decimal(args["min_coin_amount"])
+    max_coin_amount = Decimal(args["max_coin_amount"])
+    exclude_coin_ids: List[str] = args["exclude_coin_ids"]
     memo = args["memo"]
     if memo is None:
         memos = None
@@ -207,26 +214,38 @@ async def send(args: dict, wallet_client: WalletRpcClient, fingerprint: int) ->
 
     try:
         typ = await get_wallet_type(wallet_id=wallet_id, wallet_client=wallet_client)
+        vojo_per_unit = get_vojo_per_unit(typ)
     except LookupError:
         print(f"Wallet id: {wallet_id} not found.")
         return
 
-    final_fee = uint64(int(fee * units["chinilla"]))
-    final_amount: uint64
-    final_min_coin_amount: uint64
+    final_fee: uint64 = uint64(int(fee * units["chinilla"]))  # fees are always in HCX vojos
+    final_amount: uint64 = uint64(int(amount * vojo_per_unit))
+    final_min_coin_amount: uint64 = uint64(int(min_coin_amount * vojo_per_unit))
+    final_max_coin_amount: uint64 = uint64(int(max_coin_amount * vojo_per_unit))
     if typ == WalletType.STANDARD_WALLET:
-        final_amount = uint64(int(amount * units["chinilla"]))
-        final_min_coin_amount = uint64(int(min_coin_amount * units["chinilla"]))
         print("Submitting transaction...")
         res = await wallet_client.send_transaction(
-            str(wallet_id), final_amount, address, final_fee, memos, final_min_coin_amount
+            str(wallet_id),
+            final_amount,
+            address,
+            final_fee,
+            memos,
+            final_min_coin_amount,
+            final_max_coin_amount,
+            exclude_coin_ids=exclude_coin_ids,
         )
     elif typ == WalletType.CAT:
-        final_amount = uint64(int(amount * units["cat"]))
-        final_min_coin_amount = uint64(int(min_coin_amount * units["cat"]))
         print("Submitting transaction...")
         res = await wallet_client.cat_spend(
-            str(wallet_id), final_amount, address, final_fee, memos, final_min_coin_amount
+            str(wallet_id),
+            final_amount,
+            address,
+            final_fee,
+            memos,
+            final_min_coin_amount,
+            final_max_coin_amount,
+            exclude_coin_ids=exclude_coin_ids,
         )
     else:
         print("Only standard wallet and CAT wallets are supported")
@@ -291,7 +310,7 @@ async def add_token(args: dict, wallet_client: WalletRpcClient, fingerprint: int
         if "fromhex()" in str(e):
             print(f"{asset_id} is not a valid Asset ID")
         else:
-            raise e
+            raise
 
 
 async def make_offer(args: dict, wallet_client: WalletRpcClient, fingerprint: int) -> None:
@@ -299,6 +318,7 @@ async def make_offer(args: dict, wallet_client: WalletRpcClient, fingerprint: in
     requests: List[str] = args["requests"]
     filepath: str = args["filepath"]
     fee: int = int(Decimal(args["fee"]) * units["chinilla"])
+    config = load_config(DEFAULT_ROOT_PATH, "config.yaml")
 
     if [] in [offers, requests]:
         print("Not creating offer: Must be offering and requesting at least one asset")
@@ -306,12 +326,21 @@ async def make_offer(args: dict, wallet_client: WalletRpcClient, fingerprint: in
         offer_dict: Dict[Union[uint32, str], int] = {}
         driver_dict: Dict[str, Any] = {}
         printable_dict: Dict[str, Tuple[str, int, int]] = {}  # Dict[asset_name, Tuple[amount, unit, multiplier]]
-        nft_warning: bool = False
+        royalty_asset_dict: Dict[Any, Tuple[Any, uint16]] = {}
+        fungible_asset_dict: Dict[Any, uint64] = {}
         for item in [*offers, *requests]:
             name, amount = tuple(item.split(":")[0:2])
             try:
-                id: Union[uint32, str] = bytes32.from_hexstr(name).hex()
-                unit = 1
+                b32_id = bytes32.from_hexstr(name)
+                id: Union[uint32, str] = b32_id.hex()
+                result = await wallet_client.cat_asset_id_to_name(b32_id)
+                if result is not None:
+                    name = result[1]
+                else:
+                    name = "Unknown CAT"
+                unit = units["cat"]
+                if item in offers:
+                    fungible_asset_dict[name] = uint64(abs(int(Decimal(amount) * unit)))
             except ValueError:
                 try:
                     hrp, _ = bech32_decode(name)
@@ -319,7 +348,6 @@ async def make_offer(args: dict, wallet_client: WalletRpcClient, fingerprint: in
                         coin_id = decode_puzzle_hash(name)
                         unit = 1
                         info = NFTInfo.from_json_dict((await wallet_client.get_nft_info(coin_id.hex()))["nft_info"])
-                        nft_warning = True
                         id = info.launcher_id.hex()
                         assert isinstance(id, str)
                         if item in requests:
@@ -335,6 +363,7 @@ async def make_offer(args: dict, wallet_client: WalletRpcClient, fingerprint: in
                             }
                             if info.supports_did:
                                 assert info.royalty_puzzle_hash is not None
+                                assert info.royalty_percentage is not None
                                 driver_dict[id]["also"]["also"] = {
                                     "type": "ownership",
                                     "owner": "()",
@@ -345,6 +374,10 @@ async def make_offer(args: dict, wallet_client: WalletRpcClient, fingerprint: in
                                         "royalty_percentage": str(info.royalty_percentage),
                                     },
                                 }
+                                royalty_asset_dict[name] = (
+                                    encode_puzzle_hash(info.royalty_puzzle_hash, AddressType.HCX.hrp(config)),
+                                    info.royalty_percentage,
+                                )
                     else:
                         id = decode_puzzle_hash(name).hex()
                         assert hrp is not None
@@ -357,6 +390,8 @@ async def make_offer(args: dict, wallet_client: WalletRpcClient, fingerprint: in
                     else:
                         name = await wallet_client.get_cat_name(str(id))
                         unit = units["cat"]
+                    if item in offers:
+                        fungible_asset_dict[name] = uint64(abs(int(Decimal(amount) * unit)))
             multiplier: int = -1 if item in offers else 1
             printable_dict[name] = (amount, unit, multiplier)
             if id in offer_dict:
@@ -379,9 +414,34 @@ async def make_offer(args: dict, wallet_client: WalletRpcClient, fingerprint: in
                 if multiplier > 0:
                     print(f"  - {amount} {name} ({int(Decimal(amount) * unit)} vojos)")
 
-            if nft_warning:
+            if royalty_asset_dict != {}:
+                royalty_summary: Dict[Any, List[Dict[str, Any]]] = await wallet_client.nft_calculate_royalties(
+                    royalty_asset_dict, fungible_asset_dict
+                )
+                total_amounts_requested: Dict[Any, int] = {}
+                print()
+                print("Royalties Summary:")
+                for nft_id, summaries in royalty_summary.items():
+                    print(f"  - For {nft_id}:")
+                    for summary in summaries:
+                        divisor = units["chinilla"] if summary["asset"] == "HCX" else units["cat"]
+                        converted_amount = Decimal(summary["amount"]) / divisor
+                        total_amounts_requested.setdefault(summary["asset"], fungible_asset_dict[summary["asset"]])
+                        total_amounts_requested[summary["asset"]] += summary["amount"]
+                        print(
+                            f"    - {converted_amount} {summary['asset']} ({summary['amount']} vojos) to {summary['address']}"  # noqa
+                        )
+
+                print()
+                print("Total Amounts Offered:")
+                for asset, requested_amount in total_amounts_requested.items():
+                    divisor = units["chinilla"] if asset == "HCX" else units["cat"]
+                    converted_amount = Decimal(requested_amount) / divisor
+                    print(f"  - {converted_amount} {asset} ({requested_amount} vojos)")
+
+                print()
                 nft_confirmation = input(
-                    "Offers for NFTs will have royalties automatically added.  "
+                    "Offers for NFTs will have royalties automatically added. "
                     + "Are you sure you would like to continue? (y/n): "
                 )
                 if nft_confirmation not in ["y", "yes"]:
@@ -514,7 +574,7 @@ async def get_offers(args: dict, wallet_client: WalletRpcClient, fingerprint: in
 
 
 async def take_offer(args: dict, wallet_client: WalletRpcClient, fingerprint: int) -> None:
-    if "." in args["file"]:
+    if os.path.exists(args["file"]):
         filepath = pathlib.Path(args["file"])
         with open(filepath, "r") as file:
             offer_hex: str = file.read()
@@ -524,6 +584,7 @@ async def take_offer(args: dict, wallet_client: WalletRpcClient, fingerprint: in
 
     examine_only: bool = args["examine_only"]
     fee: int = int(Decimal(args["fee"]) * units["chinilla"])
+    config = load_config(DEFAULT_ROOT_PATH, "config.yaml")
 
     try:
         offer = Offer.from_bech32(offer_hex)
@@ -556,7 +617,7 @@ async def take_offer(args: dict, wallet_client: WalletRpcClient, fingerprint: in
             raise ValueError("CAT1s are no longer supported")
     ###
 
-    offered, requested, driver_dict = offer.summary()
+    offered, requested, _ = offer.summary()
     cat_name_resolver = wallet_client.cat_asset_id_to_name
     print("Summary:")
     print("  OFFERED:")
@@ -566,38 +627,55 @@ async def take_offer(args: dict, wallet_client: WalletRpcClient, fingerprint: in
 
     print()
 
-    nft_coin_id: Optional[bytes32] = nft_coin_id_supporting_royalties_from_offer(driver_dict)
-    nft_royalty_percentage: int = (
-        0 if nft_coin_id is None else await get_nft_royalty_percentage(nft_coin_id, wallet_client)
-    )
-    nft_total_amount_requested_str: Optional[str] = None
-    if nft_coin_id is not None and nft_royalty_percentage > 0:
-        print("NFT Royalty Fee:")
-        nft_royalty_asset_id, nft_royalty_amount, nft_total_amount_requested = calculate_nft_royalty_amount(
-            offered, requested, nft_coin_id, nft_royalty_percentage
-        )
-        nft_royalty_currency: str = "Unknown CAT"
-        if nft_royalty_asset_id == "hcx":
-            nft_royalty_currency = "HCX"
-        else:
-            result = await cat_name_resolver(bytes32.fromhex(nft_royalty_asset_id))
-            if result is not None:
-                nft_royalty_currency = result[1]
+    royalty_asset_dict: Dict[Any, Tuple[Any, uint16]] = {}
+    for royalty_asset_id in nft_coin_ids_supporting_royalties_from_offer(offer):
+        if royalty_asset_id.hex() in offered:
+            percentage, address = await get_nft_royalty_percentage_and_address(royalty_asset_id, wallet_client)
+            royalty_asset_dict[encode_puzzle_hash(royalty_asset_id, AddressType.NFT.hrp(config))] = (
+                encode_puzzle_hash(address, AddressType.HCX.hrp(config)),
+                percentage,
+            )
 
-        nft_royalty_divisor = units["chinilla"] if nft_royalty_asset_id == "hcx" else units["cat"]
-        nft_total_amount_requested_str = (
-            f"{Decimal(nft_total_amount_requested) / nft_royalty_divisor} {nft_royalty_currency}"
-        )
-        print(
-            f"      {Decimal(nft_royalty_amount) / nft_royalty_divisor} {nft_royalty_currency} "
-            f"({nft_royalty_amount} vojos)"
-        )
+    if royalty_asset_dict != {}:
+        fungible_asset_dict: Dict[Any, uint64] = {}
+        for fungible_asset_id in fungible_assets_from_offer(offer):
+            fungible_asset_id_str = fungible_asset_id.hex() if fungible_asset_id is not None else "hcx"
+            if fungible_asset_id_str in requested:
+                nft_royalty_currency: str = "Unknown CAT"
+                if fungible_asset_id is None:
+                    nft_royalty_currency = "HCX"
+                else:
+                    result = await wallet_client.cat_asset_id_to_name(fungible_asset_id)
+                    if result is not None:
+                        nft_royalty_currency = result[1]
+                fungible_asset_dict[nft_royalty_currency] = uint64(requested[fungible_asset_id_str])
+
+        if fungible_asset_dict != {}:
+            royalty_summary: Dict[Any, List[Dict[str, Any]]] = await wallet_client.nft_calculate_royalties(
+                royalty_asset_dict, fungible_asset_dict
+            )
+            total_amounts_requested: Dict[Any, int] = {}
+            print("Royalties Summary:")
+            for nft_id, summaries in royalty_summary.items():
+                print(f"  - For {nft_id}:")
+                for summary in summaries:
+                    divisor = units["chinilla"] if summary["asset"] == "HCX" else units["cat"]
+                    converted_amount = Decimal(summary["amount"]) / divisor
+                    total_amounts_requested.setdefault(summary["asset"], fungible_asset_dict[summary["asset"]])
+                    total_amounts_requested[summary["asset"]] += summary["amount"]
+                    print(
+                        f"    - {converted_amount} {summary['asset']} ({summary['amount']} vojos) to {summary['address']}"  # noqa
+                    )
+
+            print()
+            print("Total Amounts Requested:")
+            for asset, amount in total_amounts_requested.items():
+                divisor = units["chinilla"] if asset == "HCX" else units["cat"]
+                converted_amount = Decimal(amount) / divisor
+                print(f"  - {converted_amount} {asset} ({amount} vojos)")
 
     print(f"Included Fees: {Decimal(offer.bundle.fees()) / units['chinilla']}")
 
-    if nft_total_amount_requested_str is not None:
-        print(f"Total Amount Requested: {nft_total_amount_requested_str}")
-
     if not examine_only:
         print()
         confirmation = input("Would you like to take this offer? (y/n): ")
@@ -626,13 +704,17 @@ async def cancel_offer(args: dict, wallet_client: WalletRpcClient, fingerprint:
 def wallet_coin_unit(typ: WalletType, address_prefix: str) -> Tuple[str, int]:
     if typ == WalletType.CAT:
         return "", units["cat"]
-    if typ in [WalletType.STANDARD_WALLET, WalletType.POOLING_WALLET, WalletType.MULTI_SIG, WalletType.RATE_LIMITED]:
+    if typ in [WalletType.STANDARD_WALLET, WalletType.POOLING_WALLET, WalletType.MULTI_SIG]:
         return address_prefix, units["chinilla"]
     return "", units["vojo"]
 
 
-def print_balance(amount: int, scale: int, address_prefix: str) -> str:
-    ret = f"{amount / scale} {address_prefix} "
+def print_balance(amount: int, scale: int, address_prefix: str, *, decimal_only: bool = False) -> str:
+    if decimal_only:  # dont use scientific notation.
+        final_amount = f"{amount / scale:.12f}"
+    else:
+        final_amount = f"{amount / scale}"
+    ret = f"{final_amount} {address_prefix} "
     if scale > 1:
         ret += f"({amount} vojo)"
     return ret
@@ -855,6 +937,7 @@ async def transfer_nft(args: Dict, wallet_client: WalletRpcClient, fingerprint:
 def print_nft_info(nft: NFTInfo, *, config: Dict[str, Any]) -> None:
     indent: str = "   "
     owner_did = None if nft.owner_did is None else encode_puzzle_hash(nft.owner_did, AddressType.DID.hrp(config))
+    minter_did = None if nft.minter_did is None else encode_puzzle_hash(nft.minter_did, AddressType.DID.hrp(config))
     print()
     print(f"{'NFT identifier:'.ljust(26)} {encode_puzzle_hash(nft.launcher_id, AddressType.NFT.hrp(config))}")
     print(f"{'Launcher coin ID:'.ljust(26)} {nft.launcher_id}")
@@ -862,6 +945,7 @@ def print_nft_info(nft: NFTInfo, *, config: Dict[str, Any]) -> None:
     print(f"{'Current NFT coin ID:'.ljust(26)} {nft.nft_coin_id}")
     print(f"{'On-chain data/info:'.ljust(26)} {nft.chain_info}")
     print(f"{'Owner DID:'.ljust(26)} {owner_did}")
+    print(f"{'Minter DID:'.ljust(26)} {minter_did}")
     print(f"{'Royalty percentage:'.ljust(26)} {nft.royalty_percentage}")
     print(f"{'Royalty puzhash:'.ljust(26)} {nft.royalty_puzzle_hash}")
     print(f"{'NFT content hash:'.ljust(26)} {nft.data_hash.hex()}")
@@ -927,9 +1011,13 @@ async def get_nft_info(args: Dict, wallet_client: WalletRpcClient, fingerprint:
         print(f"Failed to get NFT info: {e}")
 
 
-async def get_nft_royalty_percentage(nft_coin_id: bytes32, wallet_client: WalletRpcClient) -> int:
+async def get_nft_royalty_percentage_and_address(
+    nft_coin_id: bytes32, wallet_client: WalletRpcClient
+) -> Tuple[uint16, bytes32]:
     info = NFTInfo.from_json_dict((await wallet_client.get_nft_info(nft_coin_id.hex()))["nft_info"])
-    return info.royalty_percentage if info.royalty_percentage is not None else 0
+    assert info.royalty_puzzle_hash is not None
+    percentage = uint16(info.royalty_percentage) if info.royalty_percentage is not None else 0
+    return uint16(percentage), info.royalty_puzzle_hash
 
 
 def calculate_nft_royalty_amount(
@@ -948,17 +1036,83 @@ def calculate_nft_royalty_amount(
     return royalty_asset_id, royalty_amount, total_amount_requested
 
 
-def driver_dict_asset_is_nft_supporting_royalties(driver_dict: Dict[str, Any], asset_id: str) -> bool:
-    asset_dict: Dict[str, Any] = driver_dict[asset_id]
-    return (
-        asset_dict.get("type") == "singleton"
-        and asset_dict.get("also", {}).get("type") == "metadata"
-        and asset_dict.get("also", {}).get("also", {}).get("type") == "ownership"
+def driver_dict_asset_is_nft_supporting_royalties(driver_dict: Dict[bytes32, PuzzleInfo], asset_id: bytes32) -> bool:
+    asset_dict: PuzzleInfo = driver_dict[asset_id]
+    return asset_dict.check_type(
+        [
+            AssetType.SINGLETON.value,
+            AssetType.METADATA.value,
+            AssetType.OWNERSHIP.value,
+        ]
     )
 
 
-def nft_coin_id_supporting_royalties_from_offer(driver_dict: Dict[str, Any]) -> Optional[bytes32]:
-    nft_asset_id: Optional[str] = next(
-        (key for key in driver_dict.keys() if driver_dict_asset_is_nft_supporting_royalties(driver_dict, key)), None
+def driver_dict_asset_is_fungible(driver_dict: Dict[bytes32, PuzzleInfo], asset_id: bytes32) -> bool:
+    asset_dict: PuzzleInfo = driver_dict[asset_id]
+    return not asset_dict.check_type(
+        [
+            AssetType.SINGLETON.value,
+        ]
     )
-    return bytes32.fromhex(nft_asset_id) if nft_asset_id is not None else None
+
+
+def nft_coin_ids_supporting_royalties_from_offer(offer: Offer) -> List[bytes32]:
+    return [
+        key for key in offer.driver_dict.keys() if driver_dict_asset_is_nft_supporting_royalties(offer.driver_dict, key)
+    ]
+
+
+def fungible_assets_from_offer(offer: Offer) -> List[Optional[bytes32]]:
+    return [
+        asset for asset in offer.arbitrage() if asset is None or driver_dict_asset_is_fungible(offer.driver_dict, asset)
+    ]
+
+
+async def send_notification(args: Dict, wallet_client: WalletRpcClient, fingerprint: int) -> None:
+    address: bytes32 = decode_puzzle_hash(args["address"])
+    amount: uint64 = uint64(Decimal(args["amount"]) * units["chinilla"])
+    message: bytes = bytes(args["message"], "utf8")
+    fee: uint64 = uint64(Decimal(args["fee"]) * units["chinilla"])
+
+    tx = await wallet_client.send_notification(address, message, amount, fee)
+
+    print("Notification sent successfully.")
+    print(f"To get status, use command: chinilla wallet get_transaction -f {fingerprint} -tx 0x{tx.name}")
+
+
+async def get_notifications(args: Dict, wallet_client: WalletRpcClient, fingerprint: int) -> None:
+    ids: Optional[List[bytes32]] = [bytes32.from_hexstr(id) for id in args["ids"]]
+    if ids is not None and len(ids) == 0:
+        ids = None
+
+    notifications = await wallet_client.get_notifications(ids=ids, pagination=(args["start"], args["end"]))
+    for notification in notifications:
+        print("")
+        print(f"ID: {notification.coin_id.hex()}")
+        print(f"message: {notification.message.decode('utf-8')}")
+        print(f"amount: {notification.amount}")
+
+
+async def delete_notifications(args: Dict, wallet_client: WalletRpcClient, fingerprint: int) -> None:
+    ids: Optional[List[bytes32]] = [bytes32.from_hexstr(id) for id in args["ids"]]
+
+    if args["all"]:
+        print(f"Success: {await wallet_client.delete_notifications()}")
+    else:
+        print(f"Success: {await wallet_client.delete_notifications(ids=ids)}")
+
+
+async def sign_message(args: Dict, wallet_client: WalletRpcClient, fingerprint: int) -> None:
+    if args["type"] == AddressType.HCX:
+        pubkey, signature = await wallet_client.sign_message_by_address(args["address"], args["message"])
+    elif args["type"] == AddressType.DID:
+        pubkey, signature = await wallet_client.sign_message_by_id(args["did_id"], args["message"])
+    elif args["type"] == AddressType.NFT:
+        pubkey, signature = await wallet_client.sign_message_by_id(args["nft_id"], args["message"])
+    else:
+        print("Invalid wallet type.")
+        return
+    print("")
+    print(f'Message: {args["message"]}')
+    print(f"Public Key: {pubkey}")
+    print(f"Signature: {signature}")
