@@ -5,10 +5,10 @@ import random
 from dataclasses import replace
 from typing import Callable, Dict, List, Optional, Tuple
 
-import chinillablspy
-from chinillablspy import G1Element, G2Element
-from chinilla_rs import compute_merkle_set_root
-from chinillabip158 import PyBIP158
+import blspy
+from blspy import G1Element, G2Element
+from chia_rs import compute_merkle_set_root
+from chiabip158 import PyBIP158
 
 from chinilla.consensus.block_record import BlockRecord
 from chinilla.consensus.block_rewards import calculate_base_farmer_reward, calculate_pool_reward
@@ -375,7 +375,7 @@ def create_unfinished_block(
     rc_sp_signature: Optional[G2Element] = get_plot_signature(rc_sp_hash, proof_of_space.plot_public_key)
     assert cc_sp_signature is not None
     assert rc_sp_signature is not None
-    assert chinillablspy.AugSchemeMPL.verify(proof_of_space.plot_public_key, cc_sp_hash, cc_sp_signature)
+    assert blspy.AugSchemeMPL.verify(proof_of_space.plot_public_key, cc_sp_hash, cc_sp_signature)
 
     total_iters = uint128(sub_slot_start_total_iters + ip_iters + (sub_slot_iters if overflow else 0))
 
