@@ -5,7 +5,7 @@ import logging
 import time
 from typing import Optional, Tuple
 
-from chinillablspy import AugSchemeMPL
+from blspy import AugSchemeMPL
 
 from chinilla.consensus.block_record import BlockRecord
 from chinilla.consensus.blockchain_interface import BlockchainInterface
@@ -23,6 +23,7 @@ from chinilla.consensus.pot_iterations import (
 )
 from chinilla.consensus.vdf_info_computation import get_signage_point_vdf_info
 from chinilla.types.blockchain_format.classgroup import ClassgroupElement
+from chinilla.types.blockchain_format.proof_of_space import verify_and_get_quality_string
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.types.blockchain_format.slots import ChallengeChainSubSlot, RewardChainSubSlot, SubSlotProofs
 from chinilla.types.blockchain_format.vdf import VDFInfo, VDFProof
@@ -484,8 +485,8 @@ def validate_unfinished_header_block(
     else:
         cc_sp_hash = header_block.reward_chain_block.challenge_chain_sp_vdf.output.get_hash()
 
-    q_str: Optional[bytes32] = header_block.reward_chain_block.proof_of_space.verify_and_get_quality_string(
-        constants, challenge, cc_sp_hash
+    q_str: Optional[bytes32] = verify_and_get_quality_string(
+        header_block.reward_chain_block.proof_of_space, constants, challenge, cc_sp_hash
     )
     if q_str is None:
         return None, ValidationError(Err.INVALID_POSPACE)
