@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import asyncio
 import json
 import ssl
@@ -46,25 +48,28 @@ class DaemonProxy:
             raise
 
         async def listener_task() -> None:
-            await self.listener()
-            await self.close()
+            try:
+                await self.listener()
+            finally:
+                await self.close()
 
         asyncio.create_task(listener_task())
         await asyncio.sleep(1)
 
     async def listener(self) -> None:
-        if self.websocket is not None:
-            while True:
-                message = await self.websocket.receive()
-                if message.type == aiohttp.WSMsgType.TEXT:
-                    decoded: WsRpcMessage = json.loads(message.data)
-                    request_id = decoded["request_id"]
+        if self.websocket is None:
+            raise TypeError("Websocket is None in listener!")
+        while True:
+            message = await self.websocket.receive()
+            if message.type == aiohttp.WSMsgType.TEXT:
+                decoded: WsRpcMessage = json.loads(message.data)
+                request_id = decoded["request_id"]
 
-                    if request_id in self._request_dict:
-                        self.response_dict[request_id] = decoded
-                        self._request_dict[request_id].set()
-                else:
-                    return None
+                if request_id in self._request_dict:
+                    self.response_dict[request_id] = decoded
+                    self._request_dict[request_id].set()
+            else:
+                return None
 
     async def _get(self, request: WsRpcMessage) -> WsRpcMessage:
         request_id = request["request_id"]
@@ -73,23 +78,15 @@ class DaemonProxy:
         if self.websocket is None or self.websocket.closed:
             raise Exception("Websocket is not connected")
         asyncio.create_task(self.websocket.send_str(string))
-
-        async def timeout() -> None:
-            await asyncio.sleep(30)
-            if request_id in self._request_dict:
-                print("Error, timeout.")
-                self._request_dict[request_id].set()
-
-        asyncio.create_task(timeout())
-        await self._request_dict[request_id].wait()
-        if request_id in self.response_dict:
+        try:
+            await asyncio.wait_for(self._request_dict[request_id].wait(), timeout=30)
+            self._request_dict.pop(request_id)
             response: WsRpcMessage = self.response_dict[request_id]
             self.response_dict.pop(request_id)
-        else:
-            response = None  # type: ignore
-        self._request_dict.pop(request_id)
-
-        return response
+            return response
+        except asyncio.TimeoutError:
+            self._request_dict.pop(request_id)
+            raise Exception(f"No response from daemon for request_id: {request_id}")
 
     async def get_version(self) -> WsRpcMessage:
         data: Dict[str, Any] = {}
@@ -131,12 +128,6 @@ class DaemonProxy:
         response = await self._get(request)
         return response
 
-    async def notify_keyring_migration_completed(self, passphrase: Optional[str]) -> WsRpcMessage:
-        data: Dict[str, Any] = {"key": passphrase}
-        request: WsRpcMessage = self.format_request("notify_keyring_migration_completed", data)
-        response: WsRpcMessage = await self._get(request)
-        return response
-
     async def ping(self) -> WsRpcMessage:
         request = self.format_request("ping", {})
         response = await self._get(request)
