@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import asyncio
 import functools
 import json
@@ -16,7 +18,7 @@ from pathlib import Path
 from typing import Any, Dict, List, Optional, TextIO, Tuple
 
 from chinilla import __version__
-from chinilla.cmds.init_funcs import check_keys, chinilla_init, chinilla_full_version_str
+from chinilla.cmds.init_funcs import check_keys, chinilla_full_version_str, chinilla_init
 from chinilla.cmds.passphrase_funcs import default_passphrase, using_default_passphrase
 from chinilla.daemon.keychain_server import KeychainServer, keychain_commands
 from chinilla.daemon.windows_signal import kill
@@ -27,13 +29,9 @@ from chinilla.ssl.create_ssl import get_mozilla_ca_crt
 from chinilla.util.beta_metrics import BetaMetricsLogger
 from chinilla.util.chinilla_logging import initialize_service_logging
 from chinilla.util.config import load_config
-from chinilla.util.errors import KeychainRequiresMigration, KeychainCurrentPassphraseIsInvalid
+from chinilla.util.errors import KeychainCurrentPassphraseIsInvalid
 from chinilla.util.json_util import dict_to_json_str
-from chinilla.util.keychain import (
-    Keychain,
-    passphrase_requirements,
-    supports_os_passphrase_storage,
-)
+from chinilla.util.keychain import Keychain, passphrase_requirements, supports_os_passphrase_storage
 from chinilla.util.lock import Lockfile, LockfileError
 from chinilla.util.network import WebServer
 from chinilla.util.service_groups import validate_service
@@ -150,7 +148,7 @@ class WebSocketServer:
         self.run_check_keys_on_unlock = run_check_keys_on_unlock
         self.shutdown_event = shutdown_event
 
-    async def start(self):
+    async def start(self) -> None:
         self.log.info("Starting Daemon Server")
 
         # Note: the minimum_version has been already set to TLSv1_2
@@ -208,7 +206,9 @@ class WebSocketServer:
     async def stop(self) -> Dict[str, Any]:
         self.cancel_task_safe(self.ping_job)
         service_names = list(self.services.keys())
-        stop_service_jobs = [kill_service(self.root_path, self.services, s_n) for s_n in service_names]
+        stop_service_jobs = [
+            asyncio.create_task(kill_service(self.root_path, self.services, s_n)) for s_n in service_names
+        ]
         if stop_service_jobs:
             await asyncio.wait(stop_service_jobs)
         self.services.clear()
@@ -345,14 +345,10 @@ class WebSocketServer:
             response = await self.unlock_keyring(data)
         elif command == "validate_keyring_passphrase":
             response = await self.validate_keyring_passphrase(data)
-        elif command == "migrate_keyring":
-            response = await self.migrate_keyring(data)
         elif command == "set_keyring_passphrase":
             response = await self.set_keyring_passphrase(data)
         elif command == "remove_keyring_passphrase":
             response = await self.remove_keyring_passphrase(data)
-        elif command == "notify_keyring_migration_completed":
-            response = await self.notify_keyring_migration_completed(data)
         elif command == "exit":
             response = await self.stop()
         elif command == "register_service":
@@ -379,8 +375,6 @@ class WebSocketServer:
         can_save_passphrase: bool = supports_os_passphrase_storage()
         user_passphrase_is_set: bool = Keychain.has_master_passphrase() and not using_default_passphrase()
         locked: bool = Keychain.is_keyring_locked()
-        needs_migration: bool = Keychain.needs_migration()
-        can_remove_legacy_keys: bool = False  # Disabling GUI support for removing legacy keys post-migration
         can_set_passphrase_hint: bool = True
         passphrase_hint: str = Keychain.get_master_passphrase_hint() or ""
         requirements: Dict[str, Any] = passphrase_requirements()
@@ -389,8 +383,6 @@ class WebSocketServer:
             "is_keyring_locked": locked,
             "can_save_passphrase": can_save_passphrase,
             "user_passphrase_is_set": user_passphrase_is_set,
-            "needs_migration": needs_migration,
-            "can_remove_legacy_keys": can_remove_legacy_keys,
             "can_set_passphrase_hint": can_set_passphrase_hint,
             "passphrase_hint": passphrase_hint,
             "passphrase_requirements": requirements,
@@ -448,54 +440,6 @@ class WebSocketServer:
         response: Dict[str, Any] = {"success": success, "error": error}
         return response
 
-    async def migrate_keyring(self, request: Dict[str, Any]) -> Dict[str, Any]:
-        if Keychain.needs_migration() is False:
-            # If the keyring has already been migrated, we'll raise an error to the client.
-            # The reason for raising an error is because the migration request has side-
-            # effects beyond copying keys from the legacy keyring to the new keyring. The
-            # request may have set a passphrase and indicated that keys should be cleaned
-            # from the legacy keyring. If we were to return early and indicate success,
-            # the client and user's expectations may not match reality (were my keys
-            # deleted from the legacy keyring? was my passphrase set?).
-            return {"success": False, "error": "migration not needed"}
-
-        success: bool = False
-        error: Optional[str] = None
-        passphrase: Optional[str] = request.get("passphrase", None)
-        passphrase_hint: Optional[str] = request.get("passphrase_hint", None)
-        save_passphrase: bool = request.get("save_passphrase", False)
-        cleanup_legacy_keyring: bool = request.get("cleanup_legacy_keyring", False)
-
-        if passphrase is not None and type(passphrase) is not str:
-            return {"success": False, "error": 'expected string value for "passphrase"'}
-
-        if passphrase_hint is not None and type(passphrase_hint) is not str:
-            return {"success": False, "error": 'expected string value for "passphrase_hint"'}
-
-        if not Keychain.passphrase_meets_requirements(passphrase):
-            return {"success": False, "error": "passphrase doesn't satisfy requirements"}
-
-        if type(cleanup_legacy_keyring) is not bool:
-            return {"success": False, "error": 'expected bool value for "cleanup_legacy_keyring"'}
-
-        try:
-            Keychain.migrate_legacy_keyring(
-                passphrase=passphrase,
-                passphrase_hint=passphrase_hint,
-                save_passphrase=save_passphrase,
-                cleanup_legacy_keyring=cleanup_legacy_keyring,
-            )
-            success = True
-            # Inform the GUI of keyring status changes
-            self.keyring_status_changed(await self.keyring_status(), "wallet_ui")
-        except Exception as e:
-            tb = traceback.format_exc()
-            self.log.error(f"Legacy keyring migration failed: {e} {tb}")
-            error = f"keyring migration failed: {e}"
-
-        response: Dict[str, Any] = {"success": success, "error": error}
-        return response
-
     async def set_keyring_passphrase(self, request: Dict[str, Any]) -> Dict[str, Any]:
         success: bool = False
         error: Optional[str] = None
@@ -527,8 +471,6 @@ class WebSocketServer:
                 passphrase_hint=passphrase_hint,
                 save_passphrase=save_passphrase,
             )
-        except KeychainRequiresMigration:
-            error = "keyring requires migration"
         except KeychainCurrentPassphraseIsInvalid:
             error = "current passphrase is invalid"
         except Exception as e:
@@ -569,32 +511,6 @@ class WebSocketServer:
         response: Dict[str, Any] = {"success": success, "error": error}
         return response
 
-    async def notify_keyring_migration_completed(self, request: Dict[str, Any]) -> Dict[str, Any]:
-        success: bool = False
-        error: Optional[str] = None
-        key: Optional[str] = request.get("key", None)
-
-        if type(key) is not str:
-            return {"success": False, "error": "missing key"}
-
-        Keychain.handle_migration_completed()
-
-        try:
-            if Keychain.master_passphrase_is_valid(key, force_reload=True):
-                Keychain.set_cached_master_passphrase(key)
-                success = True
-                # Inform the GUI of keyring status changes
-                self.keyring_status_changed(await self.keyring_status(), "wallet_ui")
-            else:
-                error = "bad passphrase"
-        except Exception as e:
-            tb = traceback.format_exc()
-            self.log.error(f"Keyring passphrase validation failed: {e} {tb}")
-            error = "validation exception"
-
-        response: Dict[str, Any] = {"success": success, "error": error}
-        return response
-
     def get_status(self) -> Dict[str, Any]:
         response = {"success": True, "genesis_initialized": True}
         return response
@@ -611,7 +527,7 @@ class WebSocketServer:
     async def _keyring_status_changed(self, keyring_status: Dict[str, Any], destination: str):
         """
         Attempt to communicate with the GUI to inform it of any keyring status changes
-        (e.g. keyring becomes unlocked or migration completes)
+        (e.g. keyring becomes unlocked)
         """
         websockets = self.connections.get("wallet_ui", None)
 
@@ -699,12 +615,10 @@ class WebSocketServer:
         plotter: str = config["plotter"]
         final_words: List[str] = []
 
-        if plotter == "chinillapos":
+        if plotter == "chiapos":
             final_words = ["Renamed final file"]
         elif plotter == "bladebit":
             final_words = ["Finished plotting in"]
-        elif plotter == "bladebit2":
-            final_words = ["Finished plotting in"]
         elif plotter == "madmax":
             temp_dir = config["temp_dir"]
             final_dir = config["final_dir"]
@@ -746,21 +660,21 @@ class WebSocketServer:
         p = request.get("p")  # Pool pubkey
         c = request.get("c")  # Pool contract address
 
-        command_args: List[str] = []
-        command_args.append(f"-n{n}")
-        command_args.append(f"-d{d}")
-        command_args.append(f"-r{r}")
+        command_args: List[str] = ["-n", str(n), "-d", d, "-r", str(r)]
 
         if f is not None:
-            command_args.append(f"-f{f}")
+            command_args.append("-f")
+            command_args.append(str(f))
         if p is not None:
-            command_args.append(f"-p{p}")
+            command_args.append("-p")
+            command_args.append(str(p))
         if c is not None:
-            command_args.append(f"-c{c}")
+            command_args.append("-c")
+            command_args.append(str(c))
 
         return command_args
 
-    def _chinillapos_plotting_command_args(self, request: Any, ignoreCount: bool) -> List[str]:
+    def _chiapos_plotting_command_args(self, request: Any, ignoreCount: bool) -> List[str]:
         k = request["k"]  # Plot size
         t = request["t"]  # Temp directory
         t2 = request["t2"]  # Temp2 directory
@@ -771,15 +685,11 @@ class WebSocketServer:
         x = request["x"]  # Exclude final directory
         override_k = request["overrideK"]  # Force plot sizes < k32
 
-        command_args: List[str] = []
-        command_args.append(f"-k{k}")
-        command_args.append(f"-t{t}")
-        command_args.append(f"-2{t2}")
-        command_args.append(f"-b{b}")
-        command_args.append(f"-u{u}")
+        command_args: List[str] = ["-k", str(k), "-t", t, "-2", t2, "-b", str(b), "-u", str(u)]
 
         if a is not None:
-            command_args.append(f"-a{a}")
+            command_args.append("-a")
+            command_args.append(str(a))
         if e is True:
             command_args.append("-e")
         if x is True:
@@ -790,22 +700,26 @@ class WebSocketServer:
         return command_args
 
     def _bladebit_plotting_command_args(self, request: Any, ignoreCount: bool) -> List[str]:
-        w = request.get("w", False)  # Warm start
-        m = request.get("m", False)  # Disable NUMA
-        no_cpu_affinity = request.get("no_cpu_affinity", False)
+        plot_type = request["plot_type"]
+        assert plot_type == "ramplot" or plot_type == "diskplot"
 
         command_args: List[str] = []
 
-        if w is True:
-            command_args.append("-w")
-        if m is True:
-            command_args.append("-m")
-        if no_cpu_affinity is True:
-            command_args.append("--no-cpu-affinity")
+        if plot_type == "ramplot":
+            w = request.get("w", False)  # Warm start
+            m = request.get("m", False)  # Disable NUMA
+            no_cpu_affinity = request.get("no_cpu_affinity", False)
 
-        return command_args
+            if w is True:
+                command_args.append("--warmstart")
+            if m is True:
+                command_args.append("--nonuma")
+            if no_cpu_affinity is True:
+                command_args.append("--no-cpu-affinity")
 
-    def _bladebit2_plotting_command_args(self, request: Any, ignoreCount: bool) -> List[str]:
+            return command_args
+
+        # if plot_type == "diskplot"
         w = request.get("w", False)  # Warm start
         m = request.get("m", False)  # Disable NUMA
         no_cpu_affinity = request.get("no_cpu_affinity", False)
@@ -823,20 +737,21 @@ class WebSocketServer:
         no_t1_direct = request.get("no_t1_direct", False)
         no_t2_direct = request.get("no_t2_direct", False)
 
-        command_args: List[str] = []
-
         if w is True:
-            command_args.append("-w")
+            command_args.append("--warmstart")
         if m is True:
-            command_args.append("-m")
+            command_args.append("--nonuma")
         if no_cpu_affinity is True:
             command_args.append("--no-cpu-affinity")
 
-        command_args.append(f"-t{t1}")
+        command_args.append("-t")
+        command_args.append(t1)
         if t2:
-            command_args.append(f"-2{t2}")
+            command_args.append("-2")
+            command_args.append(t2)
         if u:
-            command_args.append(f"-u{u}")
+            command_args.append("-u")
+            command_args.append(str(u))
         if cache:
             command_args.append("--cache")
             command_args.append(str(cache))
@@ -891,19 +806,24 @@ class WebSocketServer:
         return command_args
 
     def _build_plotting_command_args(self, request: Any, ignoreCount: bool, index: int) -> List[str]:
-        plotter: str = request.get("plotter", "chinillapos")
+        plotter: str = request.get("plotter", "chiapos")
         command_args: List[str] = ["chinilla", "plotters", plotter]
 
+        if plotter == "bladebit":
+            # plotter command must be either
+            # 'chinilla plotters bladebit ramplot' or 'chinilla plotters bladebit diskplot'
+            plot_type = request["plot_type"]
+            assert plot_type == "diskplot" or plot_type == "ramplot"
+            command_args.append(plot_type)
+
         command_args.extend(self._common_plotting_command_args(request, ignoreCount))
 
-        if plotter == "chinillapos":
-            command_args.extend(self._chinillapos_plotting_command_args(request, ignoreCount))
+        if plotter == "chiapos":
+            command_args.extend(self._chiapos_plotting_command_args(request, ignoreCount))
         elif plotter == "madmax":
             command_args.extend(self._madmax_plotting_command_args(request, ignoreCount, index))
         elif plotter == "bladebit":
             command_args.extend(self._bladebit_plotting_command_args(request, ignoreCount))
-        elif plotter == "bladebit2":
-            command_args.extend(self._bladebit2_plotting_command_args(request, ignoreCount))
 
         return command_args
 
@@ -1012,7 +932,7 @@ class WebSocketServer:
     async def start_plotting(self, request: Dict[str, Any]):
         service_name = request["service"]
 
-        plotter = request.get("plotter", "chinillapos")
+        plotter = request.get("plotter", "chiapos")
         delay = int(request.get("delay", 0))
         parallel = request.get("parallel", False)
         size = request.get("k")
