@@ -9,9 +9,8 @@ from pathlib import Path
 from typing import Any, Callable, Dict, List, Optional, Set, Tuple
 
 import aiohttp
-from chinillablspy import AugSchemeMPL, G1Element, G2Element, PrivateKey
+from blspy import AugSchemeMPL, G1Element, G2Element, PrivateKey
 
-import chinilla.server.ws_connection as ws  # lgtm [py/import-and-import-from]
 from chinilla.consensus.constants import ConsensusConstants
 from chinilla.daemon.keychain_proxy import KeychainProxy, connect_to_keychain_and_validate, wrap_local_keychain
 from chinilla.plot_sync.delta import Delta
@@ -257,7 +256,7 @@ class Farmer:
             ErrorResponse(uint16(PoolErrorCode.REQUEST_FAILED.value), error_message).to_json_dict()
         )
 
-    def on_disconnect(self, connection: ws.WSChinillaConnection):
+    def on_disconnect(self, connection: WSChinillaConnection):
         self.log.info(f"peer disconnected {connection.get_peer_logging()}")
         self.state_changed("close_connection", {})
         if connection.connection_type is NodeType.HARVESTER:
