@@ -1,26 +1,28 @@
+from __future__ import annotations
+
 import asyncio
 import dataclasses
+import functools
 import logging
 import time
 import traceback
-import functools
+from concurrent.futures import ThreadPoolExecutor
 from datetime import datetime, timezone
 from secrets import token_bytes
-from typing import Dict, List, Optional, Tuple, Set
+from typing import Dict, List, Optional, Set, Tuple
 
-from chinillablspy import AugSchemeMPL, G2Element, G1Element
-from chinillabip158 import PyBIP158
+from blspy import AugSchemeMPL, G1Element, G2Element
+from chiabip158 import PyBIP158
 
-import chinilla.server.ws_connection as ws
 from chinilla.consensus.block_creation import create_unfinished_block
 from chinilla.consensus.block_record import BlockRecord
 from chinilla.consensus.pot_iterations import calculate_ip_iters, calculate_iterations_quality, calculate_sp_iters
 from chinilla.full_node.bundle_tools import best_solution_generator_from_template, simple_solution_generator
 from chinilla.full_node.fee_estimate import FeeEstimate, FeeEstimateGroup
-from chinilla.full_node.full_node import FullNode
-from chinilla.full_node.mempool_check_conditions import get_puzzle_and_solution_for_coin
-from chinilla.full_node.signage_point import SignagePoint
 from chinilla.full_node.fee_estimator_interface import FeeEstimatorInterface
+from chinilla.full_node.full_node import FullNode
+from chinilla.full_node.mempool_check_conditions import get_name_puzzle_conditions, get_puzzle_and_solution_for_coin
+from chinilla.full_node.signage_point import SignagePoint
 from chinilla.protocols import (
     farmer_protocol,
     full_node_protocol,
@@ -31,20 +33,21 @@ from chinilla.protocols import (
 from chinilla.protocols.full_node_protocol import RejectBlock, RejectBlocks
 from chinilla.protocols.protocol_message_types import ProtocolMessageTypes
 from chinilla.protocols.wallet_protocol import (
+    CoinState,
     PuzzleSolutionResponse,
     RejectBlockHeaders,
     RejectHeaderBlocks,
     RejectHeaderRequest,
-    CoinState,
     RespondFeeEstimates,
     RespondSESInfo,
 )
-from chinilla.server.server import ChinillaServer
-from concurrent.futures import ThreadPoolExecutor
-from chinilla.types.block_protocol import BlockInfo
 from chinilla.server.outbound_message import Message, make_msg
+from chinilla.server.server import ChinillaServer
+from chinilla.server.ws_connection import WSChinillaConnection
+from chinilla.types.block_protocol import BlockInfo
 from chinilla.types.blockchain_format.coin import Coin, hash_coin_ids
 from chinilla.types.blockchain_format.pool_target import PoolTarget
+from chinilla.types.blockchain_format.proof_of_space import verify_and_get_quality_string
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.types.blockchain_format.sub_epoch_summary import SubEpochSummary
 from chinilla.types.coin_record import CoinRecord
@@ -56,13 +59,13 @@ from chinilla.types.mempool_item import MempoolItem
 from chinilla.types.peer_info import PeerInfo
 from chinilla.types.transaction_queue_entry import TransactionQueueEntry
 from chinilla.types.unfinished_block import UnfinishedBlock
-from chinilla.util.api_decorators import api_request, peer_required, bytes_required, execute_task, reply_type
+from chinilla.util.api_decorators import api_request
 from chinilla.util.full_block_utils import header_block_from_block
 from chinilla.util.generator_tools import get_block_header, tx_removals_and_additions
 from chinilla.util.hash import std_hash
 from chinilla.util.ints import uint8, uint32, uint64, uint128
+from chinilla.util.limited_semaphore import LimitedSemaphoreFullError
 from chinilla.util.merkle_set import MerkleSet
-from chinilla.full_node.mempool_check_conditions import get_name_puzzle_conditions
 
 
 class FullNodeAPI:
@@ -86,11 +89,9 @@ class FullNodeAPI:
     def api_ready(self) -> bool:
         return self.full_node.initialized
 
-    @peer_required
-    @api_request
-    @reply_type([ProtocolMessageTypes.respond_peers])
+    @api_request(peer_required=True, reply_types=[ProtocolMessageTypes.respond_peers])
     async def request_peers(
-        self, _request: full_node_protocol.RequestPeers, peer: ws.WSChinillaConnection
+        self, _request: full_node_protocol.RequestPeers, peer: WSChinillaConnection
     ) -> Optional[Message]:
         if peer.peer_server_port is None:
             return None
@@ -100,20 +101,18 @@ class FullNodeAPI:
             return msg
         return None
 
-    @peer_required
-    @api_request
+    @api_request(peer_required=True)
     async def respond_peers(
-        self, request: full_node_protocol.RespondPeers, peer: ws.WSChinillaConnection
+        self, request: full_node_protocol.RespondPeers, peer: WSChinillaConnection
     ) -> Optional[Message]:
         self.log.debug(f"Received {len(request.peer_list)} peers")
         if self.full_node.full_node_peers is not None:
             await self.full_node.full_node_peers.respond_peers(request, peer.get_peer_info(), True)
         return None
 
-    @peer_required
-    @api_request
+    @api_request(peer_required=True)
     async def respond_peers_introducer(
-        self, request: introducer_protocol.RespondPeersIntroducer, peer: ws.WSChinillaConnection
+        self, request: introducer_protocol.RespondPeersIntroducer, peer: WSChinillaConnection
     ) -> Optional[Message]:
         self.log.debug(f"Received {len(request.peer_list)} peers from introducer")
         if self.full_node.full_node_peers is not None:
@@ -122,33 +121,25 @@ class FullNodeAPI:
         await peer.close()
         return None
 
-    @execute_task
-    @peer_required
-    @api_request
-    async def new_peak(self, request: full_node_protocol.NewPeak, peer: ws.WSChinillaConnection) -> None:
+    @api_request(peer_required=True, execute_task=True)
+    async def new_peak(self, request: full_node_protocol.NewPeak, peer: WSChinillaConnection) -> None:
         """
         A peer notifies us that they have added a new peak to their blockchain. If we don't have it,
         we can ask for it.
         """
         # this semaphore limits the number of tasks that can call new_peak() at
         # the same time, since it can be expensive
-        new_peak_sem = self.full_node.new_peak_sem
-        waiter_count = 0 if new_peak_sem._waiters is None else len(new_peak_sem._waiters)
-
-        if waiter_count > 0:
-            self.full_node.log.debug(f"new_peak Waiters: {waiter_count}")
-
-        if waiter_count > 20:
+        try:
+            async with self.full_node.new_peak_sem.acquire():
+                await self.full_node.new_peak(request, peer)
+        except LimitedSemaphoreFullError:
             return None
 
-        async with new_peak_sem:
-            await self.full_node.new_peak(request, peer)
         return None
 
-    @peer_required
-    @api_request
+    @api_request(peer_required=True)
     async def new_transaction(
-        self, transaction: full_node_protocol.NewTransaction, peer: ws.WSChinillaConnection
+        self, transaction: full_node_protocol.NewTransaction, peer: WSChinillaConnection
     ) -> Optional[Message]:
         """
         A peer notifies us of a new transaction.
@@ -229,8 +220,7 @@ class FullNodeAPI:
             return None
         return None
 
-    @api_request
-    @reply_type([ProtocolMessageTypes.respond_transaction])
+    @api_request(reply_types=[ProtocolMessageTypes.respond_transaction])
     async def request_transaction(self, request: full_node_protocol.RequestTransaction) -> Optional[Message]:
         """Peer has requested a full transaction from us."""
         # Ignore if syncing
@@ -245,13 +235,11 @@ class FullNodeAPI:
         msg = make_msg(ProtocolMessageTypes.respond_transaction, transaction)
         return msg
 
-    @peer_required
-    @api_request
-    @bytes_required
+    @api_request(peer_required=True, bytes_required=True)
     async def respond_transaction(
         self,
         tx: full_node_protocol.RespondTransaction,
-        peer: ws.WSChinillaConnection,
+        peer: WSChinillaConnection,
         tx_bytes: bytes = b"",
         test: bool = False,
     ) -> Optional[Message]:
@@ -277,8 +265,7 @@ class FullNodeAPI:
         )
         return None
 
-    @api_request
-    @reply_type([ProtocolMessageTypes.respond_proof_of_weight])
+    @api_request(reply_types=[ProtocolMessageTypes.respond_proof_of_weight])
     async def request_proof_of_weight(self, request: full_node_protocol.RequestProofOfWeight) -> Optional[Message]:
         if self.full_node.weight_proof_handler is None:
             return None
@@ -318,13 +305,12 @@ class FullNodeAPI:
         self.full_node.full_node_store.serialized_wp_message = message
         return message
 
-    @api_request
+    @api_request()
     async def respond_proof_of_weight(self, request: full_node_protocol.RespondProofOfWeight) -> Optional[Message]:
         self.log.warning("Received proof of weight too late.")
         return None
 
-    @api_request
-    @reply_type([ProtocolMessageTypes.respond_block, ProtocolMessageTypes.reject_block])
+    @api_request(reply_types=[ProtocolMessageTypes.respond_block, ProtocolMessageTypes.reject_block])
     async def request_block(self, request: full_node_protocol.RequestBlock) -> Optional[Message]:
         if not self.full_node.blockchain.contains_height(request.height):
             reject = RejectBlock(request.height)
@@ -341,8 +327,7 @@ class FullNodeAPI:
             return make_msg(ProtocolMessageTypes.respond_block, full_node_protocol.RespondBlock(block))
         return make_msg(ProtocolMessageTypes.reject_block, RejectBlock(request.height))
 
-    @api_request
-    @reply_type([ProtocolMessageTypes.respond_blocks, ProtocolMessageTypes.reject_blocks])
+    @api_request(reply_types=[ProtocolMessageTypes.respond_blocks, ProtocolMessageTypes.reject_blocks])
     async def request_blocks(self, request: full_node_protocol.RequestBlocks) -> Optional[Message]:
         if request.end_height < request.start_height or request.end_height - request.start_height > 32:
             reject = RejectBlocks(request.start_height, request.end_height)
@@ -398,25 +383,24 @@ class FullNodeAPI:
 
         return msg
 
-    @api_request
+    @api_request()
     async def reject_block(self, request: full_node_protocol.RejectBlock) -> None:
         self.log.debug(f"reject_block {request.height}")
 
-    @api_request
+    @api_request()
     async def reject_blocks(self, request: full_node_protocol.RejectBlocks) -> None:
         self.log.debug(f"reject_blocks {request.start_height} {request.end_height}")
 
-    @api_request
+    @api_request()
     async def respond_blocks(self, request: full_node_protocol.RespondBlocks) -> None:
         self.log.warning("Received unsolicited/late blocks")
         return None
 
-    @api_request
-    @peer_required
+    @api_request(peer_required=True)
     async def respond_block(
         self,
         respond_block: full_node_protocol.RespondBlock,
-        peer: ws.WSChinillaConnection,
+        peer: WSChinillaConnection,
     ) -> Optional[Message]:
         """
         Receive a full block from a peer full node (or ourselves).
@@ -425,7 +409,7 @@ class FullNodeAPI:
         self.log.warning(f"Received unsolicited/late block from peer {peer.get_peer_logging()}")
         return None
 
-    @api_request
+    @api_request()
     async def new_unfinished_block(
         self, new_unfinished_block: full_node_protocol.NewUnfinishedBlock
     ) -> Optional[Message]:
@@ -457,8 +441,7 @@ class FullNodeAPI:
 
         return msg
 
-    @api_request
-    @reply_type([ProtocolMessageTypes.respond_unfinished_block])
+    @api_request(reply_types=[ProtocolMessageTypes.respond_unfinished_block])
     async def request_unfinished_block(
         self, request_unfinished_block: full_node_protocol.RequestUnfinishedBlock
     ) -> Optional[Message]:
@@ -473,13 +456,11 @@ class FullNodeAPI:
             return msg
         return None
 
-    @peer_required
-    @api_request
-    @bytes_required
+    @api_request(peer_required=True, bytes_required=True)
     async def respond_unfinished_block(
         self,
         respond_unfinished_block: full_node_protocol.RespondUnfinishedBlock,
-        peer: ws.WSChinillaConnection,
+        peer: WSChinillaConnection,
         respond_unfinished_block_bytes: bytes = b"",
     ) -> Optional[Message]:
         if self.full_node.sync_store.get_sync_mode():
@@ -489,10 +470,9 @@ class FullNodeAPI:
         )
         return None
 
-    @api_request
-    @peer_required
+    @api_request(peer_required=True)
     async def new_signage_point_or_end_of_sub_slot(
-        self, new_sp: full_node_protocol.NewSignagePointOrEndOfSubSlot, peer: ws.WSChinillaConnection
+        self, new_sp: full_node_protocol.NewSignagePointOrEndOfSubSlot, peer: WSChinillaConnection
     ) -> Optional[Message]:
         # Ignore if syncing
         if self.full_node.sync_store.get_sync_mode():
@@ -572,8 +552,7 @@ class FullNodeAPI:
 
         return make_msg(ProtocolMessageTypes.request_signage_point_or_end_of_sub_slot, full_node_request)
 
-    @api_request
-    @reply_type([ProtocolMessageTypes.respond_signage_point, ProtocolMessageTypes.respond_end_of_sub_slot])
+    @api_request(reply_types=[ProtocolMessageTypes.respond_signage_point, ProtocolMessageTypes.respond_end_of_sub_slot])
     async def request_signage_point_or_end_of_sub_slot(
         self, request: full_node_protocol.RequestSignagePointOrEndOfSubSlot
     ) -> Optional[Message]:
@@ -616,10 +595,9 @@ class FullNodeAPI:
                 self.log.info(f"Don't have signage point {request}")
         return None
 
-    @peer_required
-    @api_request
+    @api_request(peer_required=True)
     async def respond_signage_point(
-        self, request: full_node_protocol.RespondSignagePoint, peer: ws.WSChinillaConnection
+        self, request: full_node_protocol.RespondSignagePoint, peer: WSChinillaConnection
     ) -> Optional[Message]:
         if self.full_node.sync_store.get_sync_mode():
             return None
@@ -672,22 +650,20 @@ class FullNodeAPI:
 
             return None
 
-    @peer_required
-    @api_request
+    @api_request(peer_required=True)
     async def respond_end_of_sub_slot(
-        self, request: full_node_protocol.RespondEndOfSubSlot, peer: ws.WSChinillaConnection
+        self, request: full_node_protocol.RespondEndOfSubSlot, peer: WSChinillaConnection
     ) -> Optional[Message]:
         if self.full_node.sync_store.get_sync_mode():
             return None
         msg, _ = await self.full_node.respond_end_of_sub_slot(request, peer)
         return msg
 
-    @peer_required
-    @api_request
+    @api_request(peer_required=True)
     async def request_mempool_transactions(
         self,
         request: full_node_protocol.RequestMempoolTransactions,
-        peer: ws.WSChinillaConnection,
+        peer: WSChinillaConnection,
     ) -> Optional[Message]:
         received_filter = PyBIP158(bytearray(request.filter))
 
@@ -700,10 +676,9 @@ class FullNodeAPI:
         return None
 
     # FARMER PROTOCOL
-    @api_request
-    @peer_required
+    @api_request(peer_required=True)
     async def declare_proof_of_space(
-        self, request: farmer_protocol.DeclareProofOfSpace, peer: ws.WSChinillaConnection
+        self, request: farmer_protocol.DeclareProofOfSpace, peer: WSChinillaConnection
     ) -> Optional[Message]:
         """
         Creates a block body and header, with the proof of space, coinbase, and fee targets provided
@@ -753,8 +728,8 @@ class FullNodeAPI:
             # 3. In a future sub-slot that we already know of
 
             # Checks that the proof of space is valid
-            quality_string: Optional[bytes32] = request.proof_of_space.verify_and_get_quality_string(
-                self.full_node.constants, cc_challenge_hash, request.challenge_chain_sp
+            quality_string: Optional[bytes32] = verify_and_get_quality_string(
+                request.proof_of_space, self.full_node.constants, cc_challenge_hash, request.challenge_chain_sp
             )
             assert quality_string is not None and len(quality_string) == 32
 
@@ -990,10 +965,9 @@ class FullNodeAPI:
                 )
         return None
 
-    @api_request
-    @peer_required
+    @api_request(peer_required=True)
     async def signed_values(
-        self, farmer_request: farmer_protocol.SignedValues, peer: ws.WSChinillaConnection
+        self, farmer_request: farmer_protocol.SignedValues, peer: WSChinillaConnection
     ) -> Optional[Message]:
         """
         Signature of header hash, by the harvester. This is enough to create an unfinished
@@ -1057,10 +1031,9 @@ class FullNodeAPI:
         return None
 
     # TIMELORD PROTOCOL
-    @peer_required
-    @api_request
+    @api_request(peer_required=True)
     async def new_infusion_point_vdf(
-        self, request: timelord_protocol.NewInfusionPointVDF, peer: ws.WSChinillaConnection
+        self, request: timelord_protocol.NewInfusionPointVDF, peer: WSChinillaConnection
     ) -> Optional[Message]:
         if self.full_node.sync_store.get_sync_mode():
             return None
@@ -1068,10 +1041,9 @@ class FullNodeAPI:
         async with self.full_node.timelord_lock:
             return await self.full_node.new_infusion_point_vdf(request, peer)
 
-    @peer_required
-    @api_request
+    @api_request(peer_required=True)
     async def new_signage_point_vdf(
-        self, request: timelord_protocol.NewSignagePointVDF, peer: ws.WSChinillaConnection
+        self, request: timelord_protocol.NewSignagePointVDF, peer: WSChinillaConnection
     ) -> None:
         if self.full_node.sync_store.get_sync_mode():
             return None
@@ -1085,10 +1057,9 @@ class FullNodeAPI:
         )
         await self.respond_signage_point(full_node_message, peer)
 
-    @peer_required
-    @api_request
+    @api_request(peer_required=True)
     async def new_end_of_sub_slot_vdf(
-        self, request: timelord_protocol.NewEndOfSubSlotVDF, peer: ws.WSChinillaConnection
+        self, request: timelord_protocol.NewEndOfSubSlotVDF, peer: WSChinillaConnection
     ) -> Optional[Message]:
         if self.full_node.sync_store.get_sync_mode():
             return None
@@ -1111,7 +1082,7 @@ class FullNodeAPI:
         else:
             return msg
 
-    @api_request
+    @api_request()
     async def request_block_header(self, request: wallet_protocol.RequestBlockHeader) -> Optional[Message]:
         header_hash = self.full_node.blockchain.height_to_hash(request.height)
         if header_hash is None:
@@ -1152,7 +1123,7 @@ class FullNodeAPI:
         )
         return msg
 
-    @api_request
+    @api_request()
     async def request_additions(self, request: wallet_protocol.RequestAdditions) -> Optional[Message]:
         if request.header_hash is None:
             header_hash: Optional[bytes32] = self.full_node.blockchain.height_to_hash(request.height)
@@ -1207,7 +1178,7 @@ class FullNodeAPI:
             response = wallet_protocol.RespondAdditions(request.height, header_hash, coins_map, proofs_map)
         return make_msg(ProtocolMessageTypes.respond_additions, response)
 
-    @api_request
+    @api_request()
     async def request_removals(self, request: wallet_protocol.RequestRemovals) -> Optional[Message]:
         block: Optional[FullBlock] = await self.full_node.block_store.get_full_block(request.header_hash)
 
@@ -1272,7 +1243,7 @@ class FullNodeAPI:
         msg = make_msg(ProtocolMessageTypes.respond_removals, response)
         return msg
 
-    @api_request
+    @api_request()
     async def send_transaction(
         self, request: wallet_protocol.SendTransaction, *, test: bool = False
     ) -> Optional[Message]:
@@ -1313,7 +1284,7 @@ class FullNodeAPI:
                     response = wallet_protocol.TransactionAck(spend_name, uint8(status.value), error_name)
         return make_msg(ProtocolMessageTypes.transaction_ack, response)
 
-    @api_request
+    @api_request()
     async def request_puzzle_solution(self, request: wallet_protocol.RequestPuzzleSolution) -> Optional[Message]:
         coin_name = request.coin_name
         height = request.height
@@ -1349,7 +1320,7 @@ class FullNodeAPI:
         response_msg = make_msg(ProtocolMessageTypes.respond_puzzle_solution, response)
         return response_msg
 
-    @api_request
+    @api_request()
     async def request_block_headers(self, request: wallet_protocol.RequestBlockHeaders) -> Optional[Message]:
         """Returns header blocks by directly streaming bytes into Message
 
@@ -1396,7 +1367,7 @@ class FullNodeAPI:
         respond_header_blocks_manually_streamed += b"".join(header_blocks_bytes)
         return make_msg(ProtocolMessageTypes.respond_block_headers, respond_header_blocks_manually_streamed)
 
-    @api_request
+    @api_request()
     async def request_header_blocks(self, request: wallet_protocol.RequestHeaderBlocks) -> Optional[Message]:
         """DEPRECATED: please use RequestBlockHeaders"""
         if request.end_height < request.start_height or request.end_height - request.start_height > 32:
@@ -1430,29 +1401,20 @@ class FullNodeAPI:
         )
         return msg
 
-    @api_request
+    @api_request()
     async def respond_compact_proof_of_time(self, request: timelord_protocol.RespondCompactProofOfTime) -> None:
         if self.full_node.sync_store.get_sync_mode():
             return None
         await self.full_node.respond_compact_proof_of_time(request)
         return None
 
-    @execute_task
-    @peer_required
-    @api_request
-    @bytes_required
+    @api_request(peer_required=True, bytes_required=True, execute_task=True)
     async def new_compact_vdf(
-        self, request: full_node_protocol.NewCompactVDF, peer: ws.WSChinillaConnection, request_bytes: bytes = b""
+        self, request: full_node_protocol.NewCompactVDF, peer: WSChinillaConnection, request_bytes: bytes = b""
     ) -> None:
         if self.full_node.sync_store.get_sync_mode():
             return None
 
-        compact_vdf_sem = self.full_node.compact_vdf_sem
-        waiter_count = 0 if compact_vdf_sem._waiters is None else len(compact_vdf_sem._waiters)
-        if waiter_count > 20:
-            self.log.debug(f"Ignoring NewCompactVDF: {request}, _waiters")
-            return None
-
         name = std_hash(request_bytes)
         if name in self.full_node.compact_vdf_requests:
             self.log.debug(f"Ignoring NewCompactVDF: {request}, already requested")
@@ -1461,38 +1423,39 @@ class FullNodeAPI:
 
         # this semaphore will only allow a limited number of tasks call
         # new_compact_vdf() at a time, since it can be expensive
-        async with self.full_node.compact_vdf_sem:
-            try:
-                await self.full_node.new_compact_vdf(request, peer)
-            finally:
-                self.full_node.compact_vdf_requests.remove(name)
+        try:
+            async with self.full_node.compact_vdf_sem.acquire():
+                try:
+                    await self.full_node.new_compact_vdf(request, peer)
+                finally:
+                    self.full_node.compact_vdf_requests.remove(name)
+        except LimitedSemaphoreFullError:
+            self.log.debug(f"Ignoring NewCompactVDF: {request}, _waiters")
+            return None
+
         return None
 
-    @peer_required
-    @api_request
-    @reply_type([ProtocolMessageTypes.respond_compact_vdf])
+    @api_request(peer_required=True, reply_types=[ProtocolMessageTypes.respond_compact_vdf])
     async def request_compact_vdf(
-        self, request: full_node_protocol.RequestCompactVDF, peer: ws.WSChinillaConnection
+        self, request: full_node_protocol.RequestCompactVDF, peer: WSChinillaConnection
     ) -> None:
         if self.full_node.sync_store.get_sync_mode():
             return None
         await self.full_node.request_compact_vdf(request, peer)
         return None
 
-    @peer_required
-    @api_request
+    @api_request(peer_required=True)
     async def respond_compact_vdf(
-        self, request: full_node_protocol.RespondCompactVDF, peer: ws.WSChinillaConnection
+        self, request: full_node_protocol.RespondCompactVDF, peer: WSChinillaConnection
     ) -> None:
         if self.full_node.sync_store.get_sync_mode():
             return None
         await self.full_node.respond_compact_vdf(request, peer)
         return None
 
-    @peer_required
-    @api_request
+    @api_request(peer_required=True)
     async def register_interest_in_puzzle_hash(
-        self, request: wallet_protocol.RegisterForPhUpdates, peer: ws.WSChinillaConnection
+        self, request: wallet_protocol.RegisterForPhUpdates, peer: WSChinillaConnection
     ) -> Message:
         if peer.peer_node_id not in self.full_node.peer_puzzle_hash:
             self.full_node.peer_puzzle_hash[peer.peer_node_id] = set()
@@ -1531,10 +1494,9 @@ class FullNodeAPI:
         msg = make_msg(ProtocolMessageTypes.respond_to_ph_update, response)
         return msg
 
-    @peer_required
-    @api_request
+    @api_request(peer_required=True)
     async def register_interest_in_coin(
-        self, request: wallet_protocol.RegisterForCoinUpdates, peer: ws.WSChinillaConnection
+        self, request: wallet_protocol.RegisterForCoinUpdates, peer: WSChinillaConnection
     ) -> Message:
         if peer.peer_node_id not in self.full_node.peer_coin_ids:
             self.full_node.peer_coin_ids[peer.peer_node_id] = set()
@@ -1561,7 +1523,7 @@ class FullNodeAPI:
         msg = make_msg(ProtocolMessageTypes.respond_to_coin_update, response)
         return msg
 
-    @api_request
+    @api_request()
     async def request_children(self, request: wallet_protocol.RequestChildren) -> Optional[Message]:
         coin_records: List[CoinRecord] = await self.full_node.coin_store.get_coin_records_by_parent_ids(
             True, [request.coin_name]
@@ -1571,7 +1533,7 @@ class FullNodeAPI:
         msg = make_msg(ProtocolMessageTypes.respond_children, response)
         return msg
 
-    @api_request
+    @api_request()
     async def request_ses_hashes(self, request: wallet_protocol.RequestSESInfo) -> Message:
         """Returns the start and end height of a sub-epoch for the height specified in request"""
 
@@ -1607,9 +1569,7 @@ class FullNodeAPI:
         msg = make_msg(ProtocolMessageTypes.respond_ses_hashes, response)
         return msg
 
-    @peer_required
-    @api_request
-    @reply_type([ProtocolMessageTypes.respond_fee_estimates])
+    @api_request(peer_required=True, reply_types=[ProtocolMessageTypes.respond_fee_estimates])
     async def request_fee_estimates(self, request: wallet_protocol.RequestFeeEstimates) -> Message:
         def get_fee_estimates(est: FeeEstimatorInterface, req_times: List[uint64]) -> List[FeeEstimate]:
             now = datetime.now(timezone.utc)
