@@ -27,7 +27,9 @@ from chinilla.protocols.harvester_protocol import (
     PlotSyncResponse,
     PlotSyncStart,
 )
-from chinilla.server.ws_connection import ProtocolMessageTypes, WSChinillaConnection, make_msg
+from chinilla.protocols.protocol_message_types import ProtocolMessageTypes
+from chinilla.server.outbound_message import make_msg
+from chinilla.server.ws_connection import WSChinillaConnection
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.util.ints import int16, uint32, uint64
 from chinilla.util.misc import get_list_or_len
