@@ -6,8 +6,8 @@ from enum import Enum
 from pathlib import Path
 from typing import Dict, List, Optional, Tuple, Union
 
-from chinillablspy import G1Element, PrivateKey
-from chinillapos import DiskProver
+from blspy import G1Element, PrivateKey
+from chiapos import DiskProver
 
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.util.config import load_config, lock_and_load_config, save_config
@@ -78,7 +78,11 @@ def get_plot_filenames(root_path: Path) -> Dict[Path, List[Path]]:
     config = load_config(root_path, "config.yaml")
     recursive_scan: bool = config["harvester"].get("recursive_plot_scan", False)
     for directory_name in get_plot_directories(root_path, config):
-        directory = Path(directory_name).resolve()
+        try:
+            directory = Path(directory_name).resolve()
+        except (OSError, RuntimeError):
+            log.exception(f"Failed to resolve {directory_name}")
+            continue
         all_files[directory] = get_filenames(directory, recursive_scan)
     return all_files
 
