@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from pathlib import Path
 from typing import Any, Dict, List, Optional
 
@@ -12,8 +14,11 @@ class DataLayerRpcClient(RpcClient):
         # TODO: better hinting for .fetch() (probably a TypedDict)
         return response  # type: ignore[no-any-return]
 
-    async def get_value(self, store_id: bytes32, key: bytes) -> Dict[str, Any]:
-        response = await self.fetch("get_value", {"id": store_id.hex(), "key": key.hex()})
+    async def get_value(self, store_id: bytes32, key: bytes, root_hash: Optional[bytes32]) -> Dict[str, Any]:
+        request: Dict[str, Any] = {"id": store_id.hex(), "key": key.hex()}
+        if root_hash is not None:
+            request["root_hash"] = root_hash.hex()
+        response = await self.fetch("get_value", request)
         # TODO: better hinting for .fetch() (probably a TypedDict)
         return response  # type: ignore[no-any-return]
 
@@ -24,13 +29,19 @@ class DataLayerRpcClient(RpcClient):
         # TODO: better hinting for .fetch() (probably a TypedDict)
         return response  # type: ignore[no-any-return]
 
-    async def get_keys_values(self, store_id: bytes32) -> Dict[str, Any]:
-        response = await self.fetch("get_keys_values", {"id": store_id.hex()})
+    async def get_keys_values(self, store_id: bytes32, root_hash: Optional[bytes32]) -> Dict[str, Any]:
+        request: Dict[str, Any] = {"id": store_id.hex()}
+        if root_hash is not None:
+            request["root_hash"] = root_hash.hex()
+        response = await self.fetch("get_keys_values", request)
         # TODO: better hinting for .fetch() (probably a TypedDict)
         return response  # type: ignore[no-any-return]
 
-    async def get_keys(self, store_id: bytes32) -> Dict[str, Any]:
-        response = await self.fetch("get_keys", {"id": store_id.hex()})
+    async def get_keys(self, store_id: bytes32, root_hash: Optional[bytes32]) -> Dict[str, Any]:
+        request: Dict[str, Any] = {"id": store_id.hex()}
+        if root_hash is not None:
+            request["root_hash"] = root_hash.hex()
+        response = await self.fetch("get_keys", request)
         # TODO: better hinting for .fetch() (probably a TypedDict)
         return response  # type: ignore[no-any-return]
 
@@ -110,3 +121,7 @@ class DataLayerRpcClient(RpcClient):
     async def get_owned_stores(self) -> Dict[str, Any]:
         response = await self.fetch("get_owned_stores", {})
         return response  # type: ignore[no-any-return]
+
+    async def get_sync_status(self, store_id: bytes32) -> Dict[str, Any]:
+        response = await self.fetch("get_sync_status", {"id": store_id.hex()})
+        return response  # type: ignore[no-any-return]
