@@ -4,7 +4,8 @@ from typing import Dict, List, Optional, Any
 
 import aiohttp
 
-from chinilla.server.server import NodeType, ssl_context_for_client
+from chinilla.server.outbound_message import NodeType
+from chinilla.server.server import ssl_context_for_client
 from chinilla.server.ssl_context import private_ssl_ca_paths
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.util.byte_types import hexstr_to_bytes
