@@ -19,21 +19,6 @@ class NodeType(IntEnum):
     DATA_LAYER = 7
 
 
-class Delivery(IntEnum):
-    # A message is sent to the same peer that we received a message from
-    RESPOND = 1
-    # A message is sent to all peers
-    BROADCAST = 2
-    # A message is sent to all peers except the one from which we received the API call
-    BROADCAST_TO_OTHERS = 3
-    # A message is sent to a random peer
-    RANDOM = 4
-    # Pseudo-message to close the current connection
-    CLOSE = 5
-    # A message is sent to a specific peer
-    SPECIFIC = 6
-
-
 @streamable
 @dataclass(frozen=True)
 class Message(Streamable):
