@@ -1,6 +1,8 @@
+from __future__ import annotations
+
 import pathlib
 import sys
-from typing import Dict, Optional
+from typing import Any, Dict, Optional
 
 from chinilla.consensus.constants import ConsensusConstants
 from chinilla.consensus.default_constants import DEFAULT_CONSTANTS
@@ -10,7 +12,7 @@ from chinilla.rpc.farmer_rpc_api import FarmerRpcApi
 from chinilla.server.outbound_message import NodeType
 from chinilla.server.start_service import RpcInfo, Service, async_run
 from chinilla.types.peer_info import PeerInfo
-from chinilla.util.chinilla_logging import initialize_logging
+from chinilla.util.chinilla_logging import initialize_service_logging
 from chinilla.util.config import load_config, load_config_cli
 from chinilla.util.default_root import DEFAULT_ROOT_PATH
 from chinilla.util.keychain import Keychain
@@ -23,12 +25,12 @@ SERVICE_NAME = "farmer"
 
 def create_farmer_service(
     root_path: pathlib.Path,
-    config: Dict,
-    config_pool: Dict,
+    config: Dict[str, Any],
+    config_pool: Dict[str, Any],
     consensus_constants: ConsensusConstants,
     keychain: Optional[Keychain] = None,
     connect_to_daemon: bool = True,
-) -> Service:
+) -> Service[Farmer]:
     service_config = config[SERVICE_NAME]
 
     connect_peers = []
@@ -57,7 +59,6 @@ def create_farmer_service(
         service_name=SERVICE_NAME,
         server_listen_ports=[service_config["port"]],
         connect_peers=connect_peers,
-        auth_connect_peers=False,
         on_connect_callback=farmer.on_connect,
         network_id=network_id,
         rpc_info=rpc_info,
@@ -72,11 +73,7 @@ async def async_main() -> int:
     config[SERVICE_NAME] = service_config
     config_pool = load_config_cli(DEFAULT_ROOT_PATH, "config.yaml", "pool")
     config["pool"] = config_pool
-    initialize_logging(
-        service_name=SERVICE_NAME,
-        logging_config=service_config["logging"],
-        root_path=DEFAULT_ROOT_PATH,
-    )
+    initialize_service_logging(service_name=SERVICE_NAME, config=config)
     service = create_farmer_service(DEFAULT_ROOT_PATH, config, config_pool, DEFAULT_CONSTANTS)
     await service.setup_process_global_state()
     await service.run()
