@@ -2,13 +2,11 @@ from __future__ import annotations
 
 import asyncio
 import sys
-import tempfile
 from pathlib import Path
 from typing import Any, AsyncGenerator, Dict, Optional, Tuple
 
-from chinillablspy import PrivateKey
+from blspy import PrivateKey
 
-from chinilla.cmds.init_funcs import create_all_ssl
 from chinilla.consensus.coinbase import create_puzzlehash_for_pk
 from chinilla.daemon.server import WebSocketServer, daemon_launch_lock_path
 from chinilla.simulator.full_node_simulator import FullNodeSimulator
@@ -21,9 +19,11 @@ from chinilla.simulator.ssl_certs import (
     get_next_private_ca_cert_and_key,
 )
 from chinilla.simulator.start_simulator import async_main as start_simulator_main
+from chinilla.ssl.create_ssl import create_all_ssl
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.util.bech32m import encode_puzzle_hash
 from chinilla.util.config import create_default_chinilla_config, load_config, save_config
+from chinilla.util.errors import KeychainFingerprintExists
 from chinilla.util.ints import uint32
 from chinilla.util.keychain import Keychain
 from chinilla.util.lock import Lockfile
@@ -34,20 +34,23 @@ These functions are used to test the simulator.
 """
 
 
-def mnemonic_fingerprint() -> Tuple[str, int]:
+def mnemonic_fingerprint(keychain: Keychain) -> Tuple[str, int]:
     mnemonic = (
         "today grape album ticket joy idle supreme sausage "
         "oppose voice angle roast you oven betray exact "
         "memory riot escape high dragon knock food blade"
     )
     # add key to keychain
-    sk = Keychain().add_private_key(mnemonic)
+    try:
+        sk = keychain.add_private_key(mnemonic)
+    except KeychainFingerprintExists:
+        pass
     fingerprint = sk.get_g1().get_fingerprint()
     return mnemonic, fingerprint
 
 
-def get_puzzle_hash_from_key(fingerprint: int, key_id: int = 1) -> bytes32:
-    priv_key_and_entropy = Keychain().get_private_key_by_fingerprint(fingerprint)
+def get_puzzle_hash_from_key(keychain: Keychain, fingerprint: int, key_id: int = 1) -> bytes32:
+    priv_key_and_entropy = keychain.get_private_key_by_fingerprint(fingerprint)
     if priv_key_and_entropy is None:
         raise Exception("Fingerprint not found")
     private_key = priv_key_and_entropy[0]
@@ -61,6 +64,7 @@ def create_config(
     fingerprint: int,
     private_ca_crt_and_key: Tuple[bytes, bytes],
     node_certs_and_keys: Dict[str, Dict[str, Dict[str, bytes]]],
+    keychain: Keychain,
 ) -> Dict[str, Any]:
     # create chinilla directories
     create_default_chinilla_config(chinilla_root)
@@ -95,7 +99,7 @@ def create_config(
     config["full_node"]["rpc_port"] = find_available_listen_port("Node RPC")
     # simulator overrides
     config["simulator"]["key_fingerprint"] = fingerprint
-    config["simulator"]["farming_address"] = encode_puzzle_hash(get_puzzle_hash_from_key(fingerprint), "thcx")
+    config["simulator"]["farming_address"] = encode_puzzle_hash(get_puzzle_hash_from_key(keychain, fingerprint), "thcx")
     config["simulator"]["plot_directory"] = "test-simulator/plots"
     # save config
     save_config(chinilla_root, "config.yaml", config)
@@ -116,43 +120,53 @@ async def start_simulator(
 
 
 async def get_full_chinilla_simulator(
-    automated_testing: bool = False, chinilla_root: Optional[Path] = None, config: Optional[Dict[str, Any]] = None
-) -> AsyncGenerator[Tuple[FullNodeSimulator, Path, Dict[str, Any], str, int], None]:
+    chinilla_root: Path,
+    keychain: Optional[Keychain] = None,
+    automated_testing: bool = False,
+    config: Optional[Dict[str, Any]] = None,
+) -> AsyncGenerator[Tuple[FullNodeSimulator, Path, Dict[str, Any], str, int, Keychain], None]:
     """
-    A chinilla root directory can be provided, otherwise a temporary one is created.
+    A chinilla root Path is required.
+    The chinilla root Path can be a temporary directory (tempfile.TemporaryDirectory)
+    Passing in a Keychain prevents test keys from being added to the default key location
     This test can either be run in automated mode or not, which determines which mode block tools run in.
     This test is fully interdependent and can be used without the rest of the chinilla test suite.
     Please refer to the documentation for more information.
     """
-    # Create and setup temporary chinilla directories.
-    if chinilla_root is None:
-        chinilla_root = Path(tempfile.TemporaryDirectory().name)
-    mnemonic, fingerprint = mnemonic_fingerprint()
-    ssl_ca_cert_and_key_wrapper: SSLTestCollateralWrapper[
-        SSLTestCACertAndPrivateKey
-    ] = get_next_private_ca_cert_and_key()
-    ssl_nodes_certs_and_keys_wrapper: SSLTestCollateralWrapper[
-        SSLTestNodeCertsAndKeys
-    ] = get_next_nodes_certs_and_keys()
-    if config is None:
-        config = create_config(
-            chinilla_root,
-            fingerprint,
-            ssl_ca_cert_and_key_wrapper.collateral.cert_and_key,
-            ssl_nodes_certs_and_keys_wrapper.collateral.certs_and_keys,
-        )
-    crt_path = chinilla_root / config["daemon_ssl"]["private_crt"]
-    key_path = chinilla_root / config["daemon_ssl"]["private_key"]
-    ca_crt_path = chinilla_root / config["private_ssl_ca"]["crt"]
-    ca_key_path = chinilla_root / config["private_ssl_ca"]["key"]
+
+    if keychain is None:
+        keychain = Keychain()
+
     with Lockfile.create(daemon_launch_lock_path(chinilla_root)):
+
+        mnemonic, fingerprint = mnemonic_fingerprint(keychain)
+
+        ssl_ca_cert_and_key_wrapper: SSLTestCollateralWrapper[
+            SSLTestCACertAndPrivateKey
+        ] = get_next_private_ca_cert_and_key()
+        ssl_nodes_certs_and_keys_wrapper: SSLTestCollateralWrapper[
+            SSLTestNodeCertsAndKeys
+        ] = get_next_nodes_certs_and_keys()
+        if config is None:
+            config = create_config(
+                chinilla_root,
+                fingerprint,
+                ssl_ca_cert_and_key_wrapper.collateral.cert_and_key,
+                ssl_nodes_certs_and_keys_wrapper.collateral.certs_and_keys,
+                keychain,
+            )
+        crt_path = chinilla_root / config["daemon_ssl"]["private_crt"]
+        key_path = chinilla_root / config["daemon_ssl"]["private_key"]
+        ca_crt_path = chinilla_root / config["private_ssl_ca"]["crt"]
+        ca_key_path = chinilla_root / config["private_ssl_ca"]["key"]
+
         shutdown_event = asyncio.Event()
         ws_server = WebSocketServer(chinilla_root, ca_crt_path, ca_key_path, crt_path, key_path, shutdown_event)
         await ws_server.setup_process_global_state()
-        await ws_server.start()  # type: ignore[no-untyped-call]
+        await ws_server.start()
 
         async for simulator in start_simulator(chinilla_root, automated_testing):
-            yield simulator, chinilla_root, config, mnemonic, fingerprint
+            yield simulator, chinilla_root, config, mnemonic, fingerprint, keychain
 
         await ws_server.stop()
         await shutdown_event.wait()  # wait till shutdown is complete
