@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from typing import List
 
 from clvm.casts import int_from_bytes
@@ -6,10 +8,7 @@ from chinilla.types.blockchain_format.coin import Coin
 from chinilla.types.blockchain_format.program import SerializedProgram
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.types.condition_opcodes import ConditionOpcode
-from chinilla.util.condition_tools import (
-    conditions_dict_for_solution,
-    created_outputs_for_conditions_dict,
-)
+from chinilla.util.condition_tools import conditions_dict_for_solution, created_outputs_for_conditions_dict
 
 
 def additions_for_solution(
