@@ -4,9 +4,9 @@ import io
 from dataclasses import dataclass
 from typing import Callable, List, Optional, Tuple
 
-from chinillablspy import G1Element, G2Element
-from chinilla_rs import serialized_length
-from chinillabip158 import PyBIP158
+from blspy import G1Element, G2Element
+from chia_rs import serialized_length
+from chiabip158 import PyBIP158
 
 from chinilla.types.blockchain_format.coin import Coin
 from chinilla.types.blockchain_format.foliage import TransactionsInfo
