@@ -5,9 +5,9 @@ import logging
 import time
 import traceback
 from secrets import token_bytes
-from typing import Any, Dict, List, Optional, Set, Tuple, TYPE_CHECKING
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Set, Tuple
 
-from chinillablspy import AugSchemeMPL, G2Element, G1Element
+from blspy import AugSchemeMPL, G1Element, G2Element
 
 from chinilla.consensus.cost_calculator import NPCResult
 from chinilla.full_node.bundle_tools import simple_solution_generator
@@ -33,20 +33,21 @@ from chinilla.wallet.cat_wallet.cat_utils import (
     match_cat_puzzle,
     unsigned_spend_bundle_for_spendable_cats,
 )
-from chinilla.wallet.puzzles.cat_loader import CAT_MOD
 from chinilla.wallet.cat_wallet.lineage_store import CATLineageStore
 from chinilla.wallet.coin_selection import select_coins
 from chinilla.wallet.derivation_record import DerivationRecord
 from chinilla.wallet.lineage_proof import LineageProof
 from chinilla.wallet.outer_puzzles import AssetType
-from chinilla.wallet.puzzle_drivers import PuzzleInfo
 from chinilla.wallet.payment import Payment
-from chinilla.wallet.puzzles.tails import ALL_LIMITATIONS_PROGRAMS
+from chinilla.wallet.puzzle_drivers import PuzzleInfo
+from chinilla.wallet.puzzles.cat_loader import CAT_MOD
 from chinilla.wallet.puzzles.p2_delegated_puzzle_or_hidden_puzzle import (
     DEFAULT_HIDDEN_PUZZLE_HASH,
     calculate_synthetic_secret_key,
 )
+from chinilla.wallet.puzzles.tails import ALL_LIMITATIONS_PROGRAMS
 from chinilla.wallet.transaction_record import TransactionRecord
+from chinilla.wallet.uncurried_puzzle import uncurry_puzzle
 from chinilla.wallet.util.compute_memos import compute_memos
 from chinilla.wallet.util.curry_and_treehash import calculate_hash_of_quoted_mod_hash, curry_and_treehash
 from chinilla.wallet.util.transaction_type import TransactionType
@@ -54,7 +55,6 @@ from chinilla.wallet.util.wallet_types import AmountWithPuzzlehash, WalletType
 from chinilla.wallet.wallet import Wallet
 from chinilla.wallet.wallet_coin_record import WalletCoinRecord
 from chinilla.wallet.wallet_info import WalletInfo
-from chinilla.wallet.uncurried_puzzle import uncurry_puzzle
 
 if TYPE_CHECKING:
     from chinilla.wallet.wallet_state_manager import WalletStateManager
@@ -453,6 +453,7 @@ class CATWallet:
         exclude: Optional[List[Coin]] = None,
         min_coin_amount: Optional[uint64] = None,
         max_coin_amount: Optional[uint64] = None,
+        excluded_coin_amounts: Optional[List[uint64]] = None,
     ) -> Set[Coin]:
         """
         Returns a set of coins that can be used for generating a new transaction.
@@ -477,6 +478,7 @@ class CATWallet:
             uint128(amount),
             exclude,
             min_coin_amount,
+            excluded_coin_amounts,
         )
         assert sum(c.amount for c in coins) >= amount
         return coins
@@ -540,6 +542,8 @@ class CATWallet:
         amount_to_claim: uint64,
         announcement_to_assert: Optional[Announcement] = None,
         min_coin_amount: Optional[uint64] = None,
+        max_coin_amount: Optional[uint64] = None,
+        exclude_coin_amounts: Optional[List[uint64]] = None,
     ) -> Tuple[TransactionRecord, Optional[Announcement]]:
         """
         This function creates a non-CAT transaction to pay fees, contribute funds for issuance, and absorb melt value.
@@ -548,7 +552,12 @@ class CATWallet:
         """
         announcement = None
         if fee > amount_to_claim:
-            chinilla_coins = await self.standard_wallet.select_coins(fee, min_coin_amount=min_coin_amount)
+            chinilla_coins = await self.standard_wallet.select_coins(
+                fee,
+                min_coin_amount=min_coin_amount,
+                max_coin_amount=max_coin_amount,
+                excluded_coin_amounts=exclude_coin_amounts,
+            )
             origin_id = list(chinilla_coins)[0].name()
             chinilla_tx = await self.standard_wallet.generate_signed_transaction(
                 uint64(0),
@@ -572,7 +581,12 @@ class CATWallet:
             assert message is not None
             announcement = Announcement(origin_id, message)
         else:
-            chinilla_coins = await self.standard_wallet.select_coins(fee, min_coin_amount=min_coin_amount)
+            chinilla_coins = await self.standard_wallet.select_coins(
+                fee,
+                min_coin_amount=min_coin_amount,
+                max_coin_amount=max_coin_amount,
+                excluded_coin_amounts=exclude_coin_amounts,
+            )
             selected_amount = sum([c.amount for c in chinilla_coins])
             chinilla_tx = await self.standard_wallet.generate_signed_transaction(
                 uint64(selected_amount + amount_to_claim - fee),
@@ -594,6 +608,9 @@ class CATWallet:
         coin_announcements_to_consume: Optional[Set[Announcement]] = None,
         puzzle_announcements_to_consume: Optional[Set[Announcement]] = None,
         min_coin_amount: Optional[uint64] = None,
+        max_coin_amount: Optional[uint64] = None,
+        exclude_coin_amounts: Optional[List[uint64]] = None,
+        exclude_coins: Optional[Set[Coin]] = None,
     ) -> Tuple[SpendBundle, Optional[TransactionRecord]]:
         if coin_announcements_to_consume is not None:
             coin_announcements_bytes: Optional[Set[bytes32]] = {a.name() for a in coin_announcements_to_consume}
@@ -613,7 +630,17 @@ class CATWallet:
         starting_amount: int = payment_amount - extra_delta
 
         if coins is None:
-            cat_coins = await self.select_coins(uint64(starting_amount), min_coin_amount=min_coin_amount)
+            if exclude_coins is None:
+                exclude_coins = set()
+            cat_coins = await self.select_coins(
+                uint64(starting_amount),
+                exclude=list(exclude_coins),
+                min_coin_amount=min_coin_amount,
+                max_coin_amount=max_coin_amount,
+                excluded_coin_amounts=exclude_coin_amounts,
+            )
+        elif exclude_coins is not None:
+            raise ValueError("Can't exclude coins when also specifically including coins")
         else:
             cat_coins = coins
 
@@ -663,6 +690,8 @@ class CATWallet:
                             uint64(regular_chinilla_to_claim),
                             announcement_to_assert=announcement,
                             min_coin_amount=min_coin_amount,
+                            max_coin_amount=max_coin_amount,
+                            exclude_coin_amounts=exclude_coin_amounts,
                         )
                         innersol = self.standard_wallet.make_solution(
                             primaries=primaries,
@@ -672,7 +701,11 @@ class CATWallet:
                         )
                     elif regular_chinilla_to_claim > fee:
                         chinilla_tx, _ = await self.create_tandem_hcx_tx(
-                            fee, uint64(regular_chinilla_to_claim), min_coin_amount=min_coin_amount
+                            fee,
+                            uint64(regular_chinilla_to_claim),
+                            min_coin_amount=min_coin_amount,
+                            max_coin_amount=max_coin_amount,
+                            exclude_coin_amounts=exclude_coin_amounts,
                         )
                         innersol = self.standard_wallet.make_solution(
                             primaries=primaries,
@@ -732,6 +765,9 @@ class CATWallet:
         coin_announcements_to_consume: Optional[Set[Announcement]] = None,
         puzzle_announcements_to_consume: Optional[Set[Announcement]] = None,
         min_coin_amount: Optional[uint64] = None,
+        max_coin_amount: Optional[uint64] = None,
+        exclude_coin_amounts: Optional[List[uint64]] = None,
+        exclude_cat_coins: Optional[Set[Coin]] = None,
     ) -> List[TransactionRecord]:
         if memos is None:
             memos = [[] for _ in range(len(puzzle_hashes))]
@@ -749,7 +785,7 @@ class CATWallet:
         if not ignore_max_send_amount:
             max_send = await self.get_max_send_amount()
             if payment_sum > max_send:
-                raise ValueError(f"Can't send more than {max_send} in a single transaction")
+                raise ValueError(f"Can't send more than {max_send} vojos in a single transaction")
         unsigned_spend_bundle, chinilla_tx = await self.generate_unsigned_spendbundle(
             payments,
             fee,
@@ -757,6 +793,9 @@ class CATWallet:
             coin_announcements_to_consume=coin_announcements_to_consume,
             puzzle_announcements_to_consume=puzzle_announcements_to_consume,
             min_coin_amount=min_coin_amount,
+            max_coin_amount=max_coin_amount,
+            exclude_coin_amounts=exclude_coin_amounts,
+            exclude_coins=exclude_cat_coins,
         )
         spend_bundle = await self.sign(unsigned_spend_bundle)
         # TODO add support for array in stored records
@@ -826,7 +865,7 @@ class CATWallet:
         self.wallet_info = wallet_info
         await self.wallet_state_manager.user_store.update_wallet(wallet_info)
 
-    def match_puzzle_info(self, puzzle_driver: PuzzleInfo) -> bool:
+    async def match_puzzle_info(self, puzzle_driver: PuzzleInfo) -> bool:
         return (
             AssetType(puzzle_driver.type()) == AssetType.CAT
             and puzzle_driver["tail"] == bytes.fromhex(self.get_asset_id())
@@ -837,12 +876,16 @@ class CATWallet:
         return PuzzleInfo({"type": AssetType.CAT.value, "tail": "0x" + self.get_asset_id()})
 
     async def get_coins_to_offer(
-        self, asset_id: Optional[bytes32], amount: uint64, min_coin_amount: Optional[uint64] = None
+        self,
+        asset_id: Optional[bytes32],
+        amount: uint64,
+        min_coin_amount: Optional[uint64] = None,
+        max_coin_amount: Optional[uint64] = None,
     ) -> Set[Coin]:
         balance = await self.get_confirmed_balance()
         if balance < amount:
             raise Exception(f"insufficient funds in wallet {self.id()}")
-        return await self.select_coins(amount, min_coin_amount=min_coin_amount)
+        return await self.select_coins(amount, min_coin_amount=min_coin_amount, max_coin_amount=max_coin_amount)
 
 
 if TYPE_CHECKING:
