@@ -1,26 +1,28 @@
-from typing import Tuple, Dict, List, Optional, Any
+from __future__ import annotations
+
+from typing import Any, Dict, List, Optional, Tuple
 
 from chinilla.types.blockchain_format.program import Program
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.types.spend_bundle import SpendBundle
-from chinilla.util.ints import uint64
 from chinilla.util.byte_types import hexstr_to_bytes
-from chinilla.wallet.cat_wallet.lineage_store import CATLineageStore
-from chinilla.wallet.lineage_proof import LineageProof
-from chinilla.wallet.puzzles.load_clvm import load_clvm
+from chinilla.util.ints import uint64
+from chinilla.wallet.cat_wallet.cat_info import CATInfo
 from chinilla.wallet.cat_wallet.cat_utils import (
-    CAT_MOD,
+    SpendableCAT,
     construct_cat_puzzle,
     unsigned_spend_bundle_for_spendable_cats,
-    SpendableCAT,
 )
-from chinilla.wallet.cat_wallet.cat_info import CATInfo
+from chinilla.wallet.cat_wallet.lineage_store import CATLineageStore
+from chinilla.wallet.lineage_proof import LineageProof
+from chinilla.wallet.puzzles.cat_loader import CAT_MOD
+from chinilla.wallet.puzzles.load_clvm import load_clvm_maybe_recompile
 from chinilla.wallet.transaction_record import TransactionRecord
 
-GENESIS_BY_ID_MOD = load_clvm("genesis_by_coin_id.clvm")
-GENESIS_BY_PUZHASH_MOD = load_clvm("genesis_by_puzzle_hash.clvm")
-EVERYTHING_WITH_SIG_MOD = load_clvm("everything_with_signature.clvm")
-DELEGATED_LIMITATIONS_MOD = load_clvm("delegated_tail.clvm")
+GENESIS_BY_ID_MOD = load_clvm_maybe_recompile("genesis_by_coin_id.clvm")
+GENESIS_BY_PUZHASH_MOD = load_clvm_maybe_recompile("genesis_by_puzzle_hash.clvm")
+EVERYTHING_WITH_SIG_MOD = load_clvm_maybe_recompile("everything_with_signature.clvm")
+DELEGATED_LIMITATIONS_MOD = load_clvm_maybe_recompile("delegated_tail.clvm")
 
 
 class LimitationsProgram:
