@@ -2,8 +2,8 @@ from __future__ import annotations
 
 from chinilla.types.blockchain_format.program import Program
 from chinilla.wallet.puzzles import (
-    p2_delegated_puzzle_or_hidden_puzzle,
-)  # import (puzzle_for_pk, puzzle_hash_for_pk, MOD)
+    p2_delegated_puzzle_or_hidden_puzzle,  # import (puzzle_for_pk, puzzle_hash_for_pk, MOD),
+)
 from chinilla.wallet.util.curry_and_treehash import calculate_hash_of_quoted_mod_hash, curry_and_treehash
 
 
