@@ -2,7 +2,7 @@ import pytest
 
 from typing import List, Tuple, Optional
 
-from chinillablspy import AugSchemeMPL, G1Element, G2Element, PrivateKey
+from blspy import AugSchemeMPL, G1Element, G2Element, PrivateKey
 
 from chinilla.types.blockchain_format.program import Program
 from chinilla.types.blockchain_format.sized_bytes import bytes32
