@@ -1,35 +1,49 @@
 # flake8: noqa E402 # See imports after multiprocessing.set_start_method
+import aiohttp
+import datetime
 import multiprocessing
 import os
-from secrets import token_bytes
+import sysconfig
+from typing import Iterator
 
+# TODO: update after resolution in https://github.com/pytest-dev/pytest/issues/7469
+from _pytest.fixtures import SubRequest
 import pytest
 import pytest_asyncio
 import tempfile
 
-from tests.setup_nodes import setup_node_and_wallet, setup_n_nodes, setup_two_nodes
-from pathlib import Path
-from typing import Any, AsyncIterator, Dict, List, Tuple
+
+from typing import Any, AsyncIterator, Dict, List, Tuple, Union
 from chinilla.server.start_service import Service
 
 # Set spawn after stdlib imports, but before other imports
 from chinilla.clvm.spend_sim import SimClient, SpendSim
+from chinilla.full_node.full_node_api import FullNodeAPI
 from chinilla.protocols import full_node_protocol
-from chinilla.simulator.simulator_protocol import FarmNewBlockProtocol
-from chinilla.types.blockchain_format.sized_bytes import bytes32
+from chinilla.server.server import ChinillaServer
+from chinilla.simulator.full_node_simulator import FullNodeSimulator
+
+
 from chinilla.types.peer_info import PeerInfo
 from chinilla.util.config import create_default_chinilla_config, lock_and_load_config
 from chinilla.util.ints import uint16
+from chinilla.simulator.setup_services import setup_daemon, setup_introducer, setup_timelord
+from chinilla.util.ints import uint16, uint64
+from chinilla.util.task_timing import (
+    main as task_instrumentation_main,
+    start_task_instrumentation,
+    stop_task_instrumentation,
+)
+from chinilla.wallet.wallet import Wallet
+from tests.core.data_layer.util import ChinillaRoot
 from tests.core.node_height import node_height_at_least
-from tests.setup_nodes import (
+from chinilla.simulator.setup_nodes import (
     setup_simulators_and_wallets,
     setup_node_and_wallet,
-    setup_full_system,
-    setup_daemon,
     setup_n_nodes,
-    setup_introducer,
-    setup_timelord,
     setup_two_nodes,
+    setup_simulators_and_wallets_service,
+    setup_full_system_connect_to_deamon,
 )
 from tests.simulation.test_simulation import test_constants_modified
 from chinilla.simulator.time_out_assert import time_out_assert
@@ -41,8 +55,29 @@ multiprocessing.set_start_method("spawn")
 from pathlib import Path
 from chinilla.util.keyring_wrapper import KeyringWrapper
 from chinilla.simulator.block_tools import BlockTools, test_constants, create_block_tools, create_block_tools_async
-from tests.util.keyring import TempKeyring
-from tests.setup_nodes import setup_farmer_multi_harvester
+from chinilla.simulator.keyring import TempKeyring
+from chinilla.simulator.setup_nodes import setup_farmer_multi_harvester
+
+
+@pytest.fixture(name="node_name_for_file")
+def node_name_for_file_fixture(request: SubRequest) -> str:
+    # TODO: handle other characters banned on windows
+    return request.node.name.replace(os.sep, "_")
+
+
+@pytest.fixture(name="test_time_for_file")
+def test_time_for_file_fixture(request: SubRequest) -> str:
+    return datetime.datetime.now().isoformat().replace(":", "_")
+
+
+@pytest.fixture(name="task_instrumentation")
+def task_instrumentation_fixture(node_name_for_file: str, test_time_for_file: str) -> Iterator[None]:
+    target_directory = f"task-profile-{node_name_for_file}-{test_time_for_file}"
+
+    start_task_instrumentation()
+    yield
+    stop_task_instrumentation(target_dir=target_directory)
+    task_instrumentation_main(args=[target_directory])
 
 
 @pytest.fixture(scope="session")
@@ -81,7 +116,7 @@ async def empty_blockchain(request):
     Provides a list of 10 valid blocks, as well as a blockchain with 9 blocks added to it.
     """
     from tests.util.blockchain import create_blockchain
-    from tests.setup_nodes import test_constants
+    from chinilla.simulator.setup_nodes import test_constants
 
     bc1, db_wrapper, db_path = await create_blockchain(test_constants, request.param)
     yield bc1
@@ -91,6 +126,11 @@ async def empty_blockchain(request):
     db_path.unlink()
 
 
+@pytest.fixture(scope="function")
+def latest_db_version():
+    return 2
+
+
 @pytest.fixture(scope="function", params=[1, 2])
 def db_version(request):
     return request.param
@@ -288,11 +328,25 @@ async def two_nodes_sim_and_wallets():
 
 
 @pytest_asyncio.fixture(scope="function")
-async def wallet_node_sim_and_wallet():
+async def two_nodes_sim_and_wallets_services():
+    async for _ in setup_simulators_and_wallets_service(2, 0, {}):
+        yield _
+
+
+@pytest_asyncio.fixture(scope="function")
+async def wallet_node_sim_and_wallet() -> AsyncIterator[
+    Tuple[List[Union[FullNodeAPI, FullNodeSimulator]], List[Tuple[Wallet, ChinillaServer]], BlockTools],
+]:
     async for _ in setup_simulators_and_wallets(1, 1, {}):
         yield _
 
 
+@pytest_asyncio.fixture(scope="function")
+async def one_wallet_and_one_simulator_services():
+    async for _ in setup_simulators_and_wallets_service(1, 1, {}):
+        yield _
+
+
 @pytest_asyncio.fixture(scope="function")
 async def wallet_node_100_pk():
     async for _ in setup_simulators_and_wallets(1, 1, {}, initial_num_public_keys=100):
@@ -308,6 +362,12 @@ async def two_wallet_nodes(request):
         yield _
 
 
+@pytest_asyncio.fixture(scope="function")
+async def two_wallet_nodes_services() -> AsyncIterator[Tuple[List[Service], List[FullNodeSimulator], BlockTools]]:
+    async for _ in setup_simulators_and_wallets_service(1, 2, {}):
+        yield _
+
+
 @pytest_asyncio.fixture(scope="function")
 async def two_wallet_nodes_custom_spam_filtering(spam_filter_after_n_txs, hcx_spam_amount):
     async for _ in setup_simulators_and_wallets(1, 2, {}, spam_filter_after_n_txs, hcx_spam_amount):
@@ -408,11 +468,14 @@ async def wallet_and_node():
 
 
 @pytest_asyncio.fixture(scope="function")
-async def one_node_one_block(wallet_a):
+async def one_node_one_block() -> AsyncIterator[
+    Tuple[Union[FullNodeAPI, FullNodeSimulator], ChinillaServer, BlockTools]
+]:
     async_gen = setup_simulators_and_wallets(1, 0, {})
     nodes, _, bt = await async_gen.__anext__()
     full_node_1 = nodes[0]
     server_1 = full_node_1.full_node.server
+    wallet_a = bt.get_pool_wallet_tool()
 
     reward_ph = wallet_a.get_new_puzzlehash()
     blocks = bt.get_consecutive_blocks(
@@ -420,7 +483,7 @@ async def one_node_one_block(wallet_a):
         guarantee_transaction_block=True,
         farmer_reward_puzzle_hash=reward_ph,
         pool_reward_puzzle_hash=reward_ph,
-        genesis_timestamp=10000,
+        genesis_timestamp=uint64(10000),
         time_per_block=10,
     )
     assert blocks[0].height == 0
@@ -437,13 +500,14 @@ async def one_node_one_block(wallet_a):
 
 
 @pytest_asyncio.fixture(scope="function")
-async def two_nodes_one_block(wallet_a):
+async def two_nodes_one_block():
     async_gen = setup_simulators_and_wallets(2, 0, {})
     nodes, _, bt = await async_gen.__anext__()
     full_node_1 = nodes[0]
     full_node_2 = nodes[1]
     server_1 = full_node_1.full_node.server
     server_2 = full_node_2.full_node.server
+    wallet_a = bt.get_pool_wallet_tool()
 
     reward_ph = wallet_a.get_new_puzzlehash()
     blocks = bt.get_consecutive_blocks(
@@ -503,12 +567,11 @@ async def farmer_three_harvester_not_started(
 # more than one simulations per process.
 @pytest_asyncio.fixture(scope="function")
 async def daemon_simulation(bt, get_b_tools, get_b_tools_1):
-    async for _ in setup_full_system(
+    async for _ in setup_full_system_connect_to_deamon(
         test_constants_modified,
         bt,
         b_tools=get_b_tools,
         b_tools_1=get_b_tools_1,
-        connect_to_daemon=True,
         db_version=1,
     ):
         yield _, get_b_tools, get_b_tools_1
@@ -520,6 +583,13 @@ async def get_daemon(bt):
         yield _
 
 
+@pytest.fixture(scope="function")
+def empty_keyring():
+    with TempKeyring(user="user-chinilla-1.8", service="chinilla-user-chinilla-1.8") as keychain:
+        yield keychain
+        KeyringWrapper.cleanup_shared_instance()
+
+
 @pytest_asyncio.fixture(scope="function")
 async def get_temp_keyring():
     with TempKeyring() as keychain:
@@ -540,9 +610,19 @@ async def get_b_tools(get_temp_keyring):
 
 
 @pytest_asyncio.fixture(scope="function")
-async def get_daemon_with_temp_keyring(get_b_tools):
+async def daemon_connection_and_temp_keychain(get_b_tools):
     async for daemon in setup_daemon(btools=get_b_tools):
-        yield get_b_tools, daemon
+        keychain = daemon.keychain_server._default_keychain
+        async with aiohttp.ClientSession() as session:
+            async with session.ws_connect(
+                f"wss://127.0.0.1:{get_b_tools._config['daemon_port']}",
+                autoclose=True,
+                autoping=True,
+                heartbeat=60,
+                ssl=get_b_tools.get_daemon_ssl_context(),
+                max_msg_size=52428800,
+            ) as ws:
+                yield ws, keychain
 
 
 @pytest_asyncio.fixture(scope="function")
@@ -550,8 +630,8 @@ async def wallets_prefarm(two_wallet_nodes, self_hostname, trusted):
     """
     Sets up the node with 10 blocks, and returns a payer and payee wallet.
     """
-    farm_blocks = 10
-    buffer = 4
+    farm_blocks = 3
+    buffer = 1
     full_nodes, wallets, _ = two_wallet_nodes
     full_node_api = full_nodes[0]
     full_node_server = full_node_api.server
@@ -560,9 +640,6 @@ async def wallets_prefarm(two_wallet_nodes, self_hostname, trusted):
     wallet_0 = wallet_node_0.wallet_state_manager.main_wallet
     wallet_1 = wallet_node_1.wallet_state_manager.main_wallet
 
-    ph0 = await wallet_0.get_new_puzzlehash()
-    ph1 = await wallet_1.get_new_puzzlehash()
-
     if trusted:
         wallet_node_0.config["trusted_peers"] = {full_node_server.node_id.hex(): full_node_server.node_id.hex()}
         wallet_node_1.config["trusted_peers"] = {full_node_server.node_id.hex(): full_node_server.node_id.hex()}
@@ -573,29 +650,41 @@ async def wallets_prefarm(two_wallet_nodes, self_hostname, trusted):
     await wallet_server_0.start_client(PeerInfo(self_hostname, uint16(full_node_server._port)), None)
     await wallet_server_1.start_client(PeerInfo(self_hostname, uint16(full_node_server._port)), None)
 
-    for i in range(0, farm_blocks):
-        await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph0))
-
-    for i in range(0, farm_blocks):
-        await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph1))
-
-    for i in range(0, buffer):
-        await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(bytes32(token_bytes(nbytes=32))))
+    wallet_0_rewards = await full_node_api.farm_blocks(count=farm_blocks, wallet=wallet_0)
+    wallet_1_rewards = await full_node_api.farm_blocks(count=farm_blocks, wallet=wallet_1)
+    await full_node_api.process_blocks(count=buffer)
 
     await time_out_assert(30, wallet_is_synced, True, wallet_node_0, full_node_api)
     await time_out_assert(30, wallet_is_synced, True, wallet_node_1, full_node_api)
 
-    return wallet_node_0, wallet_node_1, full_node_api
+    assert await wallet_0.get_confirmed_balance() == wallet_0_rewards
+    assert await wallet_0.get_unconfirmed_balance() == wallet_0_rewards
+    assert await wallet_1.get_confirmed_balance() == wallet_1_rewards
+    assert await wallet_1.get_unconfirmed_balance() == wallet_1_rewards
+
+    return (wallet_node_0, wallet_0_rewards), (wallet_node_1, wallet_1_rewards), full_node_api
 
 
 @pytest_asyncio.fixture(scope="function")
 async def introducer(bt):
+    async for service in setup_introducer(bt, 0):
+        yield service._api, service._node.server
+
+
+@pytest_asyncio.fixture(scope="function")
+async def introducer_service(bt):
     async for _ in setup_introducer(bt, 0):
         yield _
 
 
 @pytest_asyncio.fixture(scope="function")
 async def timelord(bt):
+    async for service in setup_timelord(uint16(0), False, test_constants, bt):
+        yield service._api, service._node.server
+
+
+@pytest_asyncio.fixture(scope="function")
+async def timelord_service(bt):
     async for _ in setup_timelord(uint16(0), False, test_constants, bt):
         yield _
 
@@ -631,8 +720,25 @@ def root_path_populated_with_config(tmp_chinilla_root) -> Path:
 
 @pytest.fixture(scope="function")
 def config_with_address_prefix(root_path_populated_with_config: Path, prefix: str) -> Dict[str, Any]:
-    updated_config: Dict[str, Any] = {}
     with lock_and_load_config(root_path_populated_with_config, "config.yaml") as config:
         if prefix is not None:
             config["network_overrides"]["config"][config["selected_network"]]["address_prefix"] = prefix
     return config
+
+
+@pytest.fixture(name="scripts_path", scope="session")
+def scripts_path_fixture() -> Path:
+    scripts_string = sysconfig.get_path("scripts")
+    if scripts_string is None:
+        raise Exception("These tests depend on the scripts path existing")
+
+    return Path(scripts_string)
+
+
+@pytest.fixture(name="chinilla_root", scope="function")
+def chinilla_root_fixture(tmp_path: Path, scripts_path: Path) -> ChinillaRoot:
+    root = ChinillaRoot(path=tmp_path.joinpath("chinilla_root"), scripts_path=scripts_path)
+    root.run(args=["init"])
+    root.run(args=["configure", "--set-log-level", "INFO"])
+
+    return root
