@@ -1,19 +1,19 @@
-import os
-import pytest
-import re
+from __future__ import annotations
 
-from chinilla.cmds.chinilla import cli
-from chinilla.cmds.keys import delete_all_cmd, generate_and_print_cmd, show_cmd, sign_cmd, verify_cmd
-from chinilla.util.config import load_config
-from chinilla.util.file_keyring import FileKeyring
-from chinilla.util.keychain import KeyData, DEFAULT_USER, DEFAULT_SERVICE, Keychain, generate_mnemonic
-from chinilla.util.keyring_wrapper import DEFAULT_KEYS_ROOT_PATH, KeyringWrapper, LegacyKeyring
-from click.testing import CliRunner, Result
-from keyring.backend import KeyringBackend
+import json
+import os
+import re
 from pathlib import Path
-from tests.util.keyring import TempKeyring
 from typing import Dict, List, Optional
 
+import pytest
+from click.testing import CliRunner, Result
+
+from chinilla.cmds.chinilla import cli
+from chinilla.cmds.keys import delete_all_cmd, generate_and_print_cmd, sign_cmd, verify_cmd
+from chinilla.util.config import load_config
+from chinilla.util.keychain import Keychain, KeyData, generate_mnemonic
+from chinilla.util.keyring_wrapper import DEFAULT_KEYS_ROOT_PATH, KeyringWrapper
 
 TEST_MNEMONIC_SEED = (
     "grief lock ketchup video day owner torch young work "
@@ -23,53 +23,6 @@ TEST_MNEMONIC_SEED = (
 TEST_FINGERPRINT = 2877570395
 
 
-class DummyLegacyKeyring(KeyringBackend):
-
-    # Fingerprint 2474840988
-    KEY_0 = (
-        "89e29e5f9c3105b2a853475cab2392468cbfb1d65c3faabea8ebc78fe903fd279e56a8d93f6325fc6c3d833a2ae74832"
-        "b8feaa3d6ee49998f43ce303b66dcc5abb633e5c1d80efe85c40766135e4a44c"
-    )
-
-    # Fingerprint 4149609062
-    KEY_1 = (
-        "8b0d72288727af6238fcd9b0a663cd7d4728738fca597d0046cbb42b6432e0a5ae8026683fc5f9c73df26fb3e1cec2c8"
-        "ad1b4f601107d96a99f6fa9b9d2382918fb1e107fb6655c7bdd8c77c1d9c201f"
-    )
-
-    # Fingerprint 3618811800
-    KEY_2 = (
-        "8b2a26ba319f83bd3da5b1b147a817ecc4ca557f037c9db1cfedc59b16ee6880971b7d292f023358710a292c8db0eb82"
-        "35808f914754ae24e493fad9bc7f654b0f523fb406973af5235256a39bed1283"
-    )
-
-    def __init__(self, populate: bool = True):
-        self.service_dict = {}
-
-        if populate:
-            self.service_dict[DEFAULT_SERVICE] = {
-                f"wallet-{DEFAULT_USER}-0": DummyLegacyKeyring.KEY_0,
-                f"wallet-{DEFAULT_USER}-1": DummyLegacyKeyring.KEY_1,
-                f"wallet-{DEFAULT_USER}-2": DummyLegacyKeyring.KEY_2,
-            }
-
-    def get_password(self, service, username, password=None):
-        return self.service_dict.get(service, {}).get(username)
-
-    def set_password(self, service, username, password):
-        self.service_dict.setdefault(service, {})[username] = password
-
-    def delete_password(self, service, username):
-        del self.service_dict[service][username]
-
-
-@pytest.fixture(scope="function")
-def empty_keyring():
-    with TempKeyring(user="user-chinilla-1.8", service="chinilla-user-chinilla-1.8") as keychain:
-        yield keychain
-        KeyringWrapper.cleanup_shared_instance()
-
-
 @pytest.fixture(scope="function")
 def keyring_with_one_key(empty_keyring):
     keychain = empty_keyring
@@ -95,22 +48,6 @@ def setup_keyringwrapper(tmp_path):
     KeyringWrapper.set_keys_root_path(DEFAULT_KEYS_ROOT_PATH)
 
 
-@pytest.fixture(scope="function")
-def setup_legacy_keyringwrapper(tmp_path, monkeypatch):
-    def mock_setup_keyring_file_watcher(_):
-        pass
-
-    # Silence errors in the watchdog module during testing
-    monkeypatch.setattr(FileKeyring, "setup_keyring_file_watcher", mock_setup_keyring_file_watcher)
-
-    KeyringWrapper.cleanup_shared_instance()
-    KeyringWrapper.set_keys_root_path(tmp_path)
-    KeyringWrapper.get_shared_instance().legacy_keyring = DummyLegacyKeyring()
-    yield
-    KeyringWrapper.cleanup_shared_instance()
-    KeyringWrapper.set_keys_root_path(DEFAULT_KEYS_ROOT_PATH)
-
-
 def assert_label(keychain: Keychain, label: Optional[str], index: int) -> None:
     all_keys = keychain.get_keys()
     assert len(all_keys) > index
@@ -141,7 +78,6 @@ class TestKeysCommands:
         result: Result = runner.invoke(
             cli,
             [
-                "--no-force-legacy-keyring-migration",
                 "--root-path",
                 os.fspath(tmp_path),
                 "--keys-root-path",
@@ -187,7 +123,6 @@ class TestKeysCommands:
         generate_result: Result = runner.invoke(
             cli,
             [
-                "--no-force-legacy-keyring-migration",
                 "--root-path",
                 os.fspath(tmp_path),
                 "--keys-root-path",
@@ -215,7 +150,6 @@ class TestKeysCommands:
         result: Result = runner.invoke(
             cli,
             [
-                "--no-force-legacy-keyring-migration",
                 "--root-path",
                 os.fspath(tmp_path),
                 "--keys-root-path",
@@ -257,7 +191,6 @@ class TestKeysCommands:
         keychain = empty_keyring
         keys_root_path = keychain.keyring_wrapper.keys_root_path
         base_params = [
-            "--no-force-legacy-keyring-migration",
             "--root-path",
             os.fspath(tmp_path),
             "--keys-root-path",
@@ -275,7 +208,6 @@ class TestKeysCommands:
         keychain = keyring_with_one_key
         keys_root_path = keychain.keyring_wrapper.keys_root_path
         base_params = [
-            "--no-force-legacy-keyring-migration",
             "--root-path",
             os.fspath(tmp_path),
             "--keys-root-path",
@@ -303,7 +235,6 @@ class TestKeysCommands:
         keychain = keyring_with_one_key
         keys_root_path = keychain.keyring_wrapper.keys_root_path
         base_params = [
-            "--no-force-legacy-keyring-migration",
             "--root-path",
             os.fspath(tmp_path),
             "--keys-root-path",
@@ -328,7 +259,6 @@ class TestKeysCommands:
         runner = CliRunner()
         keys_root_path = keychain.keyring_wrapper.keys_root_path
         base_params = [
-            "--no-force-legacy-keyring-migration",
             "--root-path",
             os.fspath(tmp_path),
             "--keys-root-path",
@@ -358,7 +288,7 @@ class TestKeysCommands:
             else:
                 assert label == key.label
 
-    def test_show(self, keyring_with_one_key):
+    def test_show(self, keyring_with_one_key, tmp_path):
         """
         Test that the `chinilla keys show` command shows the correct key.
         """
@@ -367,13 +297,82 @@ class TestKeysCommands:
 
         assert len(keychain.get_all_private_keys()) == 1
 
+        keys_root_path = keychain.keyring_wrapper.keys_root_path
+        base_params = [
+            "--root-path",
+            os.fspath(tmp_path),
+            "--keys-root-path",
+            os.fspath(keys_root_path),
+        ]
         runner = CliRunner()
-        result: Result = runner.invoke(show_cmd, [])
+        cmd_params = ["keys", "show"]
+        # Generate a new config
+        assert runner.invoke(cli, [*base_params, "init"]).exit_code == 0
+        # Run the command
+        result: Result = runner.invoke(cli, [*base_params, *cmd_params])
 
         # assert result.exit_code == 0
-        assert result.output.find(f"Fingerprint: {TEST_FINGERPRINT}") != 0
+        assert result.output.find(f"Fingerprint: {TEST_FINGERPRINT}") != -1
 
-    def test_show_mnemonic(self, keyring_with_one_key):
+    def test_show_fingerprint(self, keyring_with_one_key, tmp_path):
+        """
+        Test that the `chinilla keys show --fingerprint` command shows the correct key.
+        """
+
+        keychain = keyring_with_one_key
+
+        # add a key
+        keychain.add_private_key(generate_mnemonic())
+        assert len(keychain.get_all_private_keys()) == 2
+
+        keys_root_path = keychain.keyring_wrapper.keys_root_path
+        base_params = [
+            "--root-path",
+            os.fspath(tmp_path),
+            "--keys-root-path",
+            os.fspath(keys_root_path),
+        ]
+        runner = CliRunner()
+        cmd_params = ["keys", "show", "--fingerprint", TEST_FINGERPRINT]
+        # Generate a new config
+        assert runner.invoke(cli, [*base_params, "init"]).exit_code == 0
+        # Run the command
+        result: Result = runner.invoke(cli, [*base_params, *cmd_params])
+
+        assert result.exit_code == 0
+        fingerprints = [line for line in result.output.splitlines() if "Fingerprint:" in line]
+        assert len(fingerprints) == 1
+        assert str(TEST_FINGERPRINT) in fingerprints[0]
+
+    def test_show_json(self, keyring_with_one_key, tmp_path):
+        """
+        Test that the `chinilla keys show --json` command shows the correct key.
+        """
+
+        keychain = keyring_with_one_key
+
+        assert len(keychain.get_all_private_keys()) == 1
+
+        keys_root_path = keychain.keyring_wrapper.keys_root_path
+        base_params = [
+            "--root-path",
+            os.fspath(tmp_path),
+            "--keys-root-path",
+            os.fspath(keys_root_path),
+        ]
+        runner = CliRunner()
+        cmd_params = ["keys", "show", "--json"]
+        # Generate a new config
+        assert runner.invoke(cli, [*base_params, "init"]).exit_code == 0
+        # Run the command
+        result: Result = runner.invoke(cli, [*base_params, *cmd_params])
+
+        json_result = json.loads(result.output)
+
+        # assert result.exit_code == 0
+        assert json_result["keys"][0]["fingerprint"] == TEST_FINGERPRINT
+
+    def test_show_mnemonic(self, keyring_with_one_key, tmp_path):
         """
         Test that the `chinilla keys show --show-mnemonic-seed` command shows the key's mnemonic seed.
         """
@@ -382,13 +381,52 @@ class TestKeysCommands:
 
         assert len(keychain.get_all_private_keys()) == 1
 
+        keys_root_path = keychain.keyring_wrapper.keys_root_path
+        base_params = [
+            "--root-path",
+            os.fspath(tmp_path),
+            "--keys-root-path",
+            os.fspath(keys_root_path),
+        ]
         runner = CliRunner()
-        result: Result = runner.invoke(show_cmd, ["--show-mnemonic-seed"])
+        cmd_params = ["keys", "show", "--show-mnemonic-seed"]
+        # Generate a new config
+        assert runner.invoke(cli, [*base_params, "init"]).exit_code == 0
+        # Run the command
+        result: Result = runner.invoke(cli, [*base_params, *cmd_params])
 
         # assert result.exit_code == 0
-        assert result.output.find(f"Fingerprint: {TEST_FINGERPRINT}") != 0
-        assert result.output.find("Mnemonic: seed (24 secret words):") != 0
-        assert result.output.find(TEST_MNEMONIC_SEED) != 0
+        assert result.output.find(f"Fingerprint: {TEST_FINGERPRINT}") != -1
+        assert result.output.find("Mnemonic seed (24 secret words):") != -1
+        assert result.output.find(TEST_MNEMONIC_SEED) != -1
+
+    def test_show_mnemonic_json(self, keyring_with_one_key, tmp_path):
+        """
+        Test that the `chinilla keys show --show-mnemonic-seed --json` command shows the key's mnemonic seed.
+        """
+
+        keychain = keyring_with_one_key
+
+        assert len(keychain.get_all_private_keys()) == 1
+
+        keys_root_path = keychain.keyring_wrapper.keys_root_path
+        base_params = [
+            "--root-path",
+            os.fspath(tmp_path),
+            "--keys-root-path",
+            os.fspath(keys_root_path),
+        ]
+        runner = CliRunner()
+        cmd_params = ["keys", "show", "--show-mnemonic-seed", "--json"]
+        # Generate a new config
+        assert runner.invoke(cli, [*base_params, "init"]).exit_code == 0
+        # Run the command
+        result: Result = runner.invoke(cli, [*base_params, *cmd_params])
+        json_result = json.loads(result.output)
+
+        # assert result.exit_code == 0
+        assert json_result["keys"][0]["fingerprint"] == TEST_FINGERPRINT
+        assert json_result["keys"][0]["mnemonic"] == TEST_MNEMONIC_SEED
 
     def test_add_interactive(self, tmp_path, empty_keyring):
         """
@@ -410,7 +448,6 @@ class TestKeysCommands:
         result: Result = runner.invoke(
             cli,
             [
-                "--no-force-legacy-keyring-migration",
                 "--root-path",
                 os.fspath(tmp_path),
                 "--keys-root-path",
@@ -444,7 +481,6 @@ class TestKeysCommands:
         result: Result = runner.invoke(
             cli,
             [
-                "--no-force-legacy-keyring-migration",
                 "--root-path",
                 os.fspath(tmp_path),
                 "--keys-root-path",
@@ -480,7 +516,6 @@ class TestKeysCommands:
         add_result: Result = runner.invoke(
             cli,
             [
-                "--no-force-legacy-keyring-migration",
                 "--root-path",
                 os.fspath(tmp_path),
                 "--keys-root-path",
@@ -500,7 +535,6 @@ class TestKeysCommands:
         result: Result = runner.invoke(
             cli,
             [
-                "--no-force-legacy-keyring-migration",
                 "--root-path",
                 os.fspath(tmp_path),
                 "--keys-root-path",
@@ -545,7 +579,7 @@ class TestKeysCommands:
         result: Result = runner.invoke(generate_and_print_cmd, [])
 
         assert result.exit_code == 0
-        assert result.output.find("Mnemonic (24 secret words):") != 0
+        assert result.output.find("Mnemonic (24 secret words):") != -1
 
     def test_sign(self, keyring_with_one_key):
         """
@@ -693,7 +727,6 @@ class TestKeysCommands:
         result: Result = runner.invoke(
             cli,
             [
-                "--no-force-legacy-keyring-migration",
                 "--root-path",
                 os.fspath(tmp_path),
                 "--keys-root-path",
@@ -752,7 +785,6 @@ class TestKeysCommands:
         result: Result = runner.invoke(
             cli,
             [
-                "--no-force-legacy-keyring-migration",
                 "--root-path",
                 os.fspath(tmp_path),
                 "--keys-root-path",
@@ -801,7 +833,6 @@ class TestKeysCommands:
         result: Result = runner.invoke(
             cli,
             [
-                "--no-force-legacy-keyring-migration",
                 "--root-path",
                 os.fspath(tmp_path),
                 "--keys-root-path",
@@ -891,7 +922,6 @@ class TestKeysCommands:
         result: Result = runner.invoke(
             cli,
             [
-                "--no-force-legacy-keyring-migration",
                 "--root-path",
                 os.fspath(tmp_path),
                 "--keys-root-path",
@@ -942,7 +972,6 @@ class TestKeysCommands:
         result: Result = runner.invoke(
             cli,
             [
-                "--no-force-legacy-keyring-migration",
                 "--root-path",
                 os.fspath(tmp_path),
                 "--keys-root-path",
@@ -1001,7 +1030,6 @@ class TestKeysCommands:
         result: Result = runner.invoke(
             cli,
             [
-                "--no-force-legacy-keyring-migration",
                 "--root-path",
                 os.fspath(tmp_path),
                 "--keys-root-path",
@@ -1062,7 +1090,6 @@ class TestKeysCommands:
         result: Result = runner.invoke(
             cli,
             [
-                "--no-force-legacy-keyring-migration",
                 "--root-path",
                 os.fspath(tmp_path),
                 "--keys-root-path",
@@ -1120,115 +1147,3 @@ class TestKeysCommands:
             )
             != -1
         )
-
-    def test_migration_not_needed(self, tmp_path, setup_keyringwrapper, monkeypatch):
-        """
-        Test the `chinilla keys migrate` command when no migration is necessary
-        """
-        keys_root_path = KeyringWrapper.get_shared_instance().keys_root_path
-        runner = CliRunner()
-        init_result = runner.invoke(
-            cli, ["--root-path", os.fspath(tmp_path), "--keys-root-path", os.fspath(keys_root_path), "init"]
-        )
-        assert init_result.exit_code == 0
-
-        def mock_keychain_needs_migration() -> bool:
-            return False
-
-        monkeypatch.setattr(Keychain, "needs_migration", mock_keychain_needs_migration)
-
-        runner = CliRunner()
-        result: Result = runner.invoke(
-            cli,
-            [
-                "--root-path",
-                os.fspath(tmp_path),
-                "keys",
-                "migrate",
-            ],
-        )
-
-        assert result.exit_code == 0
-        assert result.output.find("No keys need migration") != -1
-
-    def test_migration_full(self, tmp_path, setup_legacy_keyringwrapper):
-        """
-        Test the `chinilla keys migrate` command when a full migration is needed
-        """
-
-        legacy_keyring = KeyringWrapper.get_shared_instance().legacy_keyring
-
-        assert legacy_keyring is not None
-        assert len(legacy_keyring.service_dict[DEFAULT_SERVICE]) == 3
-
-        runner = CliRunner()
-        init_result: Result = runner.invoke(
-            cli,
-            ["--root-path", os.fspath(tmp_path), "init"],
-        )
-
-        assert init_result.exit_code == 0
-
-        runner = CliRunner()
-        result: Result = runner.invoke(
-            cli,
-            [
-                "--root-path",
-                os.fspath(tmp_path),
-                "keys",
-                "migrate",
-            ],
-            input="n\ny\ny\n",  # Prompts: 'n' = don't set a passphrase, 'y' = begin migration, 'y' = remove legacy keys
-        )
-
-        assert result.exit_code == 0
-        assert KeyringWrapper.get_shared_instance().using_legacy_keyring() is False  # legacy keyring unset
-        assert type(KeyringWrapper.get_shared_instance().keyring) is FileKeyring  # new keyring set
-        assert len(Keychain().get_all_public_keys()) == 3  # new keyring has 3 keys
-        assert len(legacy_keyring.service_dict[DEFAULT_SERVICE]) == 0  # legacy keys removed
-
-    def test_migration_incremental(self, tmp_path, keyring_with_one_key, monkeypatch):
-        KeyringWrapper.set_keys_root_path(tmp_path)
-        KeyringWrapper.cleanup_shared_instance()
-
-        keychain = keyring_with_one_key
-        legacy_keyring = DummyLegacyKeyring()
-
-        def mock_get_legacy_keyring_instance() -> Optional[LegacyKeyring]:
-            nonlocal legacy_keyring
-            return legacy_keyring
-
-        from chinilla.util import keyring_wrapper
-
-        monkeypatch.setattr(keyring_wrapper, "get_legacy_keyring_instance", mock_get_legacy_keyring_instance)
-
-        assert len(keychain.get_all_private_keys()) == 1
-        assert keychain.keyring_wrapper.legacy_keyring is None
-        assert legacy_keyring is not None
-        assert len(legacy_keyring.service_dict[DEFAULT_SERVICE]) == 3
-
-        runner = CliRunner()
-        init_result: Result = runner.invoke(
-            cli,
-            ["--root-path", os.fspath(tmp_path), "init"],
-        )
-
-        assert init_result.exit_code == 0
-
-        runner = CliRunner()
-        result: Result = runner.invoke(
-            cli,
-            [
-                "--root-path",
-                os.fspath(tmp_path),
-                "keys",
-                "migrate",
-            ],
-            input="y\ny\n",  # Prompts: 'y' = migrate keys, 'y' = remove legacy keys
-        )
-
-        assert result.exit_code == 0
-        assert KeyringWrapper.get_shared_instance().using_legacy_keyring() is False  # legacy keyring is not set
-        assert type(KeyringWrapper.get_shared_instance().keyring) is FileKeyring  # new keyring set
-        assert len(Keychain().get_all_public_keys()) == 4  # new keyring has 4 keys
-        assert len(legacy_keyring.service_dict[DEFAULT_SERVICE]) == 0  # legacy keys removed
