@@ -3,17 +3,18 @@ These are quick-to-run test that check spends can be added to the blockchain whe
 or that they're failing for the right reason when they're invalid.
 """
 
+from __future__ import annotations
+
 import logging
 import time
-
 from typing import List, Optional, Tuple
 
 import pytest
-
-from chinillablspy import G2Element
-
+from blspy import G2Element
 from clvm_tools.binutils import assemble
 
+from chinilla.simulator.block_tools import BlockTools
+from chinilla.simulator.keyring import TempKeyring
 from chinilla.types.announcement import Announcement
 from chinilla.types.blockchain_format.program import Program
 from chinilla.types.coin_record import CoinRecord
@@ -23,11 +24,9 @@ from chinilla.types.full_block import FullBlock
 from chinilla.types.spend_bundle import SpendBundle
 from chinilla.util.errors import Err
 from chinilla.util.ints import uint32
-from chinilla.simulator.block_tools import BlockTools
-from tests.util.keyring import TempKeyring
 
-from .ram_db import create_ram_blockchain
 from ...blockchain.blockchain_test_utils import _validate_and_add_block
+from .ram_db import create_ram_blockchain
 
 
 def cleanup_keyring(keyring: TempKeyring):
