@@ -1,4 +1,6 @@
 # flake8: noqa: F811, F401
+from __future__ import annotations
+
 import asyncio
 import logging
 
@@ -9,14 +11,13 @@ from chinilla.full_node.full_node_api import FullNodeAPI
 from chinilla.protocols import full_node_protocol
 from chinilla.protocols.protocol_message_types import ProtocolMessageTypes
 from chinilla.protocols.shared_protocol import Handshake
-from chinilla.server.outbound_message import make_msg, Message
+from chinilla.server.outbound_message import Message, make_msg
 from chinilla.server.rate_limits import RateLimiter
-from chinilla.server.server import ssl_context_for_client
 from chinilla.server.ws_connection import WSChinillaConnection
+from chinilla.simulator.time_out_assert import time_out_assert
 from chinilla.types.peer_info import PeerInfo
 from chinilla.util.errors import Err
 from chinilla.util.ints import uint16, uint64
-from chinilla.simulator.time_out_assert import time_out_assert
 
 log = logging.getLogger(__name__)
 
@@ -48,9 +49,7 @@ class TestDos:
         session = ClientSession(timeout=timeout)
         url = f"wss://{self_hostname}:{server_1._port}/ws"
 
-        ssl_context = ssl_context_for_client(
-            server_2.chinilla_ca_crt_path, server_2.chinilla_ca_key_path, server_2.p2p_crt_path, server_2.p2p_key_path
-        )
+        ssl_context = server_2.ssl_client_context
         ws = await session.ws_connect(
             url, autoclose=True, autoping=True, heartbeat=60, ssl=ssl_context, max_msg_size=100 * 1024 * 1024
         )
@@ -97,9 +96,7 @@ class TestDos:
         session = ClientSession(timeout=timeout)
         url = f"wss://{self_hostname}:{server_1._port}/ws"
 
-        ssl_context = ssl_context_for_client(
-            server_2.chinilla_ca_crt_path, server_2.chinilla_ca_key_path, server_2.p2p_crt_path, server_2.p2p_key_path
-        )
+        ssl_context = server_2.ssl_client_context
         ws = await session.ws_connect(
             url, autoclose=True, autoping=True, heartbeat=60, ssl=ssl_context, max_msg_size=100 * 1024 * 1024
         )
@@ -143,9 +140,7 @@ class TestDos:
         session = ClientSession(timeout=timeout)
         url = f"wss://{self_hostname}:{server_1._port}/ws"
 
-        ssl_context = ssl_context_for_client(
-            server_2.chinilla_ca_crt_path, server_2.chinilla_ca_key_path, server_2.p2p_crt_path, server_2.p2p_key_path
-        )
+        ssl_context = server_2.ssl_client_context
         ws = await session.ws_connect(
             url, autoclose=True, autoping=True, heartbeat=60, ssl=ssl_context, max_msg_size=100 * 1024 * 1024
         )
