@@ -5,7 +5,7 @@ import pathlib
 from typing import List
 
 import pytest
-from chinillablspy import G1Element
+from blspy import G1Element
 from clvm_tools import binutils
 
 from chinilla.consensus.condition_costs import ConditionCost
@@ -13,12 +13,12 @@ from chinilla.consensus.cost_calculator import NPCResult
 from chinilla.consensus.default_constants import DEFAULT_CONSTANTS
 from chinilla.full_node.bundle_tools import simple_solution_generator
 from chinilla.full_node.mempool_check_conditions import get_name_puzzle_conditions, get_puzzle_and_solution_for_coin
+from chinilla.simulator.block_tools import test_constants
 from chinilla.types.blockchain_format.coin import Coin
 from chinilla.types.blockchain_format.program import Program, SerializedProgram
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.types.generator_types import BlockGenerator
 from chinilla.wallet.puzzles import p2_delegated_puzzle_or_hidden_puzzle
-from tests.setup_nodes import test_constants
 from tests.util.misc import assert_runtime
 
 from .make_block_generator import make_block_generator
