@@ -14,11 +14,11 @@ from chinilla.consensus.default_constants import DEFAULT_CONSTANTS
 from chinilla.consensus.multiprocess_validation import PreValidationResult
 from chinilla.full_node.block_store import BlockStore
 from chinilla.full_node.coin_store import CoinStore
+from chinilla.simulator.block_tools import test_constants
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.types.full_block import FullBlock
 from chinilla.util.db_wrapper import DBWrapper2
 from chinilla.util.ints import uint64
-from tests.setup_nodes import test_constants
 from tests.util.temp_file import TempFile
 
 
