@@ -1,27 +1,31 @@
 # flake8: noqa: F811, F401
+from __future__ import annotations
+
 from typing import List
 
 import pytest
-from chinillablspy import AugSchemeMPL
+from blspy import AugSchemeMPL
+from clvm.casts import int_to_bytes
 
 from chinilla.consensus.pot_iterations import is_overflow_block
 from chinilla.full_node.signage_point import SignagePoint
 from chinilla.protocols import full_node_protocol
 from chinilla.rpc.full_node_rpc_client import FullNodeRpcClient
 from chinilla.server.outbound_message import NodeType
+from chinilla.simulator.block_tools import get_signage_point, test_constants
 from chinilla.simulator.simulator_protocol import FarmNewBlockProtocol, ReorgProtocol
+from chinilla.simulator.time_out_assert import time_out_assert
+from chinilla.simulator.wallet_tools import WalletTool
+from chinilla.types.condition_opcodes import ConditionOpcode
+from chinilla.types.condition_with_args import ConditionWithArgs
 from chinilla.types.full_block import FullBlock
 from chinilla.types.spend_bundle import SpendBundle
 from chinilla.types.unfinished_block import UnfinishedBlock
 from chinilla.util.hash import std_hash
 from chinilla.util.ints import uint8
-from chinilla.simulator.block_tools import get_signage_point
 from tests.blockchain.blockchain_test_utils import _validate_and_add_block
 from tests.connection_utils import connect_and_get_peer
-from tests.setup_nodes import test_constants
-from chinilla.simulator.time_out_assert import time_out_assert
 from tests.util.rpc import validate_get_routes
-from chinilla.simulator.wallet_tools import WalletTool
 
 
 class TestRpc:
@@ -136,6 +140,7 @@ class TestRpc:
             assert len(await client.get_all_mempool_items()) == 0
             assert len(await client.get_all_mempool_tx_ids()) == 0
             assert (await client.get_mempool_item_by_tx_id(spend_bundle.name())) is None
+            assert (await client.get_mempool_item_by_tx_id(spend_bundle.name(), False)) is None
 
             await client.push_tx(spend_bundle)
             coin = spend_bundle.additions()[0]
@@ -155,6 +160,27 @@ class TestRpc:
             )
             assert (await client.get_coin_record_by_name(coin.name())) is None
 
+            # Verify that the include_pending arg to get_mempool_item_by_tx_id works
+            coin_to_spend_pending = list(blocks[-1].get_included_reward_coins())[1]
+            ahr = ConditionOpcode.ASSERT_HEIGHT_RELATIVE  # to force pending/potential
+            condition_dic = {ahr: [ConditionWithArgs(ahr, [int_to_bytes(100)])]}
+            spend_bundle_pending = wallet.generate_signed_transaction(
+                coin_to_spend_pending.amount,
+                ph_receiver,
+                coin_to_spend_pending,
+                condition_dic=condition_dic,
+            )
+            await client.push_tx(spend_bundle_pending)
+            assert (
+                await client.get_mempool_item_by_tx_id(spend_bundle_pending.name(), False)
+            ) is None  # not strictly in the mempool
+            assert (
+                SpendBundle.from_json_dict(
+                    (await client.get_mempool_item_by_tx_id(spend_bundle_pending.name(), True))["spend_bundle"]
+                )
+                == spend_bundle_pending  # pending entry into mempool, so include_pending fetches
+            )
+
             await full_node_api_1.farm_new_transaction_block(FarmNewBlockProtocol(ph_2))
 
             coin_record = await client.get_coin_record_by_name(coin.name())
