@@ -7,7 +7,7 @@ from itertools import permutations
 from typing import List
 
 import pytest
-from chinilla_rs import compute_merkle_set_root
+from chia_rs import compute_merkle_set_root
 
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.util.merkle_set import MerkleSet, confirm_included_already_hashed
