@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import json
 import unittest
 from dataclasses import replace
@@ -5,16 +7,16 @@ from secrets import token_bytes
 from typing import Callable, List, Optional, Tuple
 
 import pytest
-from chinillablspy import AugSchemeMPL, G1Element, PrivateKey
+from blspy import AugSchemeMPL, G1Element, PrivateKey
 
-from tests.util.keyring import using_temp_file_keyring
+from chinilla.simulator.keyring import using_temp_file_keyring
 from chinilla.util.errors import (
     KeychainFingerprintExists,
-    KeychainKeyDataMismatch,
-    KeychainSecretsMissing,
     KeychainFingerprintNotFound,
+    KeychainKeyDataMismatch,
     KeychainLabelExists,
     KeychainLabelInvalid,
+    KeychainSecretsMissing,
 )
 from chinilla.util.ints import uint32
 from chinilla.util.keychain import (
