@@ -1,10 +1,18 @@
+from __future__ import annotations
+
 import logging
+from typing import Type
+
 import pytest
 
-from chinilla.util.keyring_wrapper import KeyringWrapper, DEFAULT_PASSPHRASE_IF_NO_MASTER_PASSPHRASE
-from pathlib import Path
-from sys import platform
-from tests.util.keyring import using_temp_file_keyring, using_temp_file_keyring_and_cryptfilekeyring
+from chinilla.simulator.keyring import using_temp_file_keyring
+from chinilla.util.errors import (
+    KeychainFingerprintNotFound,
+    KeychainLabelError,
+    KeychainLabelExists,
+    KeychainLabelInvalid,
+)
+from chinilla.util.keyring_wrapper import DEFAULT_PASSPHRASE_IF_NO_MASTER_PASSPHRASE, KeyringWrapper
 
 log = logging.getLogger(__name__)
 
@@ -37,84 +45,6 @@ class TestKeyringWrapper:
         # Expect: the shared instance should be cleared
         assert KeyringWrapper.get_shared_instance(create_if_necessary=False) is None
 
-    # When: creating a new file keyring with a legacy keyring in place
-    @using_temp_file_keyring_and_cryptfilekeyring()
-    @pytest.mark.skip(reason="Does only work if `test_keyring_wrapper.py` gets called separately.")
-    def test_using_legacy_cryptfilekeyring(self):
-        """
-        In the case that an existing CryptFileKeyring (legacy) keyring exists and we're
-        creating a new FileKeyring, the legacy keyring's use should be prioritized over
-        the FileKeyring (until migration is triggered by a write to the keyring.)
-        """
-
-        if platform != "linux":
-            return
-
-        # Expect: the new keyring should not have content (not actually empty though...)
-        assert KeyringWrapper.get_shared_instance().keyring.has_content() is False
-        assert Path(KeyringWrapper.get_shared_instance().keyring.keyring_path).exists() is True
-        assert Path(KeyringWrapper.get_shared_instance().keyring.keyring_path).stat().st_size != 0
-
-        # Expect: legacy keyring should be in use
-        assert KeyringWrapper.get_shared_instance().legacy_keyring is not None
-        assert KeyringWrapper.get_shared_instance().using_legacy_keyring() is True
-        assert KeyringWrapper.get_shared_instance().get_keyring() == KeyringWrapper.get_shared_instance().legacy_keyring
-
-    # When: a file keyring has content and the legacy keyring exists
-    @using_temp_file_keyring_and_cryptfilekeyring(populate=True)
-    def test_using_file_keyring_with_legacy_keyring(self):
-        """
-        In the case that an existing CryptFileKeyring (legacy) keyring exists and we're
-        using a new FileKeyring with some keys in it, the FileKeyring's use should be
-        used instead of the legacy keyring.
-        """
-        # Expect: the new keyring should have content
-        assert KeyringWrapper.get_shared_instance().keyring.has_content() is True
-
-        # Expect: the new keyring should be in use
-        assert KeyringWrapper.get_shared_instance().legacy_keyring is None
-        assert KeyringWrapper.get_shared_instance().using_legacy_keyring() is False
-        assert KeyringWrapper.get_shared_instance().get_keyring() == KeyringWrapper.get_shared_instance().keyring
-
-    # When: a file keyring has content and the legacy keyring doesn't exists
-    @using_temp_file_keyring(populate=True)
-    def test_using_file_keyring_without_legacy_keyring(self):
-        """
-        In the case of a new installation (no legacy CryptFileKeyring) using a FileKeyring
-        with some content, the legacy keyring should not be used.
-        """
-        # Expect: the new keyring should have content
-        assert KeyringWrapper.get_shared_instance().keyring.has_content() is True
-
-        # Expect: the new keyring should be in use
-        assert KeyringWrapper.get_shared_instance().legacy_keyring is None
-        assert KeyringWrapper.get_shared_instance().using_legacy_keyring() is False
-        assert KeyringWrapper.get_shared_instance().get_keyring() == KeyringWrapper.get_shared_instance().keyring
-
-    # When: a file keyring is empty/unpopulated and the legacy keyring doesn't exists
-    @using_temp_file_keyring()
-    def test_using_new_file_keyring(self):
-        """
-        In the case of a new installation using a new FileKeyring, the legacy keyring
-        should not be used.
-        """
-        # Expect: the new keyring should not have any content
-        assert KeyringWrapper.get_shared_instance().keyring.has_content() is False
-
-        # Expect: the new keyring should be in use
-        assert KeyringWrapper.get_shared_instance().legacy_keyring is None
-        assert KeyringWrapper.get_shared_instance().using_legacy_keyring() is False
-        assert KeyringWrapper.get_shared_instance().get_keyring() == KeyringWrapper.get_shared_instance().keyring
-
-    # When: using a file keyring
-    @using_temp_file_keyring()
-    def test_file_keyring_supports_master_passphrase(self):
-        """
-        File keyrings should support setting a master passphrase
-        """
-        # Expect: keyring supports a master passphrase
-        assert KeyringWrapper.get_shared_instance().keyring_supports_master_passphrase() is True
-
     # When: creating a new/unpopulated file keyring
     @using_temp_file_keyring()
     def test_empty_file_keyring_doesnt_have_master_passphrase(self):
@@ -133,17 +63,6 @@ class TestKeyringWrapper:
         # Expect: master passphrase is set
         assert KeyringWrapper.get_shared_instance().has_master_passphrase() is True
 
-    # When: creating a new file keyring with a legacy keyring in place
-    @using_temp_file_keyring_and_cryptfilekeyring
-    def test_legacy_keyring_does_not_support_master_passphrase(self):
-        """
-        CryptFileKeyring (legacy keyring) should not support setting a master passphrase
-        """
-        # Expect: legacy keyring in use and master passphrase is not supported
-        assert KeyringWrapper.get_shared_instance().legacy_keyring is not None
-        assert KeyringWrapper.get_shared_instance().using_legacy_keyring() is True
-        assert KeyringWrapper.get_shared_instance().keyring_supports_master_passphrase() is False
-
     # When: creating a new file keyring
     @using_temp_file_keyring()
     def test_default_cached_master_passphrase(self):
@@ -303,7 +222,7 @@ class TestKeyringWrapper:
 
         # When: setting a passphrase
         KeyringWrapper.get_shared_instance().set_passphrase(
-            "service-abc", "user-xyz", "super secret passphrase".encode()
+            "service-abc", "user-xyz", "super secret passphrase".encode().hex()
         )
 
         # Expect: passphrase lookup should succeed
@@ -324,7 +243,9 @@ class TestKeyringWrapper:
         Overwriting a previously-set passphrase should work
         """
         # When: initially setting the passphrase
-        KeyringWrapper.get_shared_instance().set_passphrase("service-xyz", "user-123", "initial passphrase".encode())
+        KeyringWrapper.get_shared_instance().set_passphrase(
+            "service-xyz", "user-123", "initial passphrase".encode().hex()
+        )
 
         # Expect: passphrase lookup should succeed
         assert (
@@ -333,7 +254,9 @@ class TestKeyringWrapper:
         )
 
         # When: updating the same passphrase
-        KeyringWrapper.get_shared_instance().set_passphrase("service-xyz", "user-123", "updated passphrase".encode())
+        KeyringWrapper.get_shared_instance().set_passphrase(
+            "service-xyz", "user-123", "updated passphrase".encode().hex()
+        )
 
         # Expect: the updated passphrase should be retrieved
         assert (
@@ -351,7 +274,7 @@ class TestKeyringWrapper:
         KeyringWrapper.get_shared_instance().delete_passphrase("some service", "some user")
 
         # When: setting a passphrase
-        KeyringWrapper.get_shared_instance().set_passphrase("some service", "some user", "500p3r 53cr37".encode())
+        KeyringWrapper.get_shared_instance().set_passphrase("some service", "some user", "500p3r 53cr37".encode().hex())
 
         # Expect: passphrase retrieval should succeed
         assert (
@@ -478,3 +401,115 @@ class TestKeyringWrapper:
             KeyringWrapper.get_shared_instance().get_master_passphrase_hint()
             == "Something you wouldn't expect The Shredder to say"
         )
+
+    @using_temp_file_keyring()
+    def test_get_label(self):
+        keyring_wrapper = KeyringWrapper.get_shared_instance()
+        # label lookup for 1, 2, 3 should return None
+        assert keyring_wrapper.get_label(1) is None
+        assert keyring_wrapper.get_label(2) is None
+        assert keyring_wrapper.get_label(3) is None
+
+        # Set and validate a label for 1
+        keyring_wrapper.set_label(1, "one")
+        assert keyring_wrapper.get_label(1) == "one"
+
+        # Set and validate a label for 3
+        keyring_wrapper.set_label(3, "three")
+
+        # And validate all match the expected values
+        assert keyring_wrapper.get_label(1) == "one"
+        assert keyring_wrapper.get_label(2) is None
+        assert keyring_wrapper.get_label(3) == "three"
+
+    @using_temp_file_keyring()
+    def test_set_label(self):
+        keyring_wrapper = KeyringWrapper.get_shared_instance()
+        # Set and validate a label for 1
+        keyring_wrapper.set_label(1, "one")
+        assert keyring_wrapper.get_label(1) == "one"
+
+        # Set and validate a label for 2
+        keyring_wrapper.set_label(2, "two")
+        assert keyring_wrapper.get_label(2) == "two"
+
+        # Change the label of 2
+        keyring_wrapper.set_label(2, "two!")
+        assert keyring_wrapper.get_label(2) == "two!"
+        # 1 should still have the same label
+        assert keyring_wrapper.get_label(1) == "one"
+
+        # Change the label of 2 again
+        keyring_wrapper.set_label(2, "two!!")
+        assert keyring_wrapper.get_label(2) == "two!!"
+        # 1 should still have the same label
+        assert keyring_wrapper.get_label(1) == "one"
+
+        # Change the label of 1
+        keyring_wrapper.set_label(1, "one!")
+        assert keyring_wrapper.get_label(1) == "one!"
+        # 2 should still have the same label
+        assert keyring_wrapper.get_label(2) == "two!!"
+
+    @pytest.mark.parametrize(
+        "label",
+        [
+            "🥳🤩🤪🤯😎😝😀",
+            "私は幸せな農夫です",
+            "لتفاصيل لتكتشف حقيقة وأساس ت",
+        ],
+    )
+    @using_temp_file_keyring()
+    def test_set_special_labels(self, label: str):
+        keyring_wrapper = KeyringWrapper.get_shared_instance()
+        keyring_wrapper.set_label(1, label)
+        assert keyring_wrapper.get_label(1) == label
+
+    @pytest.mark.parametrize(
+        "label, exception, message",
+        [
+            ("one", KeychainLabelExists, "label 'one' already exists for fingerprint '1"),
+            ("", KeychainLabelInvalid, "label can't be empty or whitespace only"),
+            ("   ", KeychainLabelInvalid, "label can't be empty or whitespace only"),
+            ("a\nb", KeychainLabelInvalid, "label can't contain newline or tab"),
+            ("a\tb", KeychainLabelInvalid, "label can't contain newline or tab"),
+            ("a label ", KeychainLabelInvalid, "label can't contain leading or trailing whitespaces"),
+            (" a label", KeychainLabelInvalid, "label can't contain leading or trailing whitespaces"),
+            (" a label ", KeychainLabelInvalid, "label can't contain leading or trailing whitespaces"),
+            ("  a label ", KeychainLabelInvalid, "label can't contain leading or trailing whitespaces"),
+            ("a" * 66, KeychainLabelInvalid, "label exceeds max length: 66/65"),
+            ("a" * 70, KeychainLabelInvalid, "label exceeds max length: 70/65"),
+        ],
+    )
+    @using_temp_file_keyring()
+    def test_set_label_failures(self, label: str, exception: Type[KeychainLabelError], message: str) -> None:
+        keyring_wrapper = KeyringWrapper.get_shared_instance()
+        keyring_wrapper.set_label(1, "one")
+        with pytest.raises(exception, match=message) as e:
+            keyring_wrapper.set_label(1, label)
+        assert e.value.label == label
+        if isinstance(e.value, KeychainLabelExists):
+            assert e.value.label == "one"
+            assert e.value.fingerprint == 1
+
+    @using_temp_file_keyring()
+    def test_delete_label(self) -> None:
+        keyring_wrapper = KeyringWrapper.get_shared_instance()
+        # Set labels for 1,2 and validate them
+        keyring_wrapper.set_label(1, "one")
+        keyring_wrapper.set_label(2, "two")
+        assert keyring_wrapper.get_label(1) == "one"
+        assert keyring_wrapper.get_label(2) == "two"
+        # Remove the label of 1
+        keyring_wrapper.delete_label(1)
+        assert keyring_wrapper.get_label(1) is None
+        assert keyring_wrapper.get_label(2) == "two"
+        # Remove the label of 2
+        keyring_wrapper.delete_label(2)
+        assert keyring_wrapper.get_label(1) is None
+        assert keyring_wrapper.get_label(2) is None
+        # Make sure the deletion fails for 0-2
+        for i in range(3):
+            with pytest.raises(KeychainFingerprintNotFound) as e:
+                keyring_wrapper.delete_label(i)
+            assert e.value.fingerprint == i
