@@ -6,12 +6,12 @@ from dataclasses import dataclass, field, fields
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type, get_type_hints
 
 import pytest
-from chinillablspy import G1Element
+from blspy import G1Element
 from clvm_tools import binutils
 from typing_extensions import Literal, get_args
 
 from chinilla.protocols.wallet_protocol import RespondRemovals
-from chinilla.simulator.block_tools import BlockTools
+from chinilla.simulator.block_tools import BlockTools, test_constants
 from chinilla.types.blockchain_format.coin import Coin
 from chinilla.types.blockchain_format.program import Program
 from chinilla.types.blockchain_format.sized_bytes import bytes4, bytes32
@@ -44,7 +44,6 @@ from chinilla.util.streamable import (
     streamable_from_dict,
     write_uint32,
 )
-from tests.setup_nodes import test_constants
 
 
 def test_int_not_supported() -> None:
