@@ -9,7 +9,7 @@ from typing import Any, Callable, List, Optional, Tuple
 
 import pytest
 import pytest_asyncio
-from chinillablspy import G1Element
+from blspy import G1Element
 
 from chinilla.farmer.farmer_api import Farmer
 from chinilla.harvester.harvester_api import Harvester
@@ -20,8 +20,8 @@ from chinilla.plot_sync.util import Constants, State
 from chinilla.plotting.manager import PlotManager
 from chinilla.plotting.util import add_plot_directory, remove_plot_directory
 from chinilla.protocols.harvester_protocol import Plot
+from chinilla.protocols.protocol_message_types import ProtocolMessageTypes
 from chinilla.server.start_service import Service
-from chinilla.server.ws_connection import ProtocolMessageTypes
 from chinilla.simulator.block_tools import BlockTools
 from chinilla.simulator.time_out_assert import time_out_assert
 from chinilla.types.blockchain_format.sized_bytes import bytes32
