@@ -11,7 +11,7 @@ from secrets import token_bytes
 from typing import Any, Dict, List, Optional, Set, Tuple
 
 import pytest
-from chinillablspy import G1Element
+from blspy import G1Element
 
 from chinilla.farmer.farmer_api import Farmer
 from chinilla.harvester.harvester_api import Harvester
@@ -21,8 +21,10 @@ from chinilla.plot_sync.util import Constants
 from chinilla.plotting.manager import PlotManager
 from chinilla.plotting.util import PlotInfo
 from chinilla.protocols.harvester_protocol import PlotSyncError, PlotSyncResponse
+from chinilla.protocols.protocol_message_types import ProtocolMessageTypes
+from chinilla.server.outbound_message import make_msg
 from chinilla.server.start_service import Service
-from chinilla.server.ws_connection import ProtocolMessageTypes, WSChinillaConnection, make_msg
+from chinilla.server.ws_connection import WSChinillaConnection
 from chinilla.simulator.block_tools import BlockTools
 from chinilla.simulator.time_out_assert import time_out_assert
 from chinilla.types.blockchain_format.sized_bytes import bytes32
