@@ -10,7 +10,7 @@ from shutil import copy, move
 from typing import Callable, Iterator, List, Optional
 
 import pytest
-from chinillablspy import G1Element
+from blspy import G1Element
 
 from chinilla.plotting.cache import CURRENT_VERSION, CacheDataV1
 from chinilla.plotting.manager import Cache, PlotManager
@@ -542,7 +542,7 @@ async def test_drop_too_large_cache_entries(environment, bt):
     def modify_cache_entry(index: int, additional_data: int, modify_memo: bool) -> str:
         path, cache_entry = cache_data.entries[index]
         prover_data = cache_entry.prover_data
-        # Size of length hints in chinillapos serialization currently depends on the platform
+        # Size of length hints in chiapos serialization currently depends on the platform
         size_length = 8 if sys.maxsize > 2**32 else 4
         # Version
         version_size = 2
