@@ -5,16 +5,21 @@ import dataclasses
 import enum
 import gc
 import math
+import os
+import subprocess
 from concurrent.futures import Future
 from inspect import getframeinfo, stack
 from statistics import mean
 from textwrap import dedent
 from time import thread_time
 from types import TracebackType
-from typing import Callable, Iterator, List, Optional, Type, Union
+from typing import Any, Callable, Iterator, List, Optional, Type, Union
 
+import pytest
 from typing_extensions import final
 
+from tests.core.data_layer.util import ChinillaRoot
+
 
 class GcMode(enum.Enum):
     nothing = enum.auto
@@ -276,3 +281,25 @@ class _AssertRuntime:
 # decorator, this is just here to retain the function-style naming as the public
 # interface.  Hopefully we can switch away from the class at some point.
 assert_runtime = _AssertRuntime
+
+
+@contextlib.contextmanager
+def assert_rpc_error(error: str) -> Iterator[None]:
+    with pytest.raises(ValueError) as exception_info:
+        yield
+    assert error in exception_info.value.args[0]["error"]
+
+
+@contextlib.contextmanager
+def closing_chinilla_root_popen(chinilla_root: ChinillaRoot, args: List[str]) -> Iterator[subprocess.Popen[Any]]:
+    environment = {**os.environ, "CHINILLA_ROOT": os.fspath(chinilla_root.path)}
+
+    with subprocess.Popen(args=args, env=environment) as process:
+        try:
+            yield process
+        finally:
+            process.terminate()
+            try:
+                process.wait(timeout=10)
+            except subprocess.TimeoutExpired:
+                process.kill()
