@@ -17,8 +17,8 @@ class TestNetwork:
 
     @pytest.mark.asyncio
     @pytest.mark.skipif(
-        condition=("GITHUB_ACTIONS" in os.environ) and (sys.platform == "darwin"),
-        reason="macOS runners in GitHub Actions do not seem to support IPv6",
+        condition=("GITHUB_ACTIONS" in os.environ) and (sys.platform in {"darwin", "win32"}),
+        reason="macOS and Windows runners in GitHub Actions do not seem to support IPv6",
     )
     async def test_get_host_addr6(self):
         # Run these tests forcing IPv6 resolution
