@@ -1,12 +1,17 @@
+from __future__ import annotations
+
 import asyncio
 from secrets import token_bytes
 from typing import Any, Dict, List
 
 import pytest
 
-from chinilla.full_node.mempool_manager import MempoolManager
+from chinilla.consensus.cost_calculator import NPCResult
+from chinilla.full_node.bundle_tools import simple_solution_generator
+from chinilla.full_node.mempool_check_conditions import get_name_puzzle_conditions
 from chinilla.simulator.simulator_protocol import FarmNewBlockProtocol
 from chinilla.simulator.time_out_assert import time_out_assert
+from chinilla.types.blockchain_format.program import INFINITE_COST
 from chinilla.util.ints import uint64
 from chinilla.wallet.cat_wallet.cat_wallet import CATWallet
 from chinilla.wallet.outer_puzzles import AssetType
@@ -15,14 +20,7 @@ from chinilla.wallet.trading.offer import Offer
 from chinilla.wallet.trading.trade_status import TradeStatus
 from chinilla.wallet.transaction_record import TransactionRecord
 from chinilla.wallet.util.transaction_type import TransactionType
-
-
-async def tx_in_pool(mempool: MempoolManager, tx_id):
-    tx = mempool.get_spendbundle(tx_id)
-    if tx is None:
-        return False
-    return True
-
+from tests.util.wallet_is_synced import wallets_are_synced
 
 buffer_blocks = 4
 
@@ -34,7 +32,11 @@ buffer_blocks = 4
 class TestCATTrades:
     @pytest.mark.asyncio
     async def test_cat_trades(self, wallets_prefarm):
-        wallet_node_maker, wallet_node_taker, full_node = wallets_prefarm
+        (
+            [wallet_node_maker, initial_maker_balance],
+            [wallet_node_taker, initial_taker_balance],
+            full_node,
+        ) = wallets_prefarm
         wallet_maker = wallet_node_maker.wallet_state_manager.main_wallet
         wallet_taker = wallet_node_taker.wallet_state_manager.main_wallet
 
@@ -66,8 +68,8 @@ class TestCATTrades:
         )
 
         # Create the trade parameters
-        MAKER_CHINILLA_BALANCE = 20 * 1000000000000 - 100
-        TAKER_CHINILLA_BALANCE = 20 * 1000000000000 - 100
+        MAKER_CHINILLA_BALANCE = initial_maker_balance - 100
+        TAKER_CHINILLA_BALANCE = initial_taker_balance - 100
         await time_out_assert(25, wallet_maker.get_confirmed_balance, MAKER_CHINILLA_BALANCE)
         await time_out_assert(25, wallet_taker.get_unconfirmed_balance, TAKER_CHINILLA_BALANCE)
         MAKER_CAT_BALANCE = 100
@@ -126,13 +128,13 @@ class TestCATTrades:
 
         peer = wallet_node_taker.get_full_node_peer()
         assert peer is not None
-        success, trade_take, error = await trade_manager_taker.respond_to_offer(
+        trade_take, tx_records = await trade_manager_taker.respond_to_offer(
             Offer.from_bytes(trade_make.offer), peer, fee=uint64(1)
         )
-        await asyncio.sleep(1)
-        assert error is None
-        assert success is True
         assert trade_take is not None
+        assert tx_records is not None
+
+        first_offer = Offer.from_bytes(trade_take.offer)
 
         MAKER_CHINILLA_BALANCE -= 2  # -1 and -1 for fee
         MAKER_NEW_CAT_BALANCE += 2
@@ -142,8 +144,8 @@ class TestCATTrades:
         await time_out_assert(15, wallet_taker.get_unconfirmed_balance, TAKER_CHINILLA_BALANCE)
         await time_out_assert(15, new_cat_wallet_taker.get_unconfirmed_balance, TAKER_NEW_CAT_BALANCE)
 
-        for i in range(0, buffer_blocks):
-            await full_node.farm_new_transaction_block(FarmNewBlockProtocol(token_bytes()))
+        await full_node.process_transaction_records(records=tx_records)
+        await time_out_assert(15, wallets_are_synced, True, [wallet_node_maker, wallet_node_taker], full_node)
 
         await time_out_assert(15, wallet_maker.get_confirmed_balance, MAKER_CHINILLA_BALANCE)
         await time_out_assert(15, wallet_maker.get_unconfirmed_balance, MAKER_CHINILLA_BALANCE)
@@ -170,18 +172,13 @@ class TestCATTrades:
 
         # cat_for_chinilla
         success, trade_make, error = await trade_manager_maker.create_offer_for_ids(cat_for_chinilla)
-        await asyncio.sleep(1)
         assert error is None
         assert success is True
         assert trade_make is not None
 
-        success, trade_take, error = await trade_manager_taker.respond_to_offer(
-            Offer.from_bytes(trade_make.offer), peer
-        )
-        await asyncio.sleep(1)
-        assert error is None
-        assert success is True
+        trade_take, tx_records = await trade_manager_taker.respond_to_offer(Offer.from_bytes(trade_make.offer), peer)
         assert trade_take is not None
+        assert tx_records is not None
 
         MAKER_CAT_BALANCE -= 4
         MAKER_CHINILLA_BALANCE += 3
@@ -195,8 +192,8 @@ class TestCATTrades:
         await time_out_assert(15, wallet_taker.get_unconfirmed_balance, TAKER_CHINILLA_BALANCE)
         await time_out_assert(15, cat_wallet_taker.get_unconfirmed_balance, TAKER_CAT_BALANCE)
 
-        for i in range(0, buffer_blocks):
-            await full_node.farm_new_transaction_block(FarmNewBlockProtocol(token_bytes()))
+        await full_node.process_transaction_records(records=tx_records)
+        await time_out_assert(15, wallets_are_synced, True, [wallet_node_maker, wallet_node_taker], full_node)
 
         await time_out_assert(15, wallet_maker.get_confirmed_balance, MAKER_CHINILLA_BALANCE)
         await time_out_assert(15, wallet_maker.get_unconfirmed_balance, MAKER_CHINILLA_BALANCE)
@@ -217,13 +214,12 @@ class TestCATTrades:
         assert error is None
         assert success is True
         assert trade_make is not None
-        success, trade_take, error = await trade_manager_taker.respond_to_offer(
-            Offer.from_bytes(trade_make.offer), peer
-        )
-        await asyncio.sleep(1)
-        assert error is None
-        assert success is True
+        trade_take, tx_records = await trade_manager_taker.respond_to_offer(Offer.from_bytes(trade_make.offer), peer)
+        await time_out_assert(15, full_node.txs_in_mempool, True, tx_records)
         assert trade_take is not None
+        assert tx_records is not None
+
+        second_offer = Offer.from_bytes(trade_take.offer)
 
         MAKER_CAT_BALANCE -= 5
         MAKER_NEW_CAT_BALANCE += 6
@@ -233,8 +229,8 @@ class TestCATTrades:
         await time_out_assert(15, new_cat_wallet_taker.get_unconfirmed_balance, TAKER_NEW_CAT_BALANCE)
         await time_out_assert(15, cat_wallet_taker.get_unconfirmed_balance, TAKER_CAT_BALANCE)
 
-        for i in range(0, buffer_blocks):
-            await full_node.farm_new_transaction_block(FarmNewBlockProtocol(token_bytes()))
+        await full_node.process_transaction_records(records=tx_records)
+        await time_out_assert(15, wallets_are_synced, True, [wallet_node_maker, wallet_node_taker], full_node)
 
         await time_out_assert(15, new_cat_wallet_maker.get_confirmed_balance, MAKER_NEW_CAT_BALANCE)
         await time_out_assert(15, new_cat_wallet_maker.get_unconfirmed_balance, MAKER_NEW_CAT_BALANCE)
@@ -255,13 +251,12 @@ class TestCATTrades:
         assert error is None
         assert success is True
         assert trade_make is not None
-        success, trade_take, error = await trade_manager_taker.respond_to_offer(
-            Offer.from_bytes(trade_make.offer), peer
-        )
-        await asyncio.sleep(1)
-        assert error is None
-        assert success is True
+        trade_take, tx_records = await trade_manager_taker.respond_to_offer(Offer.from_bytes(trade_make.offer), peer)
+        await time_out_assert(15, full_node.txs_in_mempool, True, tx_records)
         assert trade_take is not None
+        assert tx_records is not None
+
+        third_offer = Offer.from_bytes(trade_take.offer)
 
         MAKER_CHINILLA_BALANCE -= 7
         MAKER_CAT_BALANCE += 8
@@ -273,8 +268,8 @@ class TestCATTrades:
         await time_out_assert(15, new_cat_wallet_taker.get_unconfirmed_balance, TAKER_NEW_CAT_BALANCE)
         await time_out_assert(15, cat_wallet_taker.get_unconfirmed_balance, TAKER_CAT_BALANCE)
 
-        for i in range(0, buffer_blocks):
-            await full_node.farm_new_transaction_block(FarmNewBlockProtocol(token_bytes()))
+        await full_node.process_transaction_records(records=tx_records)
+        await time_out_assert(15, wallets_are_synced, True, [wallet_node_maker, wallet_node_taker], full_node)
 
         await time_out_assert(15, new_cat_wallet_maker.get_confirmed_balance, MAKER_NEW_CAT_BALANCE)
         await time_out_assert(15, new_cat_wallet_maker.get_unconfirmed_balance, MAKER_NEW_CAT_BALANCE)
@@ -293,13 +288,12 @@ class TestCATTrades:
         assert error is None
         assert success is True
         assert trade_make is not None
-        success, trade_take, error = await trade_manager_taker.respond_to_offer(
-            Offer.from_bytes(trade_make.offer), peer
-        )
-        await asyncio.sleep(1)
-        assert error is None
-        assert success is True
+        trade_take, tx_records = await trade_manager_taker.respond_to_offer(Offer.from_bytes(trade_make.offer), peer)
+        await time_out_assert(15, full_node.txs_in_mempool, True, tx_records)
         assert trade_take is not None
+        assert tx_records is not None
+
+        fourth_offer = Offer.from_bytes(trade_take.offer)
 
         MAKER_CAT_BALANCE -= 11
         MAKER_NEW_CAT_BALANCE -= 12
@@ -311,8 +305,8 @@ class TestCATTrades:
         await time_out_assert(15, new_cat_wallet_taker.get_unconfirmed_balance, TAKER_NEW_CAT_BALANCE)
         await time_out_assert(15, cat_wallet_taker.get_unconfirmed_balance, TAKER_CAT_BALANCE)
 
-        for i in range(0, buffer_blocks):
-            await full_node.farm_new_transaction_block(FarmNewBlockProtocol(token_bytes()))
+        await full_node.process_transaction_records(records=tx_records)
+        await time_out_assert(15, wallets_are_synced, True, [wallet_node_maker, wallet_node_taker], full_node)
 
         await time_out_assert(15, new_cat_wallet_maker.get_confirmed_balance, MAKER_NEW_CAT_BALANCE)
         await time_out_assert(15, new_cat_wallet_maker.get_unconfirmed_balance, MAKER_NEW_CAT_BALANCE)
@@ -331,13 +325,12 @@ class TestCATTrades:
         assert error is None
         assert success is True
         assert trade_make is not None
-        success, trade_take, error = await trade_manager_taker.respond_to_offer(
-            Offer.from_bytes(trade_make.offer), peer
-        )
-        await asyncio.sleep(1)
-        assert error is None
-        assert success is True
+        trade_take, tx_records = await trade_manager_taker.respond_to_offer(Offer.from_bytes(trade_make.offer), peer)
+        await time_out_assert(15, full_node.txs_in_mempool, True, tx_records)
         assert trade_take is not None
+        assert tx_records is not None
+
+        fifth_offer = Offer.from_bytes(trade_take.offer)
 
         MAKER_CHINILLA_BALANCE -= 13
         MAKER_CAT_BALANCE -= 14
@@ -349,8 +342,8 @@ class TestCATTrades:
         await time_out_assert(15, new_cat_wallet_taker.get_unconfirmed_balance, TAKER_NEW_CAT_BALANCE)
         await time_out_assert(15, cat_wallet_taker.get_unconfirmed_balance, TAKER_CAT_BALANCE)
 
-        for i in range(0, buffer_blocks):
-            await full_node.farm_new_transaction_block(FarmNewBlockProtocol(token_bytes()))
+        await full_node.process_transaction_records(records=tx_records)
+        await time_out_assert(15, wallets_are_synced, True, [wallet_node_maker, wallet_node_taker], full_node)
 
         await time_out_assert(15, new_cat_wallet_maker.get_confirmed_balance, MAKER_NEW_CAT_BALANCE)
         await time_out_assert(15, new_cat_wallet_maker.get_unconfirmed_balance, MAKER_NEW_CAT_BALANCE)
@@ -363,31 +356,38 @@ class TestCATTrades:
         await time_out_assert(15, get_trade_and_status, TradeStatus.CONFIRMED, trade_manager_maker, trade_make)
         await time_out_assert(15, get_trade_and_status, TradeStatus.CONFIRMED, trade_manager_taker, trade_take)
 
+        # This tests an edge case where aggregated offers the include > 2 of the same kind of CAT
+        # (and therefore are solved as a complete ring)
+        bundle = Offer.aggregate([first_offer, second_offer, third_offer, fourth_offer, fifth_offer]).to_valid_spend()
+        program = simple_solution_generator(bundle)
+        result: NPCResult = get_name_puzzle_conditions(program, INFINITE_COST, cost_per_byte=0, mempool_mode=True)
+        assert result.error is None
+
     @pytest.mark.asyncio
     async def test_trade_cancellation(self, wallets_prefarm):
-        wallet_node_maker, wallet_node_taker, full_node = wallets_prefarm
+        (
+            [wallet_node_maker, maker_funds],
+            [wallet_node_taker, taker_funds],
+            full_node,
+        ) = wallets_prefarm
         wallet_maker = wallet_node_maker.wallet_state_manager.main_wallet
         wallet_taker = wallet_node_taker.wallet_state_manager.main_wallet
 
+        hcx_to_cat_amount = uint64(100)
+
         async with wallet_node_maker.wallet_state_manager.lock:
             cat_wallet_maker: CATWallet = await CATWallet.create_new_cat_wallet(
-                wallet_node_maker.wallet_state_manager, wallet_maker, {"identifier": "genesis_by_id"}, uint64(100)
+                wallet_node_maker.wallet_state_manager, wallet_maker, {"identifier": "genesis_by_id"}, hcx_to_cat_amount
             )
 
-            tx_queue: List[TransactionRecord] = await wallet_node_maker.wallet_state_manager.tx_store.get_not_sent()
-            await time_out_assert(
-                15, tx_in_pool, True, full_node.full_node.mempool_manager, tx_queue[0].spend_bundle.name()
-            )
+            tx_records: List[TransactionRecord] = await wallet_node_maker.wallet_state_manager.tx_store.get_not_sent()
 
-        for i in range(1, buffer_blocks):
-            await full_node.farm_new_transaction_block(FarmNewBlockProtocol(token_bytes()))
+        await full_node.process_transaction_records(records=tx_records)
 
-        await time_out_assert(15, cat_wallet_maker.get_confirmed_balance, 100)
-        await time_out_assert(15, cat_wallet_maker.get_unconfirmed_balance, 100)
-        MAKER_CHINILLA_BALANCE = 20 * 1000000000000 - 100
-        MAKER_CAT_BALANCE = 100
-        TAKER_CHINILLA_BALANCE = 20 * 1000000000000
-        await time_out_assert(15, wallet_maker.get_confirmed_balance, MAKER_CHINILLA_BALANCE)
+        await time_out_assert(15, cat_wallet_maker.get_confirmed_balance, hcx_to_cat_amount)
+        await time_out_assert(15, cat_wallet_maker.get_unconfirmed_balance, hcx_to_cat_amount)
+        maker_funds -= hcx_to_cat_amount
+        await time_out_assert(15, wallet_maker.get_confirmed_balance, maker_funds)
 
         cat_for_chinilla = {
             wallet_maker.id(): 1,
@@ -418,27 +418,25 @@ class TestCATTrades:
         # Due to current mempool rules, trying to force a take out of the mempool with a cancel will not work.
         # Uncomment this when/if it does
 
-        # success, trade_take, error = await trade_manager_taker.respond_to_offer(Offer.from_bytes(trade_make.offer))
-        # await asyncio.sleep(1)
-        # assert error is None
-        # assert success is True
+        # trade_take, tx_records = await trade_manager_taker.respond_to_offer(
+        #     Offer.from_bytes(trade_make.offer),
+        # )
+        # await time_out_assert(15, full_node.txs_in_mempool, True, tx_records)
         # assert trade_take is not None
+        # assert tx_records is not None
         # await time_out_assert(15, get_trade_and_status, TradeStatus.PENDING_CONFIRM, trade_manager_taker, trade_take)
         # await time_out_assert(
         #     15,
-        #     tx_in_pool,
+        #     full_node.tx_id_in_mempool,
         #     True,
-        #     full_node.full_node.mempool_manager,
         #     Offer.from_bytes(trade_take.offer).to_valid_spend().name(),
         # )
 
-        FEE = uint64(2000000000000)
+        fee = uint64(2_000_000_000_000)
 
-        txs = await trade_manager_maker.cancel_pending_offer_safely(trade_make.trade_id, fee=FEE)
+        txs = await trade_manager_maker.cancel_pending_offer_safely(trade_make.trade_id, fee=fee)
         await time_out_assert(15, get_trade_and_status, TradeStatus.PENDING_CANCEL, trade_manager_maker, trade_make)
-        for tx in txs:
-            if tx.spend_bundle is not None:
-                await time_out_assert(15, tx_in_pool, True, full_node.full_node.mempool_manager, tx.spend_bundle.name())
+        await full_node.process_transaction_records(records=txs)
 
         sum_of_outgoing = uint64(0)
         sum_of_incoming = uint64(0)
@@ -449,50 +447,34 @@ class TestCATTrades:
                 sum_of_incoming = uint64(sum_of_incoming + tx.amount)
         assert (sum_of_outgoing - sum_of_incoming) == 0
 
-        for i in range(1, buffer_blocks):
-            await full_node.farm_new_transaction_block(FarmNewBlockProtocol(token_bytes()))
-
         await time_out_assert(15, get_trade_and_status, TradeStatus.CANCELLED, trade_manager_maker, trade_make)
         # await time_out_assert(15, get_trade_and_status, TradeStatus.FAILED, trade_manager_taker, trade_take)
 
         await time_out_assert(15, wallet_maker.get_pending_change_balance, 0)
-        await time_out_assert(15, wallet_maker.get_confirmed_balance, MAKER_CHINILLA_BALANCE - FEE)
-        await time_out_assert(15, cat_wallet_maker.get_confirmed_balance, MAKER_CAT_BALANCE)
-        await time_out_assert(15, wallet_taker.get_confirmed_balance, TAKER_CHINILLA_BALANCE)
+        await time_out_assert(15, wallet_maker.get_confirmed_balance, maker_funds - fee)
+        await time_out_assert(15, cat_wallet_maker.get_confirmed_balance, hcx_to_cat_amount)
+        await time_out_assert(15, wallet_taker.get_confirmed_balance, taker_funds)
 
         peer = wallet_node_taker.get_full_node_peer()
         assert peer is not None
-        success, trade_take, error = await trade_manager_taker.respond_to_offer(
-            Offer.from_bytes(trade_make.offer), peer
-        )
-        await asyncio.sleep(1)
-        assert error is not None
-        assert success is False
-        assert trade_take is None
+        with pytest.raises(ValueError, match="This offer is no longer valid"):
+            await trade_manager_taker.respond_to_offer(Offer.from_bytes(trade_make.offer), peer)
 
         # Now we're going to create the other way around for test coverage sake
         success, trade_make, error = await trade_manager_maker.create_offer_for_ids(chinilla_for_cat)
-        await asyncio.sleep(1)
         assert error is None
         assert success is True
         assert trade_make is not None
 
         # This take should fail since we have no CATs to fulfill it with
-        success, trade_take, error = await trade_manager_taker.respond_to_offer(
-            Offer.from_bytes(trade_make.offer), peer
-        )
-        await asyncio.sleep(1)
-        assert error is not None
-        assert success is False
-        assert trade_take is None
+        with pytest.raises(
+            ValueError,
+            match=f"Do not have a wallet for asset ID: {cat_wallet_maker.get_asset_id()} to fulfill offer",
+        ):
+            await trade_manager_taker.respond_to_offer(Offer.from_bytes(trade_make.offer), peer)
 
         txs = await trade_manager_maker.cancel_pending_offer_safely(trade_make.trade_id, fee=uint64(0))
         await time_out_assert(15, get_trade_and_status, TradeStatus.PENDING_CANCEL, trade_manager_maker, trade_make)
-        for tx in txs:
-            if tx.spend_bundle is not None:
-                await time_out_assert(15, tx_in_pool, True, full_node.full_node.mempool_manager, tx.spend_bundle.name())
-
-        for i in range(1, buffer_blocks):
-            await full_node.farm_new_transaction_block(FarmNewBlockProtocol(token_bytes()))
+        await full_node.process_transaction_records(records=txs)
 
         await time_out_assert(15, get_trade_and_status, TradeStatus.CANCELLED, trade_manager_maker, trade_make)
