@@ -1,4 +1,5 @@
-import dataclasses
+from __future__ import annotations
+
 from typing import Any, List, Tuple
 
 import pytest
@@ -41,7 +42,11 @@ def get_parent_branch(value: bytes32, proof: Tuple[int, List[bytes32]]) -> Tuple
 )
 @pytest.mark.asyncio
 async def test_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
-    wallet_node_maker, wallet_node_taker, full_node_api = wallets_prefarm
+    (
+        [wallet_node_maker, maker_funds],
+        [wallet_node_taker, taker_funds],
+        full_node_api,
+    ) = wallets_prefarm
     assert wallet_node_maker.wallet_state_manager is not None
     assert wallet_node_taker.wallet_state_manager is not None
     wsm_maker = wallet_node_maker.wallet_state_manager
@@ -50,37 +55,34 @@ async def test_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
     wallet_maker = wsm_maker.main_wallet
     wallet_taker = wsm_taker.main_wallet
 
-    funds = 20000000000000
-
-    await time_out_assert(10, wallet_maker.get_unconfirmed_balance, funds)
-    await time_out_assert(10, wallet_taker.get_confirmed_balance, funds)
-
     async with wsm_maker.lock:
         dl_wallet_maker = await DataLayerWallet.create_new_dl_wallet(wsm_maker, wallet_maker)
     async with wsm_taker.lock:
         dl_wallet_taker = await DataLayerWallet.create_new_dl_wallet(wsm_taker, wallet_taker)
 
     MAKER_ROWS = [bytes32([i] * 32) for i in range(0, 10)]
-    TAKER_ROWS = [bytes32([i] * 32) for i in range(10, 20)]
+    TAKER_ROWS = [bytes32([i] * 32) for i in range(0, 10)]
     maker_root, _ = build_merkle_tree(MAKER_ROWS)
     taker_root, _ = build_merkle_tree(TAKER_ROWS)
 
-    dl_record, std_record, launcher_id_maker = await dl_wallet_maker.generate_new_reporter(
-        maker_root, fee=uint64(1999999999999)
-    )
+    fee = uint64(1_999_999_999_999)
+
+    dl_record, std_record, launcher_id_maker = await dl_wallet_maker.generate_new_reporter(maker_root, fee=fee)
     assert await dl_wallet_maker.get_latest_singleton(launcher_id_maker) is not None
     await wsm_maker.add_pending_transaction(dl_record)
     await wsm_maker.add_pending_transaction(std_record)
     await full_node_api.process_transaction_records(records=[dl_record, std_record])
+    maker_funds -= fee
+    maker_funds -= 1
     await time_out_assert(15, is_singleton_confirmed_and_root, True, dl_wallet_maker, launcher_id_maker, maker_root)
 
-    dl_record, std_record, launcher_id_taker = await dl_wallet_taker.generate_new_reporter(
-        taker_root, fee=uint64(1999999999999)
-    )
+    dl_record, std_record, launcher_id_taker = await dl_wallet_taker.generate_new_reporter(taker_root, fee=fee)
     assert await dl_wallet_taker.get_latest_singleton(launcher_id_taker) is not None
     await wsm_taker.add_pending_transaction(dl_record)
     await wsm_taker.add_pending_transaction(std_record)
     await full_node_api.process_transaction_records(records=[dl_record, std_record])
+    taker_funds -= fee
+    taker_funds -= 1
     await time_out_assert(15, is_singleton_confirmed_and_root, True, dl_wallet_taker, launcher_id_taker, taker_root)
 
     peer = wallet_node_taker.get_full_node_peer()
@@ -94,7 +96,7 @@ async def test_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
     trade_manager_taker = wsm_taker.trade_manager
 
     maker_addition = bytes32([101] * 32)
-    taker_addition = bytes32([202] * 32)
+    taker_addition = bytes32([101] * 32)
     MAKER_ROWS.append(maker_addition)
     TAKER_ROWS.append(taker_addition)
     maker_root, maker_proofs = build_merkle_tree(MAKER_ROWS)
@@ -102,6 +104,8 @@ async def test_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
     maker_branch, maker_branch_proof = get_parent_branch(maker_addition, maker_proofs[maker_addition])
     taker_branch, taker_branch_proof = get_parent_branch(taker_addition, taker_proofs[taker_addition])
 
+    fee = uint64(2_000_000_000_000)
+
     success, offer_maker, error = await trade_manager_maker.create_offer_for_ids(
         {launcher_id_maker: -1, launcher_id_taker: 1},
         solver=Solver(
@@ -117,7 +121,7 @@ async def test_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
                 }
             }
         ),
-        fee=uint64(2000000000000),
+        fee=fee,
     )
     assert error is None
     assert success is True
@@ -138,7 +142,7 @@ async def test_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
         ]
     }
 
-    success, offer_taker, error = await trade_manager_taker.respond_to_offer(
+    offer_taker, tx_records = await trade_manager_taker.respond_to_offer(
         Offer.from_bytes(offer_maker.offer),
         peer,
         solver=Solver(
@@ -166,11 +170,10 @@ async def test_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
                 ],
             }
         ),
-        fee=uint64(2000000000000),
+        fee=fee,
     )
-    assert error is None
-    assert success is True
     assert offer_taker is not None
+    assert tx_records is not None
 
     assert await trade_manager_maker.get_offer_summary(Offer.from_bytes(offer_taker.offer)) == {
         "offered": [
@@ -197,17 +200,17 @@ async def test_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
         ]
     }
 
-    await time_out_assert(15, wallet_maker.get_unconfirmed_balance, funds - 2000000000000)
-    await time_out_assert(15, wallet_taker.get_unconfirmed_balance, funds - 4000000000000)
+    await time_out_assert(15, wallet_maker.get_unconfirmed_balance, maker_funds)
+    await time_out_assert(15, wallet_taker.get_unconfirmed_balance, taker_funds - fee)
 
-    # Let's hack a way to await this offer's confirmation
-    offer_record = dataclasses.replace(dl_record, spend_bundle=Offer.from_bytes(offer_taker.offer).bundle)
-    await full_node_api.process_transaction_records(records=[offer_record])
+    await full_node_api.process_transaction_records(records=tx_records)
+    maker_funds -= fee
+    taker_funds -= fee
 
-    await time_out_assert(15, wallet_maker.get_confirmed_balance, funds - 4000000000000)
-    await time_out_assert(15, wallet_maker.get_unconfirmed_balance, funds - 4000000000000)
-    await time_out_assert(15, wallet_taker.get_confirmed_balance, funds - 4000000000000)
-    await time_out_assert(15, wallet_taker.get_unconfirmed_balance, funds - 4000000000000)
+    await time_out_assert(15, wallet_maker.get_confirmed_balance, maker_funds)
+    await time_out_assert(15, wallet_maker.get_unconfirmed_balance, maker_funds)
+    await time_out_assert(15, wallet_taker.get_confirmed_balance, taker_funds)
+    await time_out_assert(15, wallet_taker.get_unconfirmed_balance, taker_funds)
 
     await time_out_assert(15, is_singleton_confirmed_and_root, True, dl_wallet_maker, launcher_id_taker, taker_root)
     await time_out_assert(15, is_singleton_confirmed_and_root, True, dl_wallet_taker, launcher_id_maker, maker_root)
@@ -235,16 +238,12 @@ async def test_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
 )
 @pytest.mark.asyncio
 async def test_dl_offer_cancellation(wallets_prefarm: Any, trusted: bool) -> None:
-    wallet_node, _, full_node_api = wallets_prefarm
+    [wallet_node, _], [_, _], full_node_api = wallets_prefarm
     assert wallet_node.wallet_state_manager is not None
     wsm = wallet_node.wallet_state_manager
 
     wallet = wsm.main_wallet
 
-    funds = 20000000000000
-
-    await time_out_assert(10, wallet.get_unconfirmed_balance, funds)
-
     async with wsm.lock:
         dl_wallet = await DataLayerWallet.create_new_dl_wallet(wsm, wallet)
 
@@ -283,13 +282,13 @@ async def test_dl_offer_cancellation(wallets_prefarm: Any, trusted: bool) -> Non
                 }
             }
         ),
-        fee=uint64(2000000000000),
+        fee=uint64(2_000_000_000_000),
     )
     assert error is None
     assert success is True
     assert offer is not None
 
-    cancellation_txs = await trade_manager.cancel_pending_offer_safely(offer.trade_id, fee=uint64(2000000000000))
+    cancellation_txs = await trade_manager.cancel_pending_offer_safely(offer.trade_id, fee=uint64(2_000_000_000_000))
     assert len(cancellation_txs) == 3
     await time_out_assert(15, get_trade_and_status, TradeStatus.PENDING_CANCEL, trade_manager, offer)
     await full_node_api.process_transaction_records(records=cancellation_txs)
@@ -302,7 +301,11 @@ async def test_dl_offer_cancellation(wallets_prefarm: Any, trusted: bool) -> Non
 )
 @pytest.mark.asyncio
 async def test_multiple_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
-    wallet_node_maker, wallet_node_taker, full_node_api = wallets_prefarm
+    (
+        [wallet_node_maker, maker_funds],
+        [wallet_node_taker, taker_funds],
+        full_node_api,
+    ) = wallets_prefarm
     assert wallet_node_maker.wallet_state_manager is not None
     assert wallet_node_taker.wallet_state_manager is not None
     wsm_maker = wallet_node_maker.wallet_state_manager
@@ -311,11 +314,6 @@ async def test_multiple_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
     wallet_maker = wsm_maker.main_wallet
     wallet_taker = wsm_taker.main_wallet
 
-    funds = 20000000000000
-
-    await time_out_assert(10, wallet_maker.get_unconfirmed_balance, funds)
-    await time_out_assert(10, wallet_taker.get_confirmed_balance, funds)
-
     async with wsm_maker.lock:
         dl_wallet_maker = await DataLayerWallet.create_new_dl_wallet(wsm_maker, wallet_maker)
     async with wsm_taker.lock:
@@ -326,38 +324,40 @@ async def test_multiple_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
     maker_root, _ = build_merkle_tree(MAKER_ROWS)
     taker_root, _ = build_merkle_tree(TAKER_ROWS)
 
-    dl_record, std_record, launcher_id_maker_1 = await dl_wallet_maker.generate_new_reporter(
-        maker_root, fee=uint64(1999999999999)
-    )
+    fee = uint64(1_999_999_999_999)
+
+    dl_record, std_record, launcher_id_maker_1 = await dl_wallet_maker.generate_new_reporter(maker_root, fee=fee)
     assert await dl_wallet_maker.get_latest_singleton(launcher_id_maker_1) is not None
     await wsm_maker.add_pending_transaction(dl_record)
     await wsm_maker.add_pending_transaction(std_record)
     await full_node_api.process_transaction_records(records=[dl_record, std_record])
+    maker_funds -= fee
+    maker_funds -= 1
     await time_out_assert(15, is_singleton_confirmed_and_root, True, dl_wallet_maker, launcher_id_maker_1, maker_root)
-    dl_record, std_record, launcher_id_maker_2 = await dl_wallet_maker.generate_new_reporter(
-        maker_root, fee=uint64(1999999999999)
-    )
+    dl_record, std_record, launcher_id_maker_2 = await dl_wallet_maker.generate_new_reporter(maker_root, fee=fee)
     assert await dl_wallet_maker.get_latest_singleton(launcher_id_maker_2) is not None
     await wsm_maker.add_pending_transaction(dl_record)
     await wsm_maker.add_pending_transaction(std_record)
     await full_node_api.process_transaction_records(records=[dl_record, std_record])
+    maker_funds -= fee
+    maker_funds -= 1
     await time_out_assert(15, is_singleton_confirmed_and_root, True, dl_wallet_maker, launcher_id_maker_2, maker_root)
 
-    dl_record, std_record, launcher_id_taker_1 = await dl_wallet_taker.generate_new_reporter(
-        taker_root, fee=uint64(1999999999999)
-    )
+    dl_record, std_record, launcher_id_taker_1 = await dl_wallet_taker.generate_new_reporter(taker_root, fee=fee)
     assert await dl_wallet_taker.get_latest_singleton(launcher_id_taker_1) is not None
     await wsm_taker.add_pending_transaction(dl_record)
     await wsm_taker.add_pending_transaction(std_record)
     await full_node_api.process_transaction_records(records=[dl_record, std_record])
+    taker_funds -= fee
+    taker_funds -= 1
     await time_out_assert(15, is_singleton_confirmed_and_root, True, dl_wallet_taker, launcher_id_taker_1, taker_root)
-    dl_record, std_record, launcher_id_taker_2 = await dl_wallet_taker.generate_new_reporter(
-        taker_root, fee=uint64(1999999999999)
-    )
+    dl_record, std_record, launcher_id_taker_2 = await dl_wallet_taker.generate_new_reporter(taker_root, fee=fee)
     assert await dl_wallet_taker.get_latest_singleton(launcher_id_taker_2) is not None
     await wsm_taker.add_pending_transaction(dl_record)
     await wsm_taker.add_pending_transaction(std_record)
     await full_node_api.process_transaction_records(records=[dl_record, std_record])
+    taker_funds -= fee
+    taker_funds -= 1
     await time_out_assert(15, is_singleton_confirmed_and_root, True, dl_wallet_taker, launcher_id_taker_2, taker_root)
 
     peer = wallet_node_taker.get_full_node_peer()
@@ -383,6 +383,8 @@ async def test_multiple_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
     maker_branch, maker_branch_proof = get_parent_branch(maker_addition, maker_proofs[maker_addition])
     taker_branch, taker_branch_proof = get_parent_branch(taker_addition, taker_proofs[taker_addition])
 
+    fee = uint64(2_000_000_000_000)
+
     success, offer_maker, error = await trade_manager_maker.create_offer_for_ids(
         {launcher_id_maker_1: -1, launcher_id_taker_1: 1, launcher_id_maker_2: -1, launcher_id_taker_2: 1},
         solver=Solver(
@@ -411,13 +413,13 @@ async def test_multiple_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
                 },
             }
         ),
-        fee=uint64(2000000000000),
+        fee=fee,
     )
     assert error is None
     assert success is True
     assert offer_maker is not None
 
-    success, offer_taker, error = await trade_manager_taker.respond_to_offer(
+    offer_taker, tx_records = await trade_manager_taker.respond_to_offer(
         Offer.from_bytes(offer_maker.offer),
         peer,
         solver=Solver(
@@ -458,23 +460,23 @@ async def test_multiple_dl_offers(wallets_prefarm: Any, trusted: bool) -> None:
                 ],
             }
         ),
-        fee=uint64(2000000000000),
+        fee=fee,
     )
-    assert error is None
-    assert success is True
     assert offer_taker is not None
+    assert tx_records is not None
 
-    await time_out_assert(15, wallet_maker.get_unconfirmed_balance, funds - 4000000000000)
-    await time_out_assert(15, wallet_taker.get_unconfirmed_balance, funds - 6000000000000)
+    await time_out_assert(15, wallet_maker.get_unconfirmed_balance, maker_funds)
+    await time_out_assert(15, wallet_taker.get_unconfirmed_balance, taker_funds - fee)
 
-    # Let's hack a way to await this offer's confirmation
-    offer_record = dataclasses.replace(dl_record, spend_bundle=Offer.from_bytes(offer_taker.offer).bundle)
-    await full_node_api.process_transaction_records(records=[offer_record])
+    await full_node_api.process_transaction_records(records=tx_records)
 
-    await time_out_assert(15, wallet_maker.get_confirmed_balance, funds - 6000000000000)
-    await time_out_assert(15, wallet_maker.get_unconfirmed_balance, funds - 6000000000000)
-    await time_out_assert(15, wallet_taker.get_confirmed_balance, funds - 6000000000000)
-    await time_out_assert(15, wallet_taker.get_unconfirmed_balance, funds - 6000000000000)
+    maker_funds -= fee
+    taker_funds -= fee
+
+    await time_out_assert(15, wallet_maker.get_confirmed_balance, maker_funds)
+    await time_out_assert(15, wallet_maker.get_unconfirmed_balance, maker_funds)
+    await time_out_assert(15, wallet_taker.get_confirmed_balance, taker_funds)
+    await time_out_assert(15, wallet_taker.get_unconfirmed_balance, taker_funds)
 
     await time_out_assert(15, is_singleton_confirmed_and_root, True, dl_wallet_maker, launcher_id_taker_1, taker_root)
     await time_out_assert(15, is_singleton_confirmed_and_root, True, dl_wallet_maker, launcher_id_taker_2, taker_root)
