@@ -1,10 +1,11 @@
+from __future__ import annotations
+
 from secrets import token_bytes
 from typing import Any, Dict, Optional
 
 import pytest
 
 from chinilla.consensus.block_rewards import calculate_base_farmer_reward, calculate_pool_reward
-from chinilla.full_node.mempool_manager import MempoolManager
 from chinilla.simulator.full_node_simulator import FullNodeSimulator
 from chinilla.simulator.simulator_protocol import FarmNewBlockProtocol
 from chinilla.simulator.time_out_assert import time_out_assert, time_out_assert_not_none
@@ -18,17 +19,11 @@ from chinilla.wallet.outer_puzzles import create_asset_id, match_puzzle
 from chinilla.wallet.puzzle_drivers import PuzzleInfo
 from chinilla.wallet.trading.offer import Offer
 from chinilla.wallet.trading.trade_status import TradeStatus
+from chinilla.wallet.uncurried_puzzle import uncurry_puzzle
 from tests.util.wallet_is_synced import wallets_are_synced
 from tests.wallet.nft_wallet.test_nft_1_offers import mempool_not_empty
 
 
-async def tx_in_pool(mempool: MempoolManager, tx_id: bytes32) -> bool:
-    tx = mempool.get_spendbundle(tx_id)
-    if tx is None:
-        return False
-    return True
-
-
 async def get_trade_and_status(trade_manager, trade) -> TradeStatus:  # type: ignore
     trade_rec = await trade_manager.get_trade_by_id(trade.trade_id)
     return TradeStatus(trade_rec.status)
@@ -104,9 +99,9 @@ async def test_nft_offer_with_fee(two_wallet_nodes: Any, trusted: Any) -> None:
     await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(token_ph))
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_0, wallet_node_1], full_node_api)
 
-    coins_maker = nft_wallet_maker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
     assert len(coins_maker) == 1
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_taker) == 0
 
     # MAKE FIRST TRADE: 1 NFT for 100 hcx
@@ -114,7 +109,7 @@ async def test_nft_offer_with_fee(two_wallet_nodes: Any, trusted: Any) -> None:
     taker_balance_pre = await wallet_taker.get_confirmed_balance()
 
     nft_to_offer = coins_maker[0]
-    nft_info: Optional[PuzzleInfo] = match_puzzle(nft_to_offer.full_puzzle)
+    nft_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_offer.full_puzzle))
     nft_asset_id: bytes32 = create_asset_id(nft_info)  # type: ignore
     driver_dict: Dict[bytes32, Optional[PuzzleInfo]] = {nft_asset_id: nft_info}
 
@@ -133,18 +128,14 @@ async def test_nft_offer_with_fee(two_wallet_nodes: Any, trusted: Any) -> None:
 
     peer = wallet_node_1.get_full_node_peer()
     assert peer is not None
-    success, trade_take, error = await trade_manager_taker.respond_to_offer(
+    trade_take, tx_records = await trade_manager_taker.respond_to_offer(
         Offer.from_bytes(trade_make.offer), peer, fee=taker_fee
     )
 
-    sb_id = Offer.from_bytes(trade_take.offer).to_valid_spend().name()
-    await time_out_assert_not_none(20, full_node_api.full_node.mempool_manager.get_spendbundle, sb_id)
-
-    assert success
-    assert error is None
     assert trade_take is not None
+    assert tx_records is not None
 
-    await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(token_ph))
+    await full_node_api.process_transaction_records(records=tx_records)
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_0, wallet_node_1], full_node_api)
 
     await time_out_assert(20, get_trade_and_status, TradeStatus.CONFIRMED, trade_manager_maker, trade_make)
@@ -152,8 +143,8 @@ async def test_nft_offer_with_fee(two_wallet_nodes: Any, trusted: Any) -> None:
     await time_out_assert(20, wallet_maker.get_confirmed_balance, maker_balance_pre + hcx_request - maker_fee)
     await time_out_assert(20, wallet_taker.get_confirmed_balance, taker_balance_pre - hcx_request - taker_fee)
 
-    coins_maker = nft_wallet_maker.my_nft_coins
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_maker) == 0
     assert len(coins_taker) == 1
 
@@ -163,7 +154,7 @@ async def test_nft_offer_with_fee(two_wallet_nodes: Any, trusted: Any) -> None:
     taker_balance_pre = await wallet_taker.get_confirmed_balance()
 
     nft_to_buy = coins_taker[0]
-    nft_to_buy_info: Optional[PuzzleInfo] = match_puzzle(nft_to_buy.full_puzzle)
+    nft_to_buy_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_buy.full_puzzle))
     nft_to_buy_asset_id: bytes32 = create_asset_id(nft_to_buy_info)  # type: ignore
     driver_dict_to_buy: Dict[bytes32, Optional[PuzzleInfo]] = {nft_to_buy_asset_id: nft_to_buy_info}
 
@@ -179,18 +170,14 @@ async def test_nft_offer_with_fee(two_wallet_nodes: Any, trusted: Any) -> None:
 
     taker_fee = uint64(1)
 
-    success, trade_take, error = await trade_manager_taker.respond_to_offer(
+    trade_take, tx_records = await trade_manager_taker.respond_to_offer(
         Offer.from_bytes(trade_make.offer), peer, fee=taker_fee
     )
 
-    sb_id = Offer.from_bytes(trade_take.offer).to_valid_spend().name()
-    await time_out_assert_not_none(20, full_node_api.full_node.mempool_manager.get_spendbundle, sb_id)
-
-    assert success
-    assert error is None
     assert trade_take is not None
+    assert tx_records is not None
 
-    await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(token_ph))
+    await full_node_api.process_transaction_records(records=tx_records)
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_0, wallet_node_1], full_node_api)
 
     await time_out_assert(20, get_trade_and_status, TradeStatus.CONFIRMED, trade_manager_maker, trade_make)
@@ -198,8 +185,8 @@ async def test_nft_offer_with_fee(two_wallet_nodes: Any, trusted: Any) -> None:
     await time_out_assert(20, wallet_maker.get_confirmed_balance, maker_balance_pre - hcx_offered - maker_fee)
     await time_out_assert(20, wallet_taker.get_confirmed_balance, taker_balance_pre + hcx_offered - taker_fee)
 
-    coins_maker = nft_wallet_maker.my_nft_coins
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_maker) == 1
     assert len(coins_taker) == 0
 
@@ -273,9 +260,9 @@ async def test_nft_offer_cancellations(two_wallet_nodes: Any, trusted: Any) -> N
     await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(token_ph))
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_0, wallet_node_1], full_node_api)
 
-    coins_maker = nft_wallet_maker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
     assert len(coins_maker) == 1
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_taker) == 0
 
     # maker creates offer and cancels
@@ -283,7 +270,7 @@ async def test_nft_offer_cancellations(two_wallet_nodes: Any, trusted: Any) -> N
     # taker_balance_pre = await wallet_taker.get_confirmed_balance()
 
     nft_to_offer = coins_maker[0]
-    nft_info: Optional[PuzzleInfo] = match_puzzle(nft_to_offer.full_puzzle)
+    nft_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_offer.full_puzzle))
     nft_asset_id: bytes32 = create_asset_id(nft_info)  # type: ignore
     driver_dict: Dict[bytes32, Optional[PuzzleInfo]] = {nft_asset_id: nft_info}
 
@@ -306,18 +293,14 @@ async def test_nft_offer_cancellations(two_wallet_nodes: Any, trusted: Any) -> N
     txs = await trade_manager_maker.cancel_pending_offer_safely(trade_make.trade_id, fee=cancel_fee)
 
     await time_out_assert(20, get_trade_and_status, TradeStatus.PENDING_CANCEL, trade_manager_maker, trade_make)
-    for tx in txs:
-        if tx.spend_bundle is not None:
-            await time_out_assert(20, tx_in_pool, True, full_node_api.full_node.mempool_manager, tx.spend_bundle.name())
-
-    await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(token_ph))
+    await full_node_api.process_transaction_records(records=txs)
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_0, wallet_node_1], full_node_api)
 
     await time_out_assert(20, get_trade_and_status, TradeStatus.CANCELLED, trade_manager_maker, trade_make)
 
     maker_balance = await wallet_maker.get_confirmed_balance()
     assert maker_balance == maker_balance_pre - cancel_fee
-    coins_maker = nft_wallet_maker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
     assert len(coins_maker) == 1
 
 
@@ -394,9 +377,9 @@ async def test_nft_offer_with_metadata_update(two_wallet_nodes: Any, trusted: An
     await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(token_ph))
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_0, wallet_node_1], full_node_api)
 
-    coins_maker = nft_wallet_maker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
     assert len(coins_maker) == 1
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_taker) == 0
 
     # Maker updates metadata:
@@ -411,9 +394,9 @@ async def test_nft_offer_with_metadata_update(two_wallet_nodes: Any, trusted: An
     await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(token_ph))
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_0, wallet_node_1], full_node_api)
 
-    coins_maker = nft_wallet_maker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
     updated_nft = coins_maker[0]
-    updated_nft_info = match_puzzle(updated_nft.full_puzzle)
+    updated_nft_info = match_puzzle(uncurry_puzzle(updated_nft.full_puzzle))
 
     assert url_to_add in updated_nft_info.also().info["metadata"]  # type: ignore
 
@@ -422,7 +405,7 @@ async def test_nft_offer_with_metadata_update(two_wallet_nodes: Any, trusted: An
     taker_balance_pre = await wallet_taker.get_confirmed_balance()
 
     nft_to_offer = coins_maker[0]
-    nft_info: Optional[PuzzleInfo] = match_puzzle(nft_to_offer.full_puzzle)
+    nft_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_offer.full_puzzle))
     nft_asset_id: bytes32 = create_asset_id(nft_info)  # type: ignore
     driver_dict: Dict[bytes32, Optional[PuzzleInfo]] = {nft_asset_id: nft_info}
 
@@ -441,16 +424,14 @@ async def test_nft_offer_with_metadata_update(two_wallet_nodes: Any, trusted: An
 
     peer = wallet_node_1.get_full_node_peer()
     assert peer is not None
-    success, trade_take, error = await trade_manager_taker.respond_to_offer(
+    trade_take, tx_records = await trade_manager_taker.respond_to_offer(
         Offer.from_bytes(trade_make.offer), peer, fee=taker_fee
     )
-    await time_out_assert(20, mempool_not_empty, True, full_node_api)
 
-    assert success
-    assert error is None
     assert trade_take is not None
+    assert tx_records is not None
 
-    await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(token_ph))
+    await full_node_api.process_transaction_records(records=tx_records)
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_0, wallet_node_1], full_node_api)
 
     await time_out_assert(20, get_trade_and_status, TradeStatus.CONFIRMED, trade_manager_maker, trade_make)
@@ -458,8 +439,8 @@ async def test_nft_offer_with_metadata_update(two_wallet_nodes: Any, trusted: An
     await time_out_assert(20, wallet_maker.get_confirmed_balance, maker_balance_pre + hcx_request - maker_fee)
     await time_out_assert(20, wallet_taker.get_confirmed_balance, taker_balance_pre - hcx_request - taker_fee)
 
-    coins_maker = nft_wallet_maker.my_nft_coins
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_maker) == 0
     assert len(coins_taker) == 1
 
@@ -534,9 +515,9 @@ async def test_nft_offer_nft_for_cat(two_wallet_nodes: Any, trusted: Any) -> Non
     await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(token_ph))
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_0, wallet_node_1], full_node_api)
 
-    coins_maker = nft_wallet_maker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
     assert len(coins_maker) == 1
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_taker) == 0
 
     # Create two new CATs and wallets for maker and taker
@@ -579,7 +560,7 @@ async def test_nft_offer_nft_for_cat(two_wallet_nodes: Any, trusted: Any) -> Non
     taker_cat_taker_balance_pre = await cat_wallet_taker.get_confirmed_balance()
 
     nft_to_offer = coins_maker[0]
-    nft_info: Optional[PuzzleInfo] = match_puzzle(nft_to_offer.full_puzzle)
+    nft_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_offer.full_puzzle))
     nft_asset_id: bytes32 = create_asset_id(nft_info)  # type: ignore
     driver_dict: Dict[bytes32, Optional[PuzzleInfo]] = {nft_asset_id: nft_info}
 
@@ -598,16 +579,14 @@ async def test_nft_offer_nft_for_cat(two_wallet_nodes: Any, trusted: Any) -> Non
 
     peer = wallet_node_1.get_full_node_peer()
     assert peer is not None
-    success, trade_take, error = await trade_manager_taker.respond_to_offer(
+    trade_take, tx_records = await trade_manager_taker.respond_to_offer(
         Offer.from_bytes(trade_make.offer), peer, fee=taker_fee
     )
-    await time_out_assert(20, mempool_not_empty, True, full_node_api)
 
-    assert success
-    assert error is None
     assert trade_take is not None
+    assert tx_records is not None
 
-    await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(token_ph))
+    await full_node_api.process_transaction_records(records=tx_records)
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_0, wallet_node_1], full_node_api)
 
     await time_out_assert(20, get_trade_and_status, TradeStatus.CONFIRMED, trade_manager_maker, trade_make)
@@ -621,8 +600,8 @@ async def test_nft_offer_nft_for_cat(two_wallet_nodes: Any, trusted: Any) -> Non
     taker_balance_post = await wallet_taker.get_confirmed_balance()
     assert maker_balance_post == maker_balance_pre - maker_fee
     assert taker_balance_post == taker_balance_pre - taker_fee
-    coins_maker = nft_wallet_maker.my_nft_coins
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_maker) == 0
     assert len(coins_taker) == 1
 
@@ -631,7 +610,7 @@ async def test_nft_offer_nft_for_cat(two_wallet_nodes: Any, trusted: Any) -> Non
     taker_cat_amount = 500
 
     nft_to_buy = coins_taker[0]
-    nft_to_buy_info: Optional[PuzzleInfo] = match_puzzle(nft_to_buy.full_puzzle)
+    nft_to_buy_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_buy.full_puzzle))
     nft_to_buy_asset_id: bytes32 = create_asset_id(nft_to_buy_info)  # type: ignore
 
     driver_dict_to_buy: Dict[bytes32, Optional[PuzzleInfo]] = {
@@ -654,17 +633,15 @@ async def test_nft_offer_nft_for_cat(two_wallet_nodes: Any, trusted: Any) -> Non
 
     taker_fee = uint64(1)
 
-    success, trade_take, error = await trade_manager_taker.respond_to_offer(
+    trade_take, tx_records = await trade_manager_taker.respond_to_offer(
         Offer.from_bytes(trade_make.offer), peer, fee=taker_fee
     )
-    await time_out_assert(20, mempool_not_empty, True, full_node_api)
 
-    assert success
-    assert error is None
     assert trade_take is not None
+    assert tx_records is not None
 
+    await full_node_api.process_transaction_records(records=tx_records)
     # check balances: taker wallet down an NFT, up cats
-    await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(token_ph))
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_0, wallet_node_1], full_node_api)
 
     await time_out_assert(20, get_trade_and_status, TradeStatus.CONFIRMED, trade_manager_maker, trade_make)
@@ -678,8 +655,8 @@ async def test_nft_offer_nft_for_cat(two_wallet_nodes: Any, trusted: Any) -> Non
     taker_balance_post_2 = await wallet_taker.get_confirmed_balance()
     assert maker_balance_post_2 == maker_balance_post - maker_fee
     assert taker_balance_post_2 == taker_balance_post - taker_fee
-    coins_maker = nft_wallet_maker.my_nft_coins
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_maker) == 1
     assert len(coins_taker) == 0
 
@@ -764,20 +741,20 @@ async def test_nft_offer_nft_for_nft(two_wallet_nodes: Any, trusted: Any) -> Non
     await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(token_ph))
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_0, wallet_node_1], full_node_api)
 
-    coins_maker = nft_wallet_maker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
     assert len(coins_maker) == 1
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_taker) == 1
 
     maker_balance_pre = await wallet_maker.get_confirmed_balance()
     taker_balance_pre = await wallet_taker.get_confirmed_balance()
 
     nft_to_offer = coins_maker[0]
-    nft_to_offer_info: Optional[PuzzleInfo] = match_puzzle(nft_to_offer.full_puzzle)
+    nft_to_offer_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_offer.full_puzzle))
     nft_to_offer_asset_id: bytes32 = create_asset_id(nft_to_offer_info)  # type: ignore
 
     nft_to_take = coins_taker[0]
-    nft_to_take_info: Optional[PuzzleInfo] = match_puzzle(nft_to_take.full_puzzle)
+    nft_to_take_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_take.full_puzzle))
     nft_to_take_asset_id: bytes32 = create_asset_id(nft_to_take_info)  # type: ignore
 
     driver_dict: Dict[bytes32, Optional[PuzzleInfo]] = {
@@ -799,16 +776,14 @@ async def test_nft_offer_nft_for_nft(two_wallet_nodes: Any, trusted: Any) -> Non
 
     peer = wallet_node_1.get_full_node_peer()
     assert peer is not None
-    success, trade_take, error = await trade_manager_taker.respond_to_offer(
+    trade_take, tx_records = await trade_manager_taker.respond_to_offer(
         Offer.from_bytes(trade_make.offer), peer, fee=taker_fee
     )
-    await time_out_assert(20, mempool_not_empty, True, full_node_api)
 
-    assert success
-    assert error is None
     assert trade_take is not None
+    assert tx_records is not None
 
-    await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(token_ph))
+    await full_node_api.process_transaction_records(records=tx_records)
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_0, wallet_node_1], full_node_api)
 
     await time_out_assert(20, get_trade_and_status, TradeStatus.CONFIRMED, trade_manager_maker, trade_make)
@@ -816,7 +791,7 @@ async def test_nft_offer_nft_for_nft(two_wallet_nodes: Any, trusted: Any) -> Non
     await time_out_assert(20, wallet_maker.get_confirmed_balance, maker_balance_pre - maker_fee)
     await time_out_assert(20, wallet_taker.get_confirmed_balance, taker_balance_pre - taker_fee)
 
-    coins_maker = nft_wallet_maker.my_nft_coins
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_maker) == 1
     assert len(coins_taker) == 1
