@@ -5,11 +5,10 @@ import time
 from typing import Any, Awaitable, Callable, Dict, List
 
 import pytest
-from chinillablspy import AugSchemeMPL, G1Element, G2Element
+from blspy import AugSchemeMPL, G1Element, G2Element
 from clvm_tools.binutils import disassemble
 
 from chinilla.consensus.block_rewards import calculate_base_farmer_reward, calculate_pool_reward
-from chinilla.full_node.mempool_manager import MempoolManager
 from chinilla.rpc.wallet_rpc_api import WalletRpcApi
 from chinilla.simulator.full_node_simulator import FullNodeSimulator
 from chinilla.simulator.simulator_protocol import FarmNewBlockProtocol, ReorgProtocol
@@ -18,7 +17,7 @@ from chinilla.types.blockchain_format.program import Program
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.types.peer_info import PeerInfo
 from chinilla.types.spend_bundle import SpendBundle
-from chinilla.util.bech32m import encode_puzzle_hash
+from chinilla.util.bech32m import decode_puzzle_hash, encode_puzzle_hash
 from chinilla.util.byte_types import hexstr_to_bytes
 from chinilla.util.ints import uint16, uint32, uint64
 from chinilla.wallet.did_wallet.did_wallet import DIDWallet
@@ -30,13 +29,6 @@ from chinilla.wallet.wallet_state_manager import WalletStateManager
 from tests.util.wallet_is_synced import wallet_is_synced
 
 
-async def tx_in_pool(mempool: MempoolManager, tx_id: bytes32) -> bool:
-    tx = mempool.get_spendbundle(tx_id)
-    if tx is None:
-        return False
-    return True
-
-
 async def get_nft_count(wallet: NFTWallet) -> int:
     return await wallet.get_nft_count()
 
@@ -253,7 +245,7 @@ async def test_nft_wallet_creation_and_transfer(two_wallet_nodes: Any, trusted:
         assert False
     await full_node_api.reorg_from_index_to_new_index(ReorgProtocol(uint32(height - 1), uint32(height + 1), ph1, None))
     await time_out_assert(30, get_nft_count, 0, nft_wallet_0)
-    await time_out_assert(30, get_wallet_number, 1, wallet_node_0.wallet_state_manager)
+    await time_out_assert(30, get_wallet_number, 2, wallet_node_0.wallet_state_manager)
 
     nft_wallet_0 = await NFTWallet.create_new_nft_wallet(
         wallet_node_0.wallet_state_manager, wallet_0, name="NFT WALLET 1"
@@ -854,6 +846,7 @@ async def test_nft_rpc_mint(two_wallet_nodes: Any, trusted: Any) -> None:
         }
     )
     assert resp.get("success")
+    nft_id: str = str(resp.get("nft_id"))
     sb = resp["spend_bundle"]
 
     # ensure hints are generated
@@ -879,6 +872,7 @@ async def test_nft_rpc_mint(two_wallet_nodes: Any, trusted: Any) -> None:
     assert did_nft.edition_total == st
     assert did_nft.edition_number == sn
     assert did_nft.royalty_percentage == royalty_percentage
+    assert decode_puzzle_hash(nft_id) == did_nft.launcher_id
 
 
 @pytest.mark.parametrize(
@@ -925,6 +919,7 @@ async def test_nft_transfer_nft_with_did(two_wallet_nodes: Any, trusted: Any) ->
     )
     await time_out_assert(30, wallet_0.get_unconfirmed_balance, funds)
     await time_out_assert(30, wallet_0.get_confirmed_balance, funds)
+    # Create DID
     did_wallet: DIDWallet = await DIDWallet.create_new_did_wallet(
         wallet_node_0.wallet_state_manager, wallet_0, uint64(1)
     )
@@ -939,6 +934,7 @@ async def test_nft_transfer_nft_with_did(two_wallet_nodes: Any, trusted: Any) ->
     hmr_did_id = encode_puzzle_hash(bytes32.from_hexstr(hex_did_id), AddressType.DID.hrp(wallet_node_0.config))
 
     await time_out_assert(30, wallet_is_synced, True, wallet_node_0, full_node_api)
+    # Create NFT wallet
     res = await api_0.create_new_wallet(dict(wallet_type="nft_wallet", name="NFT WALLET 1", did_id=hmr_did_id))
     assert isinstance(res, dict)
     assert res.get("success")
@@ -970,6 +966,15 @@ async def test_nft_transfer_nft_with_did(two_wallet_nodes: Any, trusted: Any) ->
     assert coins[0].owner_did.hex() == hex_did_id
 
     assert len(wallet_1.wallet_state_manager.wallets) == 1, "NFT wallet shouldn't exist yet"
+    assert len(wallet_0.wallet_state_manager.wallets) == 3
+    # transfer DID to the other wallet
+    tx = await did_wallet.transfer_did(ph1, uint64(0), True)
+    assert tx
+    for i in range(1, num_blocks):
+        await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph1))
+
+    await time_out_assert(15, len, 2, wallet_0.wallet_state_manager.wallets)
+    # Transfer NFT, wallet will be deleted
     resp = await api_0.nft_transfer_nft(
         dict(
             wallet_id=nft_wallet_0_id,
@@ -978,23 +983,18 @@ async def test_nft_transfer_nft_with_did(two_wallet_nodes: Any, trusted: Any) ->
             fee=fee,
         )
     )
-    # transfer DID to the other wallet
-    tx = await did_wallet.transfer_did(ph1, uint64(0), True)
-    assert tx
     assert resp.get("success")
     sb = resp["spend_bundle"]
     assert compute_memos(sb)
     await time_out_assert_not_none(30, full_node_api.full_node.mempool_manager.get_spendbundle, sb.name())
-
-    for i in range(1, num_blocks):
-        await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph1))
-
-    await time_out_assert(5, len, 2, wallet_0.wallet_state_manager.wallets)
+    await make_new_block_with(resp, full_node_api, ph1)
     await time_out_assert(30, wallet_0.get_unconfirmed_balance, 5999999999798)
     await time_out_assert(30, wallet_0.get_confirmed_balance, 5999999999798)
+    await time_out_assert(30, len, 1, wallet_0.wallet_state_manager.wallets)
+
     # wait for all wallets to be created
     await time_out_assert(30, len, 3, wallet_1.wallet_state_manager.wallets)
-    did_wallet_1 = wallet_1.wallet_state_manager.wallets[3]
+    did_wallet_1 = wallet_1.wallet_state_manager.wallets[2]
     assert nft_wallet_0_id not in wallet_node_0.wallet_state_manager.wallets.keys()
     # Check if the NFT owner DID is reset
     resp = await api_1.nft_get_by_did({})
@@ -1019,8 +1019,8 @@ async def test_nft_transfer_nft_with_did(two_wallet_nodes: Any, trusted: Any) ->
     coins_response = await wait_rpc_state_condition(
         5, api_1.nft_get_by_did, [dict(did_id=hmr_did_id)], lambda x: x.get("wallet_id", 0) > 0
     )
-    await time_out_assert(30, wallet_1.get_unconfirmed_balance, 10000000000100)
-    await time_out_assert(30, wallet_1.get_confirmed_balance, 10000000000100)
+    await time_out_assert(30, wallet_1.get_unconfirmed_balance, 12000000000100)
+    await time_out_assert(30, wallet_1.get_confirmed_balance, 12000000000100)
     nft_wallet_1_id = coins_response.get("wallet_id")
     assert nft_wallet_1_id
     # Check NFT DID is set now
@@ -1168,6 +1168,144 @@ async def test_update_metadata_for_nft_did(two_wallet_nodes: Any, trusted: Any)
     assert len(coin["license_uris"]) == 0
 
 
+@pytest.mark.parametrize(
+    "trusted",
+    [True, False],
+)
+@pytest.mark.asyncio
+async def test_nft_bulk_set_did(two_wallet_nodes: Any, trusted: Any) -> None:
+    num_blocks = 2
+    full_nodes, wallets, _ = two_wallet_nodes
+    full_node_api: FullNodeSimulator = full_nodes[0]
+    full_node_server = full_node_api.server
+    wallet_node_0, server_0 = wallets[0]
+    wallet_node_1, server_1 = wallets[1]
+    wallet_0 = wallet_node_0.wallet_state_manager.main_wallet
+    api_0 = WalletRpcApi(wallet_node_0)
+    ph = await wallet_0.get_new_puzzlehash()
+
+    if trusted:
+        wallet_node_0.config["trusted_peers"] = {
+            full_node_api.full_node.server.node_id.hex(): full_node_api.full_node.server.node_id.hex()
+        }
+        wallet_node_1.config["trusted_peers"] = {
+            full_node_api.full_node.server.node_id.hex(): full_node_api.full_node.server.node_id.hex()
+        }
+    else:
+        wallet_node_0.config["trusted_peers"] = {}
+        wallet_node_1.config["trusted_peers"] = {}
+
+    await server_0.start_client(PeerInfo("localhost", uint16(full_node_server._port)), None)
+    await server_1.start_client(PeerInfo("localhost", uint16(full_node_server._port)), None)
+
+    for _ in range(1, num_blocks + 1):
+        await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph))
+
+    funds = sum(
+        [calculate_pool_reward(uint32(i)) + calculate_base_farmer_reward(uint32(i)) for i in range(1, num_blocks)]
+    )
+
+    await time_out_assert(30, wallet_0.get_unconfirmed_balance, funds)
+    await time_out_assert(30, wallet_0.get_confirmed_balance, funds)
+    did_wallet: DIDWallet = await DIDWallet.create_new_did_wallet(
+        wallet_node_0.wallet_state_manager, wallet_0, uint64(1)
+    )
+    spend_bundle_list = await wallet_node_0.wallet_state_manager.tx_store.get_unconfirmed_for_wallet(did_wallet.id())
+    spend_bundle = spend_bundle_list[0].spend_bundle
+    await time_out_assert_not_none(30, full_node_api.full_node.mempool_manager.get_spendbundle, spend_bundle.name())
+
+    for _ in range(1, num_blocks):
+        await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph))
+    await time_out_assert(30, wallet_0.get_pending_change_balance, 0)
+    await time_out_assert(30, wallet_0.get_confirmed_balance, 3999999999999)
+    hex_did_id = did_wallet.get_my_DID()
+    hmr_did_id = encode_puzzle_hash(bytes32.from_hexstr(hex_did_id), AddressType.DID.hrp(wallet_node_0.config))
+
+    await time_out_assert(30, wallet_is_synced, True, wallet_node_0, full_node_api)
+    res = await api_0.create_new_wallet(dict(wallet_type="nft_wallet", name="NFT WALLET 1", did_id=hmr_did_id))
+    assert isinstance(res, dict)
+    assert res.get("success")
+    nft_wallet_0_id = res["wallet_id"]
+    res = await api_0.create_new_wallet(dict(wallet_type="nft_wallet", name="NFT WALLET 2"))
+    assert isinstance(res, dict)
+    assert res.get("success")
+    nft_wallet_1_id = res["wallet_id"]
+    await time_out_assert(30, did_wallet.get_confirmed_balance, 1)
+
+    # Create a NFT with DID
+    resp = await api_0.nft_mint_nft(
+        {
+            "wallet_id": nft_wallet_0_id,
+            "hash": "0xD4584AD463139FA8C0D9F68F4B59F185",
+            "uris": ["https://www.chinilla.com/img/branding/chinilla-logo.svg"],
+            "mu": ["https://www.chinilla.com/img/branding/chinilla-logo.svg"],
+            "did_id": hmr_did_id,
+        }
+    )
+    sb = await make_new_block_with(resp, full_node_api, ph)
+    # ensure hints are generated
+    assert compute_memos(sb)
+    await wait_rpc_state_condition(
+        30, api_0.nft_get_nfts, [{"wallet_id": nft_wallet_0_id}], lambda x: len(x["nft_list"]) > 0
+    )
+    resp = await api_0.nft_mint_nft(
+        {
+            "wallet_id": nft_wallet_1_id,
+            "hash": "0xD4584AD463139FA8C0D9F68F4B59F186",
+            "uris": ["https://www.chinilla.com/img/branding/chinilla-logo.svg"],
+            "mu": ["https://www.chinilla.com/img/branding/chinilla-logo.svg"],
+            "did_id": "",
+        }
+    )
+    sb = await make_new_block_with(resp, full_node_api, ph)
+    # ensure hints are generated
+    assert compute_memos(sb)
+
+    # Check DID NFT
+    coins_response = await wait_rpc_state_condition(
+        30, api_0.nft_get_nfts, [{"wallet_id": nft_wallet_0_id}], lambda x: len(x["nft_list"]) == 1
+    )
+    coins = coins_response["nft_list"]
+    nft1 = coins[0]
+    assert len(coins) == 1
+    assert coins[0].owner_did is not None
+    coins_response = await wait_rpc_state_condition(
+        30, api_0.nft_get_nfts, [{"wallet_id": nft_wallet_1_id}], lambda x: len(x["nft_list"]) == 1
+    )
+    coins = coins_response["nft_list"]
+    nft2 = coins[0]
+    assert len(coins) == 1
+    assert coins[0].owner_did is None
+    nft_coin_list = [
+        {"wallet_id": nft_wallet_0_id, "nft_coin_id": nft1.nft_coin_id.hex()},
+        {"wallet_id": nft_wallet_1_id, "nft_coin_id": nft2.nft_coin_id.hex()},
+    ]
+    resp = await api_0.nft_set_bulk_nft_did(dict(did_id=hmr_did_id, nft_coin_list=nft_coin_list))
+    coins_response = await wait_rpc_state_condition(
+        30, api_0.nft_get_nfts, [{"wallet_id": nft_wallet_0_id}], lambda x: len(x["nft_list"]) == 1
+    )
+    coins = coins_response["nft_list"]
+    assert coins[0].pending_transaction
+    await make_new_block_with(resp, full_node_api, ph)
+    coins_response = await wait_rpc_state_condition(
+        30, api_0.nft_get_by_did, [dict(did_id=hmr_did_id)], lambda x: x.get("wallet_id", 0) > 0
+    )
+    nft_wallet_1_id = coins_response.get("wallet_id")
+    assert nft_wallet_1_id
+    resp = await wait_rpc_state_condition(
+        30,
+        api_0.nft_get_nfts,
+        [dict(wallet_id=nft_wallet_1_id)],
+        lambda x: len(x["nft_list"]) > 1 and x["nft_list"][0].owner_did,
+    )
+    assert len(await wallet_node_0.wallet_state_manager.wallets[nft_wallet_0_id].get_current_nfts()) == 2
+
+    coins = resp["nft_list"]
+    assert len(coins) == 2
+    assert coins[0].owner_did.hex() == hex_did_id
+    assert coins[1].owner_did.hex() == hex_did_id
+
+
 @pytest.mark.parametrize(
     "trusted",
     [True, False],
@@ -1228,7 +1366,7 @@ async def test_nft_set_did(two_wallet_nodes: Any, trusted: Any) -> None:
 
     await time_out_assert(30, did_wallet.get_confirmed_balance, 1)
 
-    # Create a NFT with DID
+    # Create a NFT without DID
     resp = await api_0.nft_mint_nft(
         {
             "wallet_id": nft_wallet_0_id,
@@ -1303,7 +1441,7 @@ async def test_nft_set_did(two_wallet_nodes: Any, trusted: Any) -> None:
 
     nft_wallet_2_id = coins_response.get("wallet_id")
     assert nft_wallet_2_id
-    await time_out_assert(30, wallet_node_0.wallet_state_manager.wallets.get, 0, nft_wallet_1_id, 0)
+    await time_out_assert(30, len, 6, wallet_node_0.wallet_state_manager.wallets)
 
     # Check NFT DID
     resp = await wait_rpc_state_condition(
@@ -1328,7 +1466,7 @@ async def test_nft_set_did(two_wallet_nodes: Any, trusted: Any) -> None:
     coins = resp["nft_list"]
     assert len(coins) == 1
     assert coins[0].owner_did is None
-    assert nft_wallet_2_id not in wallet_node_0.wallet_state_manager.wallets.keys()
+    assert nft_wallet_2_id in wallet_node_0.wallet_state_manager.wallets.keys()
     nft_coin_id = coins[0].nft_coin_id
     resp = await api_0.nft_get_info(dict(coin_id=nft_coin_id.hex(), latest=True))
     assert resp["success"]
