@@ -5,12 +5,12 @@ from pathlib import Path
 from typing import Any, Dict, Optional
 
 import pytest
-from chinillablspy import PrivateKey
+from blspy import PrivateKey
 
+from chinilla.simulator.block_tools import test_constants
 from chinilla.util.config import load_config
 from chinilla.util.keychain import Keychain, generate_mnemonic
 from chinilla.wallet.wallet_node import WalletNode
-from tests.setup_nodes import test_constants
 
 
 @pytest.mark.asyncio
